//--------------------------------------------------------------------------
//
// Collection of routines which support the WSUB LS Protocol for 
// HID and HID like devices
//
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/Radio/protocol.c $
// $Modtime: 3/04/04 2:46p $
// $Revision: 1.2 $
//--------------------------------------------------------------------------
//
// Copyright 2003-2004, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

#include "oneway_protocol.h"
#include "bind.h"
#include "radio.h"

UINT8  data_toggle      = 0x55;  
LS_TX_PACKET tx_packet;
UINT8 protocol_device_type = DEVICE_TYPE;

LS_TX_PACKET tx_packet;
extern SYS_PARAMETERS sys_params;

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_init
//
//  Description: Initializes the LS Radio and bind protocol
//
//  Inputs:      Void
//
//  Returns:     Void
//
///////////////////////////////////////////////////////////////////////////////

Status protocol_init()
{
    tx_packet.first.byte = 0x50;    // Data packet    
    protocol_device_type = DEVICE_TYPE;

    // get nvram
    NVRAM_READ( &sys_params, sizeof(sys_params) );

    // check for uninitialized parameters
    if (sys_params.network_id.signature != SIGNATURE_BYTE) 
    {
        // set channel and pn_code to a value that will allow the different
        // devices to coexist and interfere less with each other
        sys_params.network_id.signature   = SIGNATURE_BYTE;
        sys_params.network_id.channel     = DEFAULT_CHANNEL;
        sys_params.network_id.pn_code      = DEFAULT_PN_CODE;
        sys_params.network_id.seed        = DEFAULT_CHECKSUM_SEED;
        sys_params.network_id.pin            = 0;
        sys_params.network_id.base_channel = 0xFF;

        #ifndef PROTOCOL_1_1
        NVRAM_WRITE( &sys_params );
        #endif // PROTOCOL_1_1
    }
    
#ifdef PROTOCOL_1_1
    else
    {
        // calculate channel, PN code, etc. from Bridge MID and store 
        // values in sys_params
        calculate_network_id();
    }
#endif


    
    // configure the Radio
    radio_init();

    // set up the channel and PN Code
    protocol_bind_init();
    
    return Sts_OK;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    calculate_network_id
//
///////////////////////////////////////////////////////////////////////////////

#ifdef PROTOCOL_1_1
void calculate_network_id(void)
{
    UINT8 base_channel, pn_code, seed;

    // calculate PN Code
    pn_code = sys_params.bridge_mid.mid_1 ^ sys_params.bridge_mid.mid_2 ^ 
             sys_params.bridge_mid.mid_3;

    while (pn_code >= NUM_PN_CODES) 
    {
        pn_code -= NUM_PN_CODES;
    }

    // calculate base channel
    base_channel = sys_params.bridge_mid.mid_1;

    while (base_channel >= NUM_CHANNELS)
    {
        base_channel -= NUM_CHANNELS;
    }

    seed = sys_params.bridge_mid.mid_1 + sys_params.bridge_mid.mid_2 + 
           sys_params.bridge_mid.mid_3 + sys_params.bridge_mid.mid_4;

    sys_params.network_id.base_channel     = base_channel;
    sys_params.network_id.channel         = base_channel;
    sys_params.network_id.pn_code          = pn_code;
    sys_params.network_id.seed            = seed;
    sys_params.network_id.pin             = 0;
}
#endif // PROTOCOL_1_1


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_get_tx_pkt
//
//  Description: Returns a pointer to the radio transmit packet 
//
//  Inputs:      Void
//
//  Returns:     void *
//
///////////////////////////////////////////////////////////////////////////////

void * protocol_get_tx_pkt(void)
{
    return &tx_packet.data.app_packet;
}


#ifdef RETURN_TX_PACKET
///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_rtrn_tx_pkt
//
//  Description: Returns the status of a returned radio transmit packet 
//
//  Inputs:      void *
//
//  Returns:     Status
//
///////////////////////////////////////////////////////////////////////////////

Status protocol_rtrn_tx_pkt(void * packet)
{
    return Sts_OK;
}
#endif RETURN_TX_PACKET

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_send_packet
//
//  Description: Transmit a packet out the LS Radio, using the One Way WUSB
//               LS Protocol 
//
//  Inputs:      Void
//
//  Returns:     void *
//
///////////////////////////////////////////////////////////////////////////////

UINT8 protocol_send_packet(UINT8 data_length)    // One Way
{
    UINT8 i;
    UINT8 length;
    
    #ifdef RADIO_SLEEP_ENABLED
    radio_wakeup();
    #endif

    // send a Null Packet to poll for back channel data or as a 
    // keep alive packet...
    if(data_length == 0) 
    {
        // NULL packet from 2-way protocol, used as keep-alive
        tx_packet.first.byte = 0x70; 
        length = 1;
    }
    
    // setup for a Data Packet and bump the data toggle
    else if (data_length < APP_TX_PACKET_SIZE + 1) 
    {     
        tx_packet.first.byte       = 0; 
        tx_packet.data.hdr.type    = DATA_PACKET;
        tx_packet.data.hdr.fill    = 0;
        tx_packet.data.hdr.dt_data = data_toggle++;
        tx_packet.data.hdr.dev_id   = (protocol_device_type & 0x01);
        tx_packet.data.hdr.parity  = protocol_gen_parity(tx_packet.first.byte);
    
        // calculate check sum, etc...
        tx_packet.data.app_packet[data_length] = 
            protocol_calc_ck_sum((UINT8 *)&tx_packet, data_length+1,
                                  sys_params.network_id.seed);
        length = data_length + 2;
    }

    radio_set_channel(sys_params.network_id.channel);
    radio_transmit_on();
    radio_transmit(length, (UINT8 *)&tx_packet);
    SPI_RADIO_OFF();

    // perform retransmit, if TX_NUM_TRANS = 1 this 
    // loop doesn't execute
    for(i = 1; TX_NUM_TRANS - 1 >= i; i++)
    {
        // wait between packet retransmissions
        TIMER_DELAY_MSEC(GAP_DELAY);
        radio_set_channel(sys_params.network_id.channel);
        radio_transmit_on();
        radio_transmit(length, (UINT8 *)&tx_packet);
        SPI_RADIO_OFF();
    }
    #ifdef RADIO_SLEEP_ENABLED
    radio_sleep();
    #endif

    return DATA_SENT;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_calc_ck_sum
//
//  Description: Calculate Packet checksum
//
//  Inputs:      Packet pointer
//               Length of packet
//               Check sum Seed
//
//  Returns:     Checksum
//
///////////////////////////////////////////////////////////////////////////////

UINT8 protocol_calc_ck_sum(UINT8 *data, UINT8 len, UINT8 seed)
{
    UINT8 sum;
    
    for(sum = seed; len > 0; len--, data++) 
    {
        sum ^= *data;
    }        

    return(sum);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_gen_parity
//
//  Description: calculate odd parity for a byte
//
//  Inputs:      byte
//
//  Returns:     1 or 0, based on input byte
//
///////////////////////////////////////////////////////////////////////////////

UINT8 protocol_gen_parity(UINT8 data)
{
    UINT8 count = 0;

    for (; data; )
    {
        data>>=1; // shift out the parity bit...

        // check for bits that are set
        if (data & 0x01)
        {
           ++count;
        }   
    }    

    // check for an odd number
    if( count & 0x01 )
        return 0;
    else
        return 1;
}
