//--------------------------------------------------------------------------
//! \file
//!  
//!  PC Gamepad application function collection 
//
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/DVK Gamepad/gamepad.c $
// $Modtime: 3/24/04 9:12a $
// $Revision: 1.4 $
//--------------------------------------------------------------------------
//
// Copyright 2003, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

//--------------------------------------
// Included files
//--------------------------------------

#include "gamepad.h"
#include "pdc9075.h"


//--------------------------------------
// Local Definitions and Types
//--------------------------------------

//! Flag to know when to perform rumble
static UINT8  rumble_flag = 0;

//! To track inactivity
UINT16        sleep_count;

//! Macro for a transmit packet
APP_TX_PACKET stored_report;

//! WEST  - The '4' button on keypad corresponds to the west direction of 
//!         "Point of View Hat"  of gamepad
//!
//! NORTH - The '8' button on keypad corresponds to the north direction of 
//!         "Point of View Hat"  of gamepad
//!
//! SOUTH - The '2' button on keypad corresponds to the south direction of 
//!         "Point of View Hat"  of gamepad 
//!
//! EAST  - The '6' button on keypad corresponds to the east direction of 
//!         "Point of View Hat"  of gamepad 
//!
//! RIGHT - The right joystick button corresponds to button 2 on gamepad
//! 
//! LEFT  - The left joystick button corresponds to button 1 on gamepad
static const UINT16 button_mask[] = {
    0x0000,                 // 0,0  keypad 7    
    WEST,                   // 0,1  keypad 4
    0x0000,                 // 0,2  keypad 1
    0x0000,                 // 0,3  keypad 0
    NORTH,                  // 1,0  keypad 8
    0x0000,                 // 1,1  keypad 5
    SOUTH,                  // 1,2  keypad 2
    0x0000,                 // 1,3  keypad .
    0x0000,                 // 2,0  keypad 9
    EAST,                   // 2,1  keypad 6
    0x0000,                 // 2,2  keypad 3
    0x0000,                 // 2,3  spare
    RIGHT,                  // 3,0  right thumb switch
    0x0000,                 // 3,1  keypad -
    0x0000,                 // 3,2  keypad +
    0x0000,                 // 3,3  keypad Enter
    LEFT,                   // 4,0  left thumb switch
    0x0000,                 // 4,1  num lock
    0x0000,                 // 4,2  keypad /
    0x0000                  // 4,3  keypad *
};


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_init
//
//!  Description: Initialize the gamepad ports for the PDC-9075 board
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void app_init()
{
    
    // remove power from the POTS
    PRT2DR = 0x05;
    PRT2DM0 = 0xF0;
    PRT2DM1 = 0x0F;

}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    scan_buttons
//
//!  Description: Scan the gamepad button ports for activity via each column/
//!               row combination and sets the button state dependent upon 
//!               which buttons are held down.  Then restores the port used by
//!               the column
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - button_state - What buttons are pressed
//!
///////////////////////////////////////////////////////////////////////////////
static UINT16 scan_buttons()
{
    UINT16 button_state = 0;
    UINT8 index;
    UINT8 row;
    UINT8 col;
    UINT8 mask;
    UINT8 rowBit;
    UINT8 column_port_shadow;

    column_port_shadow = COL_PORT_DR;
    for(col = 0; col < COLUMNS; ++col)
    {
        // drive the column
        COL_PORT_DR = ~(0x01 << col);
        // read the row
        row = PRT5DR;
        row &= (ROW_0 | ROW_1 | ROW_2 | ROW_3);
        mask = 0x01;
        for(mask = 0x01, rowBit = 0;rowBit < ROWS; ++rowBit, mask <<= 1)
        {
            index = (col * ROWS) + rowBit;           
            if(!(row & mask))   // down key
            {
                // clear button mask position
                button_state &= ~(button_mask[index]);
            }
            else                // upkey
            {
                // set button mask position
                button_state |= (button_mask[index]);
            }
        }
    }
    COL_PORT_DR = column_port_shadow;
    return(button_state);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_tx_process
//
//!  Description: Process the gamepad activity report by checking pot
//!               positions, button presses, and joystick movement.  Then the 
//!               actual report packet is generated 
//!
//!  Inputs:      
//!               - *report - report transmit packet to store any activity to
//!               transmit 
//!
//!  Returns:     
//!               - activity - whether ACTIVITY or NO_ACTIVITY to report
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 app_tx_process(APP_TX_PACKET *report)
{
    UINT8 data;
    UINT8 activity = NO_ACTIVITY;
    UINT16 button_state;
    
        // check for movement
    data = 0xFF - get_pot(LEFT_X);
    if(activity_check(data, stored_report.leftX))
    {
        activity = ACTIVITY;
        stored_report.leftX = data;
    }

    data = 0xFF - get_pot(LEFT_Y);
    if(activity_check(data, stored_report.leftY))
    {
        activity = ACTIVITY;
        stored_report.leftY = data;
    }

    data = 0xFF - get_pot(RIGHT_X);
    if(activity_check(data, stored_report.rightX))
    {
        activity = ACTIVITY;
        stored_report.rightX = data;
    }

    data = 0xFF - get_pot(RIGHT_Y);
    if(activity_check(data, stored_report.rightY))
    {
        activity = ACTIVITY;
        stored_report.rightY = data;
    }

    data = 0xFF - get_pot(POT_A);
    if(activity_check(data, stored_report.leftTrigger))
    {
        activity = ACTIVITY;
        stored_report.leftTrigger = data;
    }

    data = 0xFF - get_pot(POT_B);
    if(activity_check(data, stored_report.rightTrigger))
    {
        activity = ACTIVITY;
        stored_report.rightTrigger = data;
    }
    
    // see if the  button state has changed
    button_state = scan_buttons();
        
    // do the button field first
    data = 0;
    if(!(button_state & RIGHT))
    {
        activity = ACTIVITY;
        data |= 0x02;
    }
    if(!(button_state & LEFT))
    {
        activity = ACTIVITY;
        data |= 0x01;
    }
    if(data != stored_report.buttons)
    {
        activity = ACTIVITY;
        stored_report.buttons = data;
    }
    
    // now the hat
    // process clockwise
    data = 0;
    if(!(button_state & NORTH))
    {
        data = 1;
        activity = ACTIVITY;
    }
    if(!(button_state & EAST))
    {
        data = 3;
        activity = ACTIVITY;
    }
    if(!(button_state & SOUTH))
    {
        data = 5;
        activity = ACTIVITY;
    }
    if(!(button_state & WEST))
    {
        data = 7;
        activity = ACTIVITY;
    }
    if(!(button_state & (NORTH | WEST)))
    {
        data = 8;
        activity = ACTIVITY;
    }    
    if(!(button_state & (NORTH | EAST)))
    {
        data = 2;
        activity = ACTIVITY;
    }    
    if(!(button_state & (SOUTH | EAST)))
    {
        data = 4;
        activity = ACTIVITY;
    }    
    if(!(button_state & (SOUTH | WEST)))
    {
        data = 6;
        activity = ACTIVITY;
    }    
    if(data != stored_report.hat)
    {
        activity = ACTIVITY;
        stored_report.hat = data;
    }   
    

    if (activity == ACTIVITY)
    {
        report->leftX = stored_report.leftX;
        report->leftY = stored_report.leftY;
        report->rightX = stored_report.rightX;
        report->rightY = stored_report.rightY;
        report->leftTrigger = stored_report.leftTrigger;
        report->rightTrigger = stored_report.rightTrigger;
        report->buttons = stored_report.buttons;
        report->hat = stored_report.hat;
    }
    #ifdef BACK_CHANNEL_DATA    
    else if(activity == NO_ACTIVITY)
    {
        // logic to force back channel data, for testing 
        if(++sleep_count % 100 == 0)
            return(NULL_ACTIVITY); 
    }
    #endif
    
    return(activity);
}

#ifdef BACK_CHANNEL_DATA


// Local helper functions

void performRumble();
void checkRumble(UINT8 rumble);

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_rx_process
//
//!  Description: Receive the Gamepad packet, and calculate errors. The XOR 
//!               checksum is also used to correct missing bits. If more 
//!               than one bit is missing in the same bit position the 
//!               packet is discarded. 
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - 1 - Mouse packet received
//!              
///////////////////////////////////////////////////////////////////////////////
UINT8 app_rx_process()
{ 
    // Get the rumble data
    performRumble();

    return(1);
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    checkRumble
//
//!  Description: Check and set the rumble flag and calls performRumble()
//!
//!  Inputs:      
//!               - rumble - rumble value (0 = off, >1 = start value)
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void checkRumble(UINT8 rumble)
{

    if((rumble != 0) && (rumble_flag == 0))
    {
        rumble_flag = 1;
    }
    if((rumble == 0) && (rumble_flag == 0))
    {
        rumble_flag = 0;
    }
    // do the rumble function
    performRumble();
}

//! LED rumble array using Kludgey LED pattern
static const UINT8 rumbleLED[] = {0x00,0x00,0x18,0x24,
                                  0x42,0x81,0x42,0x24,0x18};

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    performRumble
//
//!  Description: Perform the DVK rumble interpretation by lighting LED's.
//!               Kludgey LED pattern
//!
//!  Inputs:      
//!               - rumble - rumble value (0 = off, >1 = start value)
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void performRumble()
{
    // add a delay so you can see the pattern
    if(rumble_flag > 0) 
    {
        rumble_flag++;
        if(rumble_flag >= sizeof(rumbleLED)) 
        {
            rumble_flag = 1;
        }
        // light the LED's    
        COL_PORT_DR = rumbleLED[rumble_flag];
    }
}


#endif // BACK_CHANNEL_DATA
