//--------------------------------------------------------------------------
//!
//! \file
//!  This file contains DVK Gamepad main control code
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/DVK Gamepad/main.c $
// $Modtime: 3/22/04 5:30p $
// $Revision: 1.4 $
//--------------------------------------------------------------------------
//
// Copyright 2003, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//--------------------------------------------------------------------------

//--------------------------------------
// Included files
//--------------------------------------

#include "protocol.h"
#include "gamepad.h"
#ifdef BIND_BASIC
#include "basic_bind.h"
#else
#include "bind.h"
#endif // BIND_BASIC
#include "mfgtest.h"

#ifdef BIND_BASIC
static UINT8  newChannel, lastChannel, currentChannel, swCnt = 0;
#endif // BIND_BASIC

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    main
//
//!  Description: Gamepad main routine initializes the interrupt service
//!               routine, timer, platform board, protocol, and gamepad. Then 
//!               gets transmit and receive packet pointers.  Runs the   
//!               manufacturing tests (if USE_MFG_TEST is defined).  Then it 
//!               continuously gets a time stamp and checks for activity.  If
//!               there is activity, it will send transmit a packet and turn on 
//!               the connect LED, if it is valid data.  If it isn't valid data 
//!               the connect LED will shut off.  If there is no activity, it
//!               will look for a bind button press and bind if necessary.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void main()
{
    APP_TX_PACKET *tx_pkt;
    UINT8          mode;
    UINT8          status;
    UINT8          bind_status;
    TIME_STAMP     ts;
    #ifdef USE_MFG_TEST
    UINT8          row;
    UINT8          column_port_shadow;
    UINT8          col = 0x02;
    #endif //USE_MFG_TEST

    APP_RX_PACKET *rx_pkt;

    // initialize the system
    // setup the GPIO interrupt logic
    isr_init();

    // initialize the timer and its ISR
    timer_init();

    // configure the board
    platform_init();

    // setup and configure the radio and protocol
    protocol_init();

    app_init();
    
    // get the packet pointers
    tx_pkt = protocol_get_tx_pkt();
    
    rx_pkt = protocol_get_rx_pkt();

    #ifdef BIND_BASIC
    // setup channel variables
    currentChannel = lastChannel = get_channel();
    protocol_bind(currentChannel, DEFAULT_PN_CODE);
    #endif
    
    #ifdef USE_MFG_TEST
    // using BUTTON_SPARE which is row 3 column 2
    column_port_shadow = COL_PORT_DR;
    // drive the column
    COL_PORT_DR = ~(0x01 << col); //0x01 << col = 0100 (Column 2)
    // read the row
    row = PRT5DR & ROW_3;
    COL_PORT_DR = column_port_shadow;
    if(!(row & 0x08)) // down key -> !(row & mask)  ROW_3 is defined as 8
                      //mask = 1b for ROW_0, 10b for ROW_1 ... 1000b for ROW_3
    {
        MFG_TEST();
    }
    #endif //USE_MFG_TEST

    // main infinite control loop
    while(1) 
    {   
        ts = TIMER_GET_TIME_STAMP();
        
        // perform app function 
        mode = app_tx_process( tx_pkt );  
        
        // send packet if needed   
        if( mode != NO_ACTIVITY )
        {
            // radio_transmit the application report
            #ifdef BACK_CHANNEL_DATA
            if (mode == NULL_ACTIVITY) 
            {
                status = protocol_send_packet(0); 
            }
            else 
            #endif // BACK_CHANNEL_DATA
            {
                status = protocol_send_packet(APP_TX_PACKET_SIZE); 
            }
            
            if( status == DATA_SENT )
            {
                do_leds();
                set_connect_led(ON);
            }
            #ifdef BACK_CHANNEL_DATA
            else if (status == ACK_DATA) 
            {
                app_rx_process();
                do_leds();
            }            
            #endif // BACK_CHANNEL_DATA
            else if ( status == TX_TIMEOUT_ERR )
            {
                set_connect_led(OFF);
            }  
        } //   if( mode != NO_ACTIVITY )
        
        if(mode == NO_ACTIVITY)
        {
            #ifdef BIND_BASIC
            if (check_buttons_active() == 0) 
            {
                newChannel = get_channel();
                
                // make sure a false channel 0 hasn't occured
                if((newChannel == lastChannel) && 
                   (newChannel != currentChannel))
                {
                    if(swCnt++ > 10)
                    {
                        swCnt = 0;
                        currentChannel = newChannel;
                        protocol_bind(currentChannel, DEFAULT_PN_CODE);
                    }   
                }
                else
                {
                    swCnt = 0;
                    lastChannel = newChannel;
                }
            }
            
            #else // BIND_AUTO
            if(bind_pressed())
            {
                set_bind_led(ON); 

                // turn off the Connect LED
                set_connect_led(OFF); 

                // perform the bind routine
                bind_status = protocol_bind();

                set_bind_led(OFF);

                if (bind_status == LS_STATUS_SUCCESS) 
                {
                    set_connect_led(ON);
                }
            }
            #endif // BIND_BASIC
        }
        
        // control the TX period
        TIMER_DELAY_INCREMENTAL( ts, GP_PERIOD);     

        // Prevent "Tick" roll-over by turning the timer off and on
        timer_timer_off();
        timer_timer_on(); 
        
    }  // while(1)

}  // end main
