//--------------------------------------------------------------------------
//! \file
//!
//!  Collection of routines to initialize and access the PSOC
//!  ports specifically for the PDC-9075 board
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/DVK Gamepad/pdc9075.c $
// $Modtime: 3/04/04 2:40p $
// $Revision: 1.4 $
//--------------------------------------------------------------------------
//
// Copyright 2003-2004, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

//--------------------------------------
// Included files
//--------------------------------------

#include "amux4_1.h"
#include "amux4_2.h"
#include "adcincvr_1.h"
#include "pga_1.h"
#include "spi_clk.h"
#include "pdc9075.h"
#include "timer.h"

//--------------------------------------
// Local Definitions and Types
//--------------------------------------

//! Pattern array for LED's
const  UINT8 LEDpattern[] = {0xFE, 0xFD, 0xFB, 0xF7, 0xEF, 0xDF, 0xBF,
                             0x7F, 0xBF, 0xDF, 0xEF, 0xF7, 0xFB, 0xFD};
//! For moving to different elements of the LED array
static UINT8 led_index;



///////////////////////////////////////////////////////////////////////////////
//
//   Function:    platform_init
//
//!  Description: Initialize the PSOC ports for the PDC-9075 board by setting
//!               up the serial port (if necessary), setting the A/D resolution
//!               to 8 bits, setting the power to the switched capacitor PSoC 
//!               block (ADCINCVR_1_HIGHPOWER), setting the power level for the
//!               Programmable Gain Amplifier (PGA) User Module to enhance 
//!               signal, initializing ports 4 and 5 for switches, and by  
//!               clearing the green and red LEDs
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void platform_init()
{
    SPI_Clk_Start();
    
    // setup A/D resolution to 8 bits
    ADCINCVR_1_SetResolution(8);

    // Apply power setting to the module's analog PSoc block.
    ADCINCVR_1_Start(ADCINCVR_1_HIGHPOWER);
    
    AMUX4_1_Start();
    AMUX4_2_Start();
    
    // Apply power setting to the module's PSoC block
    PGA_1_Start(PGA_1_HIGHPOWER);
    
    // initialize port 4 and 5 for reading switches
    sw_ports_init();
    
    // clear red and green LEDS
    LED_PORT_DR = LEDS_OFF;
}


///////////////////////////////////////////////////////////////////////////////
//
//   Function:    sw_ports_init
//
//!  Description: Initialize ports 4 and 5 for reading switches on
//!               the keypad, joystick, and the selector switch.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void sw_ports_init()
{

    // enable the row pull-ups
    PRT5DR |= (ROW_0 | ROW_1 | ROW_2 | ROW_3);
    // drive all the columns high
    COL_PORT_DR = (COL_0 | COL_1 | COL_2 | COL_3 | COL_4 | COL_5 | COL_6 | COL_7);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    get_adc
//
//!  Description: Sets channel and the analog IO control register. Places the
//!               signal from one of the four port pins on the analog column bus 
//!               for either AMUX 1 or 2 (depending on the channel that is
//!               passed in). Starts the A/D converter by passing 1 sample. Waits
//!               for available analog data. Receives data from A/D and the
//!               results ready flag is cleared.
//!
//!  Inputs:      
//!               - channel - Channel ID
//!
//!  Returns:     
//!               - result - Digital reading of the analog value
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 get_adc(UINT8 channel)
{
    short result;

    channel &= 0x07;
    if(channel & 1)
    {
        // set Analog 
        ABF_CR0 = 0;
        
        //  Place the signal from one of four port0 pins on the Analog Column
        //  bus for AMUX4 #1.
        AMUX4_1_InputSelect(channel / 2);
    }
    else
    {
        ABF_CR0 = 0x40;
        
        // Place the signal from one of four port0 pins on the Analog Column 
        // bus for AMUX4 #2.
        AMUX4_2_InputSelect(channel / 2);
    }

    // Starts the A/D convertor 
    ADCINCVR_1_GetSamples(1);

    // wait for new analog data to become available
    while(!ADCINCVR_1_fIsDataAvailable());

    //  Returns the data from the A/D
    result = ADCINCVR_1_iGetDataClearFlag();

    return((UINT8)result);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    get_channel
//
//! Description: This routine is used for the basic bind mode for gamepad. After
//!              saving the port configuration it drives column pins 5 and 6 low
//!              and reads the value of the selector switch on the board (to know
//!              what channel to return)
//!
//! Inputs:      Void
//!
//! Returns:     
//!              - 0 to 15 - Value of selector switch position
//!
///////////////////////////////////////////////////////////////////////////////

UINT8 get_channel(void)
{
    UINT8 result;
    UINT8 column_port_shadow;

    // save port 4 configuration
    column_port_shadow = COL_PORT_DR;
    
    // drive the common pins low, COL_5/6
    COL_PORT_DR = (COL_0 | COL_1 | COL_2 | COL_3 | COL_4 | COL_7);

    // read the selector switch
    result = PRT5DR & (ROW_0 | ROW_1 | ROW_2 | ROW_3);

    // restore port 4 configuration
    COL_PORT_DR = column_port_shadow;

    result = (15-result) * 5;

    return result;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    get_pot
//
//!  Description: If a joystick potentiometer is passed in then it powers up, 
//!               reads the value of, and powers down the given joystick 
//!               potentiometer.  If knob potentiometer is passed in then it 
//!               selects, reads the value of, and deselects the given knob 
//!               potentiometer.
//!
//!  Inputs:      
//!               - pot - Potentiometer ID
//!
//!  Returns:     
//!               - result - Potentiometer value received from get_adc()
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 get_pot(UINT8 pot)
{
    UINT8 result;

    switch(pot)
    {
        case LEFT_X:
        {
            // power the POT
            LED_PORT_DR |= COL_7;
            
            // read the pot
            result = get_adc(3);
            
            // depower the POT
            LED_PORT_DR &= ~COL_7;
            break;
        }
        case LEFT_Y:
        {
            // power the POT
            LED_PORT_DR |= COL_7;
            
            // read the pot
            result = get_adc(2);
            
            // unpower the POT
            LED_PORT_DR &= ~COL_7;
            break;
        }
        case RIGHT_X:
        {
            // power the POT
            LED_PORT_DR |= COL_7;
             
            // read the pot
            result = get_adc(1);
            
            // unpower the POT
            LED_PORT_DR &= ~COL_7;
            break;
        }
        case RIGHT_Y:
        {
            // power the POT
            LED_PORT_DR |= COL_7;
            
            // read the pot
            result = get_adc(0);
            
            // unpower the POT
            LED_PORT_DR &= ~COL_7;
            break;
        }
        case POT_A:
        {
            // select the POT
            PRT2DM0 = 0xF3;
            PRT2DM1 = 0x0C;
            
            // read the pot
            result = get_adc(4);
            
            // deselect the POT
            PRT2DM0 = 0xF0;
            PRT2DM1 = 0x0F;
            break;
        }
        case POT_B:
        {
            // select the POT
            PRT2DM0 = 0xF3;
            PRT2DM1 = 0x0C;
            
            // read the pot
            result = get_adc(5);
            
            // deselect the POT
            PRT2DM0 = 0xF0;
            PRT2DM1 = 0x0F;
            break;
        }
        case POT_C:
        {
            // select the POT
            PRT2DM0 = 0xF3;
            PRT2DM1 = 0x0C;
            
            // read the pot
            result = get_adc(6);
            
            // deselect the POT
            PRT2DM0 = 0xF0;
            PRT2DM1 = 0x0F;
            break;
        }
        case POT_D:
        {
            // select the POT
            PRT2DM0 = 0xF3;
            PRT2DM1 = 0x0C;
            
            // read the pot
            result = get_adc(7);
            
            // deselect the POT
            PRT2DM0 = 0xF0;
            PRT2DM1 = 0x0F;
            break;
        }
        case POT_E:
        {
            // select the POT
            PRT2DM0 = 0xFC;
            PRT2DM1 = 0x03;
            
            // read the pot
            result = get_adc(4);
            
            // deselect the POT
            PRT2DM0 = 0xF0;
            PRT2DM1 = 0x0F;
            break;
        }
        case POT_F:
        {
            // select the POT
            PRT2DM0 = 0xFC;
            PRT2DM1 = 0x03;
            
            // read the pot
            result = get_adc(5);
            
            // deselect the POT
            PRT2DM0 = 0xF0;
            PRT2DM1 = 0x0F;
            break;
        }
        case POT_G:
        {
            // select the POT
            PRT2DM0 = 0xFC;
            PRT2DM1 = 0x03;
            
            // read the pot
            result = get_adc(6);
            
            // deselect the POT
            PRT2DM0 = 0xF0;
            PRT2DM1 = 0x0F;
            break;
        }
        case POT_H:
        {
            // select the POT
            PRT2DM0 = 0xFC;
            PRT2DM1 = 0x03;
            
            // read the pot
            result = get_adc(7);
            
            // deselect the POT
            PRT2DM0 = 0xF0;
            PRT2DM1 = 0x0F;
            break;
        }
    }
    return(result);

}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    check_buttons_active
//
//!  Description: Scan the gamepad button ports for activity via each column/
//!               row combination and sets the button state dependent upon 
//!               which buttons are held down.  Then restores the port used by
//!               the column
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - 0 - No activity
//!               - 1 - Button down
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 check_buttons_active()
{
    UINT16 buttonState = 0;
    UINT8 index;
    UINT8 row;
    UINT8 col;
    UINT8 mask;
    UINT8 row_bit;
    UINT8 column_port_shadow;

    column_port_shadow = COL_PORT_DR;
    for(col = 0; col < COLUMNS; ++col)
    {
        // drive the column
        COL_PORT_DR = ~(0x01 << col);
        // read the row
        row = PRT5DR;
        row &= (ROW_0 | ROW_1 | ROW_2 | ROW_3);
        mask = 0x01;
        for(mask = 0x01, row_bit = 0;row_bit < ROWS; ++row_bit, mask <<= 1)
        {
            index = (col * ROWS) + row_bit;           
            if(!(row & mask))   // down key
            {
                // let PERT know if the "enter" button was pressed
#ifdef PERT
                if( index == 0x0F )
                {
                    COL_PORT_DR = column_port_shadow;
                    return 1;
                }
                else
                {
                    COL_PORT_DR = column_port_shadow;
                    return 0;
                }
#else
                COL_PORT_DR = column_port_shadow;
                return 1;
#endif
            }
        }
    }
    COL_PORT_DR = column_port_shadow;
    
    return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    activity_check
//
//!  Description: Check for POT activity
//!
//!  Inputs:      
//!               - this - Current POT value 
//!               - last - Previous POT value
//!
//!  Returns:     
//!               - 0 - No activity
//!               - 1 - Activity
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 activity_check(UINT8 this, UINT8 last)
{
    if(this > last)
    {
        if((this - last) > DEADBAND)
        {
            return(1);
        }
    }
    else
    {
        if((last - this) > DEADBAND)
        {
            return(1);
        }
    }
    return(0);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    do_leds
//
//!  Description: Increment and display activity LED's for TX/RX
//!
//!  Inputs:      Void 
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void do_leds(void)
{
    led_index++;
    if(led_index >= sizeof(LEDpattern))
    {
        led_index = 0;
    }
    // light the LED's    
    COL_PORT_DR = LEDpattern[led_index];
}

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    bind_pressed
//
//!  Description: Check to see if the bind button is pressed.  If bind button 
//!               is pressed, there is a 20msec delay for the button release
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - 1 - Bind button pressed           
//!               - 0 - Bind button not pressed
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 bind_pressed(void)
{
    UINT8  led_port_shadow;

    led_port_shadow = LED_PORT_DR;        
    if(led_port_shadow &= BUTTON_BIND)
    {
        // clear the button
        led_port_shadow &= ~BUTTON_BIND; 
        
        // wait for the button to be released
        while( LED_PORT_DR &= BUTTON_BIND )
        {
            // clear the button
            led_port_shadow &= ~BUTTON_BIND; 
            LED_PORT_DR = led_port_shadow;
            
            // wait to check for the button release
            timer_delay_msec( 20 );     
        }
        // clear the button
        led_port_shadow &= ~BUTTON_BIND;                  
        LED_PORT_DR = led_port_shadow;
        return 1;
    }
    return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    set_bind_led
//
//!  Description: Turn the bind LED on or off
//!
//!  Inputs:      
//!               - state - ON or OFF.
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void set_bind_led(UINT8 state)
{
    UINT8  led_port_shadow;

    led_port_shadow = LED_PORT_DR;        
    if( state == ON )
    {
        // clear the button
        led_port_shadow &= ~BUTTON_BIND; 

        // turn the LED on
        LED_PORT_DR = led_port_shadow & ~BIND_LED;
    }
    else  // turn it off
    {
        // clear the button
        led_port_shadow &= ~BUTTON_BIND; 

        // turn off the LED
        LED_PORT_DR = led_port_shadow | BIND_LED;
    }
}  

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    set_connect_led
//
//!  Description: Turn the connect LED on or off
//!
//!  Inputs:      
//!               - state - ON or OFF.
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void set_connect_led(UINT8 state)
{
    UINT8  led_port_shadow;

    led_port_shadow = LED_PORT_DR;        
    if( state == ON )
    {
        // clear the button
        led_port_shadow &= ~BUTTON_CONNECT; 

        // turn the LED on
        LED_PORT_DR = led_port_shadow & ~CONNECT_LED;
    }
    else  // turn it off
    {
        // clear the button
        led_port_shadow &= ~BUTTON_CONNECT; 

        // turn off the LED
        LED_PORT_DR = led_port_shadow | CONNECT_LED;
    }
} 

