//--------------------------------------------------------------------------
//! \file
//!
//!  Keypad application function collection 
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/DVK Keypad/keypad.c $
// $Modtime: 3/04/04 3:11p $
// $Revision: 1.5 $
//--------------------------------------------------------------------------
//
// Copyright 2003, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

//--------------------------------------
// Included files
//--------------------------------------

#include "keypad.h"
#include "pdc9075.h"


//--------------------------------------
// Local Definitions and Types
//--------------------------------------
//! 0x5F corresponds to button 7 on keypad
//!
//! 0x5C corresponds to button 4 on keypad
//!
//! 0x59 corresponds to button 1 on keypad
//!
//! 0x62 corresponds to button 0 on keypad
//!
//! 0x60 corresponds to button 8 on keypad
//!
//! 0x5D corresponds to button 5 on keypad
//!
//! 0x5A corresponds to button 2 on keypad
//!
//! 0x63 corresponds to button . on keypad
//!
//! 0x61 corresponds to button 9 on keypad
//!
//! 0x5E corresponds to button 6 on keypad
//!
//! 0x5B corresponds to button 3 on keypad
//!
//! 0x4C corresponds to button Spare (delete)
//!
//! 0x4D corresponds to right joystick (end)
//!
//! 0x56 corresponds to button - on keypad
//!
//! 0x57 corresponds to button + on keypad
//!
//! 0x58 corresponds to button Enter on keypad
//!
//! 0x4A corresponds to button left joystick (home)
//!
//! 0x53 corresponds to button num lock on keypad
//!
//! 0x54 corresponds to button / on keypad
//!
//! 0x55 corresponds to button * on keypad
static const UINT16 keyScan[] = {
    0x5F,           // 0,0  keypad 7    
    0x5C,           // 0,1  keypad 4
    0x59,           // 0,2  keypad 1
    0x62,           // 0,3  keypad 0
    0x60,           // 1,0  keypad 8
    0x5D,           // 1,1  keypad 5
    0x5A,           // 1,2  keypad 2
    0x63,           // 1,3  keypad .
    0x61,           // 2,0  keypad 9
    0x5E,           // 2,1  keypad 6
    0x5B,           // 2,2  keypad 3
    0x4C,           // 2,3  spare (delete)
    0x4D,           // 3,0  right thumb switch (end)
    0x56,           // 3,1  keypad -
    0x57,           // 3,2  keypad +
    0x58,           // 3,3  keypad Enter
    0x4A,           // 4,0  left thumb switch (home)
    0x53,           // 4,1  num lock
    0x54,           // 4,2  keypad /
    0x55            // 4,3  keypad *
};

//! Key array to keep track of current state
UINT8 keyState[COLUMNS + 1];

//! Key queue to buffer key presses
KEY_FIFO    keyQueue;



///////////////////////////////////////////////////////////////////////////////
//
//  Function:    keyQueueInit
//
//!  Description: Initialize the keypad activity queue
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////

static void keyQueueInit()
{
    keyQueue.in = 0;
    keyQueue.out = 0;
    keyQueue.count = 0;
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    putKeyQueue
//
//!  Description: Add a keypad button event to the keypad queue if there 
//!               is room and update keyqueue parameters where needed.
//!
//!  Inputs:      
//!               - state - the state the key being added is in              
//!               - index - index of key
//!
//!  Returns:     
//!               - 0 - no room              
//!               - 1 - state added
//!
///////////////////////////////////////////////////////////////////////////////

static UINT8 putKeyQueue(KEY_STATE state, UINT8 index)
{
    if(keyQueue.count == KEY_QUEUE_LEN)
    {
        return(0);
    }
    
    keyQueue.count++;

    keyQueue.key[keyQueue.in].index = index;
    keyQueue.key[keyQueue.in].state = state;

    keyQueue.in++;
    if(keyQueue.in == KEY_QUEUE_LEN)
    {
        keyQueue.in = 0;
    }
    return(1); 
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    getKeyQueue
//
//!  Description: Get a keypad button event from the keypad queue and update
//!               keyqueue parameters as needed
//!
//!  Inputs:      
//!               - *state - state the key being dequeued is in
//!               - *index - key index
//!
//!  Returns:     
//!               - 0 = no entries
//!               - 1 = state returned
//!
///////////////////////////////////////////////////////////////////////////////

static UINT8 getKeyQueue(KEY_STATE *state, UINT8 *index)
{
    if(keyQueue.count == 0)
    {
        return(0);
    }
    
    keyQueue.count--;
    
    *index = keyQueue.key[keyQueue.out].index;
    *state = keyQueue.key[keyQueue.out].state;

    keyQueue.out++;
    if(keyQueue.out == KEY_QUEUE_LEN)
    {
        keyQueue.out = 0;
    }
    return(1); 
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_init
//
//!  Description: Initialize the keypad queue and stated for each key
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////

void app_init()
{
    UINT8 i;

    keyQueueInit();

    for(i = 0; i < COLUMNS; ++i)
    {
        keyState[i] = 0xFF;
    }
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    scan_keys
//
//!  Description: Scan the keypad ports for activity via each column/
//!               row combination, adds the key to the queue and sets the key 
//!               state dependent upon which keys are held down.  Then restores
//!               the port used by the column
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////

void scan_keys()
{
    UINT8 col, row;
    UINT8 mask, rowBit;
    UINT8 index;
    UINT8 column_port_shadow;
    KEY key;

    // save port 4 setup
    column_port_shadow = COL_PORT_DR;

    // loop through the keypad columns
    for(col = 0; col < COLUMNS; ++col)
    {
        // drive the column
        COL_PORT_DR = ~(0x01 << col);
        // read the row
        row = PRT5DR;
        row &= (ROW_0 | ROW_1 | ROW_2 | ROW_3);
        
                // loop through the keypad row
        mask = 0x01;
        for(mask = 0x01, rowBit = 0;rowBit < ROWS; ++rowBit, mask <<= 1)
        {
            if((row & mask) == (keyState[col] & mask))
            {
                continue;   // No change
            }
            
            if(!(row & mask))   // down key
            {
                key.state = KEY_STATE_DOWN;
            }
            else                // upkey
            {
                key.state = KEY_STATE_UP;
            }
            
            key.index = (col * ROWS) + rowBit;
            
            if(!putKeyQueue(key.state, key.index))
            {
                continue;
            }
            
            // update state if everything okay          
            if(key.state == KEY_STATE_DOWN)
            {
                 keyState[col] &= ~mask;
            }
            else
            {
                  keyState[col] |= mask;        
            }
        }
    }
    // restore port 4 setup
    COL_PORT_DR = column_port_shadow;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_tx_process
//
//!  Description: Process the keypad activity report by scanning the keys for
//!               any activity, and generates the transmit report packet by 
//!               setting the packet parameters (key, state, etc.)
//!
//!  Inputs:      
//!               - *report - report transmit packet to store any activity to
//!               transmit 
//!
//!  Returns:     
//!               - 0 = no activity
//!               - 1 = report to send
//!
///////////////////////////////////////////////////////////////////////////////

UINT8 app_tx_process(APP_TX_PACKET *report)
{
    UINT8     i;
    KEY_STATE state;
    UINT8     index;

    // check for keypad events
    scan_keys();

    if(keyQueue.count == 0)
    {
        return(0);
    }
    report->modifiers = 0;
    report->reserved = 0;

    for(i = 0; i < REPORT_KEYCODES; ++i)
    {
        report->keyCode[i] = 0; 
    }

    i = 0;
    while(getKeyQueue(&state, &index))
    {
        if(state == KEY_STATE_DOWN)
        {
            report->keyCode[i++] = keyScan[index];          
        }   
    }
    
    return(1);
}
