//--------------------------------------------------------------------------
//! \file
//!
//! This module implements a manufacturing test mode
//!
//--------------------------------------------------------------------------
// $Archive: $
// $Modtime: $
// $Revision: 1.4 $
//--------------------------------------------------------------------------
//
// Copyright 2004, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------


//--------------------------------------
// Included files
//--------------------------------------

#include "GlobalParams.h"
#include "mfgtest.h"
#include "radio.h"
#include "protocol.h"


//--------------------------------------
// Local Definitions and Types
//--------------------------------------

// Optionally compile complete module
#if defined(USE_MFG_TEST)

//! Used to count good and erased bits
extern UINT8 channel_valid[];

// Length (bytes) of receive data.  Used in mfg_receive_test_packets() 
extern UINT8 rx_data_length;


//! 8 PN codes for a data rate of 32kbps or 64kbps, and 48 PN codes for a
//! data rate of 16kbps
extern const UINT8 pn_code_table[];

//! To keep track of the total received/erased bits
typedef struct 
{
    //! Total number of bits received
    UINT16 total_received_bits;
    //! Total number of bits erased
    UINT16 total_erased_bits;

} MFG_RESULTS;

//! Header for the manufacturing test packet
#define MFG_HDR_TEST      0xF1

//! Header for the first manufacturing results packet
#define MFG_HDR_RES1      0xFB

//! Header for the second manufacturing results packet
#define MFG_HDR_RES2      0xFD

//! Header for the third manufacturing results packet
#define MFG_HDR_RES3      0xFE


//! Size of manufacturing test packet
#define MFG_TEST_PKT_SIZE (MFG_PACKET_PAYLOAD_SIZE + 2) // add header & checksum

//! Size of manufacturing results packet
#define MFG_RES_PKT_SIZE  10


//--------------------------------------
// Local Function Declarations
//--------------------------------------

static void mfg_setup_radio(void);
static BOOL mfg_receive_test_packets(MFG_RESULTS *);
static void mfg_send_test_packets(void);
static void mfg_send_results(MFG_RESULTS *);
static void mfg_send_result_packet(UINT8 *pkt);


//--------------------------------------------------------------------------
// mfg_check_test
//--------------------------------------------------------------------------


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    mfg_test
//
//!  Description: The main manufacturing test routine continuously clears
//!               the watchdog and sleep, sets the radio up for manufacturing
//!               test mode, and receives data packets.  If the data packets
//!               valid manufacturing test packets, then they are transmitted
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void mfg_test()
{
    MFG_RESULTS results;
    BOOL status;

    while(1)
    {
        // Clear Watchdog and Sleep
        M8C_ClearWDTAndSleep;

        mfg_setup_radio();

        status = mfg_receive_test_packets(&results);

        if( status )
        {
            mfg_send_test_packets();

            mfg_send_results(&results);
        }

    }
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    mfg_setup_radio
//
//!  Description: This routine sets up the radio for manufacturing test mode
//!               by first waking the radio up.  Then the radio is setup with
//!               pn code, channel, power level, and thresholds used for 
//!               manufacturing test mode
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
static void mfg_setup_radio()
{
#ifdef RADIO_SLEEP_ENABLED
    radio_wakeup();
#endif

    SPI_RADIO_PUT(REG_THRESHOLD_LO, MFG_THRESHOLD_LO & THRESHOLD_MASK);

    SPI_RADIO_PUT(REG_THRESHOLD_HI, MFG_THRESHOLD_HI & THRESHOLD_MASK);

    SPI_RADIO_PUT(REG_PA, MFG_PA_BIAS & PA_BIAS_MASK);

    SPI_RADIO_PUT(REG_SERDES_CTL, 
                  SERDES_ENABLE | MFG_EOF_TIMEOUT & EOF_LEN_MASK );
    
    radio_set_pn_code(&pn_code_table[(8 * MFG_PN_CODE)]);

    radio_set_channel(MFG_CHANNEL);
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    mfg_receive_test_packets
//
//!  Description: This routine continuously receives the test packets and 
//!               checks/counts good/erased bits, unless a timeout occurred
//!               during the packet receive test or all packets have been 
//!               fully received. The packets are received through 
//!               radio_receive().  The total amount of good/erased bits for 
//!               all packets are updated as the totals for individual packets
//!               are calculated.
//!               
//!  Inputs:      
//!               - *results - Used for counting the received bits and erased 
//!               bits
//!
//!  Returns:     
//!               - TRUE - received successfully
//!               - FALSE - not received successfully
//!
///////////////////////////////////////////////////////////////////////////////
static BOOL mfg_receive_test_packets(MFG_RESULTS *results)
{
    UINT8       pkt[MFG_TEST_PKT_SIZE];
    UINT16      good_bits_in_packet;
    UINT16      erased_bits_in_packet;
    BOOL        received = FALSE;
    UINT16      count;
    UINT8       bits;
    TIME_STAMP  current_time;

    results->total_received_bits = 0;
    results->total_erased_bits = 0;

    // Get packets with timeout
    while(1)
    {
        current_time = timer_get_time_stamp();
        
        do
        {
            // Clear Watchdog and Sleep
            M8C_ClearWDTAndSleep;

            if( radio_receive(MFG_TEST_PKT_SIZE, pkt, channel_valid, 255) > 0 )
            {
                break;
            }
        }
        while( !timer_time_elapsed(current_time, MFG_RX_TIMEOUT) );

        if( rx_data_length == 0 )
        {
            if( received )
            {
                // Timeout during packet receive test
                break;
            }
            else
            {
                // Still waiting for start of packets
                continue;
            }
        }

        // Count valid bits in the packet
        received = TRUE;
        good_bits_in_packet = 0;
        erased_bits_in_packet = 0;

        for(count = 0; count < rx_data_length; count++)
        {
            for(bits=0; bits<8; bits++)
            {
                if( (channel_valid[count] & 0x01) == 0x01 )
                {
                    ++good_bits_in_packet;
                }
                else
                {
                    ++erased_bits_in_packet;
                }

                channel_valid[count] >>= 1;
            }

            // Clear Watchdog and Sleep
            M8C_ClearWDTAndSleep;
        }

        // Check for noise
        if( good_bits_in_packet >= 4 )
        {
            results->total_received_bits += 8 * rx_data_length;
            results->total_erased_bits += erased_bits_in_packet;
        }

        count = ((UINT16)*((UINT8 *) (&rx_packet.first.byte) + 1) << 8) | *((UINT8 *) (&rx_packet.first.byte) + 2);

        if( count == MFG_NUM_PACKETS - 1 )
        {
            break;
        }
    }

    return received;
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    mfg_send_test_packets
//
//!  Description: Transmits MFG_NUM_PACKETS (currently 200) manufacturing test
//!               packets.  Before each one is sent, the watchdog and sleep is 
//!               cleared and the packet is made and transmitted.  There is a
//!               delay before each packet is sent.  
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
static void mfg_send_test_packets(void)
{
    UINT8  pkt[MFG_TEST_PKT_SIZE];
    UINT16 idx;
    UINT8  idx2;


    timer_delay_msec(MFG_PRE_TX_DELAY);

    for(idx=0; idx < MFG_NUM_PACKETS; idx++)
    {
        // Clear Watchdog and Sleep
        M8C_ClearWDTAndSleep;

        // Build packet
        pkt[0] = MFG_HDR_TEST;
        pkt[1] = idx >> 8;
        pkt[2] = idx & 0xFF;
        
        for(idx2=3; idx2 < MFG_TEST_PKT_SIZE-1; idx2++)
        {
            pkt[idx2] = 'A' + idx2 - 3;
        }

        pkt[idx2] = protocol_calc_ck_sum(pkt, MFG_TEST_PKT_SIZE-1, 0);

        radio_transmit(MFG_TEST_PKT_SIZE, pkt);

        timer_delay_msec(MFG_INTER_TX_DELAY);
    }
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    mfg_send_results
//
//!  Description: Sends three result packets (Each with its own header  
//!               attribute, i.e. MFG_HDR_RES1) and checksum: First, sends a
//!               packet with number of total bits received.  Second, sends a
//!               packet with number of total bits erased.  Then, it sends a
//!               packet with everything set to 0.   
//!
//!  Inputs:      
//!               - *results - Holds the total_received_bits and 
//!               total_erased_bits that were set up in 
//!               mfg_receive_test_packets()
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////

static void mfg_send_results(MFG_RESULTS *results)
{
    UINT8 pkt[MFG_RES_PKT_SIZE];
    UINT8 idx;

    // Build results 1 packet
    pkt[0] = MFG_HDR_RES1;
    pkt[1] = 0;
    pkt[2] = 0;
    pkt[3] = 0;
    pkt[4] = 0;
    pkt[5] = 0;
    pkt[6] = 0;
    pkt[7] = results->total_received_bits >> 8;
    pkt[8] = results->total_received_bits & 0xFF;
    pkt[9] = protocol_calc_ck_sum(pkt, 9, 0);

    mfg_send_result_packet(pkt);


    // Build results 2 packet
    pkt[0] = MFG_HDR_RES2;
    pkt[1] = 0;
    pkt[2] = 0;
    pkt[3] = results->total_erased_bits >> 8;
    pkt[4] = results->total_erased_bits & 0xFF;
    pkt[5] = 0;
    pkt[6] = 0;
    pkt[7] = 0;
    pkt[8] = 0;
    pkt[9] = protocol_calc_ck_sum(pkt, 9, 0);

    mfg_send_result_packet(pkt);


    // Build results 3 packet
    pkt[0] = MFG_HDR_RES3;

    for(idx=1; idx<9; idx++)
    {
        pkt[idx] = 0;
    }

    pkt[9] = protocol_calc_ck_sum(pkt, 9, 0);

    mfg_send_result_packet(pkt);
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    mfg_send_result_packet
//
//!  Description: Transmits the actual packets that are set up in 
//!               mfg_send_results() by clearing the watchdog and sleep to get
//!               full period, the calling radio_transmit() to transmit the 
//!               packet (*pkt) being received.  A delay occurs before each
//!               transmission. 
//!
//!  Inputs:      
//!               - *pkt - Holds the packet data to transmit 
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
static void mfg_send_result_packet(UINT8 *pkt)
{
    UINT8 idx;

    timer_delay_msec(MFG_PRE_RESULTS_DELAY);

    for(idx=0; idx<MFG_NUM_RESULTS_PACKETS; idx++)
    {
        // Clear Watchdog and Sleep
        M8C_ClearWDTAndSleep;

        radio_transmit(MFG_RES_PKT_SIZE, pkt);

        timer_delay_msec(MFG_INTER_RESULTS_DELAY);
    }
}


#endif // MFG_TEST
