//--------------------------------------------------------------------------
//! \file
//!
//!  Mouse application function collection 
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/DVK Mouse/mouse.c $
// $Modtime: 3/04/04 3:11p $
// $Revision: 1.4 $
//--------------------------------------------------------------------------
//
// Copyright 2003, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------


//--------------------------------------
// Included files
//--------------------------------------

#include "mouse.h"
#include "bind.h"


//--------------------------------------
// Local Definitions and Types
//--------------------------------------

//! To keep track of previous button state
UINT8 old_button_state = 0;

//! Mask for mouse buttons
//!
//! M_MIDDLE - Right joystick button corresponds to middle mouse button
//! (Z-wheel button)
//!
//! M_RIGHT  - The '-' button on keypad corresponds to right mouse button 
//!
//! M_LEFT   - The 'NUM LOCK' button on keypad corresponds to left mouse button
static const UINT16 button_mask[] = {
    0x0000,                 // 0,0  keypad 7    
    0x0000,                 // 0,1  keypad 4
    0x0000,                 // 0,2  keypad 1
    0x0000,                 // 0,3  keypad 0
    0x0000,                 // 1,0  keypad 8
    0x0000,                 // 1,1  keypad 5
    0x0000,                 // 1,2  keypad 2
    0x0000,                 // 1,3  keypad .
    0x0000,                 // 2,0  keypad 9
    0x0000,                 // 2,1  keypad 6
    0x0000,                 // 2,2  keypad 3
    0x0000,                 // 2,3  spare
    M_MIDDLE,               // 3,0  right thumb switch
    M_RIGHT,                // 3,1  keypad -
    0x0000,                 // 3,2  keypad +
    0x0000,                 // 3,3  keypad Enter
    0x0000,                 // 4,0  left thumb switch
    M_LEFT,                 // 4,1  num lock
    0x0000,                 // 4,2  keypad /
    0x0000                  // 4,3  keypad *
};

//! Calibration array for X,Y coordinates (mouse - left joystick) and Y 
//! coordinate for (Z-wheel - right joystick)
static INT8 cal[3];

//! To calibrate the X coordinate of the mouse (left joystick)
#define CAL_X 0x00

//! To calibrate the Y coordinate of the mouse (left joystick)
#define CAL_Y 0x01

//! To calibrate the Z-wheel (right joystick)
#define CAL_Z 0x02

//! The joysticks should center (i.e. no movement) at 0x80,
//! with a range 0x00 - 0xFF
#define JOYSTICK_CENTER 0x80

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_init
//
//!  Description: Calibrates by storing the actual joystick center values so that we can
//!               normalize the joystick readings
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////

void app_init()
{
    cal[CAL_X] = 0x80 - get_pot(LEFT_X);    
    cal[CAL_Y] = 0x80 - get_pot(LEFT_Y);    
    cal[CAL_Z] = 0x80 - get_pot(RIGHT_Y);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    scan_buttons
//
//!  Description: Scan the mouse buttons ports for activity via each column/
//!               row combination and sets the button state dependent upon 
//!               which buttons are held down.  Then restores the port used by
//!               the column
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - button_state - State of all buttons (pushed or not pushed)
//!
///////////////////////////////////////////////////////////////////////////////

static UINT16 scan_buttons()
{
    UINT16 button_state = 0;
    UINT8 index;
    UINT8 row;
    UINT8 col;
    UINT8 mask;
    UINT8 row_bit;
    UINT8 columnt_port_shadow;

    // save port 4's configuration
    columnt_port_shadow = COL_PORT_DR;
    
    //loop through the buttons
    for(col = 0; col < COLUMNS; ++col)
    {
        // drive the column
        COL_PORT_DR = ~(0x01 << col);
        // read the row
        row = PRT5DR;
        row &= (ROW_0 | ROW_1 | ROW_2 | ROW_3);
        mask = 0x01;
        for(mask = 0x01, row_bit = 0;row_bit < ROWS; ++row_bit, mask <<= 1)
        {
            index = (col * ROWS) + row_bit;           
            if(!(row & mask))   // down key
            {
                button_state &= ~(button_mask[index]);
            }
            else                // upkey
            {
                button_state |= (button_mask[index]);
            }
        }
    }
    // restore Port 4 config
    COL_PORT_DR = columnt_port_shadow;
    return(button_state);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    normalize_data
//
//!  Description: Adjusts joystick reading based on the actual joystick
//!               center value.  Used to help determine any joystick activity.
//!
//!  Inputs:      
//!               - data - data from joystick coordinates
//!               - index - index of coordinate (X,Y from left or Y from right)
//!
//!  Returns:     
//!               - Normalized joystick value (0xFF, 0, or (data + cal[index]))
//!
///////////////////////////////////////////////////////////////////////////////

UINT8 normalize_data(UINT8 data, UINT8 index)
{
    if ((cal[index] > 0) && (data > 0xFF - cal[index]) )
    {
        return 0xFF;
    }
    else if ((cal[index] < 0) && (data < 0 - cal[index]))
    {
        return 0;
    }
    
    return (data + cal[index]);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_tx_process
//
//!  Description: Process the mouse activity report by first checking for
//!               joystick activity and generating a report.  Then a check for
//!               button activity and generation of the report packet happens.
//!               The button state is stored (old_button_state) 
//!
//!  Inputs:      
//!               - *report - report transmit packet to store any activity to
//!               transmit
//!
//!  Returns:     
//!               - 0 - No activity 
//!               - 1 - Activity occurred and report was generated
//!
///////////////////////////////////////////////////////////////////////////////

UINT8 app_tx_process(APP_TX_PACKET *report)
{
    UINT16 button_state;
    UINT8 activity = 0;
    UINT8 data;
    UINT8 cs;
    UINT8 i;
    
    // check for joystick movement

    data = get_pot(LEFT_X);
    data = normalize_data(data, CAL_X);
    if(activity_check(data, JOYSTICK_CENTER))
    {
        activity = 1;
        report->x = 0 - ((data / 8) - 16);
    }
    else
    {
        report->x = 0;
    }
    
    data = get_pot(LEFT_Y); 
    data = normalize_data(data, CAL_Y);
    if(activity_check(data, JOYSTICK_CENTER))
    {
        activity = 1;
        report->y = 0 - ((data / 8) - 16);
    }
    else
    {
        report->y = 0;
    }
    
    data = get_pot(RIGHT_Y); 
    data = normalize_data(data, CAL_Z);
    if(activity_check(data, JOYSTICK_CENTER))
    {
        activity = 1;
        report->z = (((data / 32) - 0x04) & 0x1F);  
    }
    else
    {
        report->z = 0;
    }
    
    // check for button activity

    button_state = scan_buttons();
        
    data = 0;

    if(!(button_state & M_RIGHT))
    {
        data |= 0x02;
    }
    
    if(!(button_state & M_LEFT))
    {
        data |= 0x01;
    }
    
    if(!(button_state & M_MIDDLE))
    {
        data |= 0x04;
    }

    report->z |= data << 5;
    
    if(data != old_button_state)
    {
        activity = 1;
        old_button_state = data;
    }

    return activity;
}

///////////////////////////////////////////////////////////////////////////////
