///--------------------------------------------------------------------------
//! \file
//!
//!  Collection of routines to access the WirelessUSB LS Radio
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/Radio/radio.c $
// $Modtime: 3/11/04 3:52p $
// $Revision: 1.4 $
//--------------------------------------------------------------------------
//
// Copyright 2003-2004, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------


//--------------------------------------
// Included files
//--------------------------------------

#include "ls_config.h"
#include "spim_1.h"
#include "radio.h"


//--------------------------------------
// Local Definitions and Types
//--------------------------------------

//! For deadman timeout loop in radio_transmit()
#define DEADMAN_MAX_COUNT       200

#ifdef RADIO_NON_BLOCKING_CALLS
#pragma interrupt_handler   radio_rx_isr

//! Mask for the 6 bit address
#define SPI_ADDRESS_MASK    0x3F
#define SPI_ISR_RADIO_GET(address, data)                              \
{                                                                     \
    RADIO_PORT &= ~N_SS;                                              \
    SPIM_1_TX_BUFFER_REG = ((address) & SPI_ADDRESS_MASK);            \
    while(!(SPIM_1_CONTROL_REG & SPIM_1_SPIM_SPI_COMPLETE));          \
    SPIM_1_TX_BUFFER_REG = 0xFF;                                      \
    while(!(SPIM_1_CONTROL_REG & SPIM_1_SPIM_SPI_COMPLETE));          \
    RADIO_PORT |= N_SS;                                               \
    data = SPIM_1_RX_BUFFER_REG;                                      \
}

// These globals are set by radio_rx_isr
//! Channel end of frame
UINT8 channel_EOF = 0;

//! Data overflow
UINT8 overflow = 0;

//! Length (bytes) of receive buffer
UINT8 rx_buffer_length = 0;

//! Buffer for receive data
UINT8 *rx_data_buffer = 0;

//! Buffer for valid receive data
UINT8 *rx_valid_buffer = 0;


#else
//--------------------------------------
// Local Function Declarations
//--------------------------------------
void radio_transmit_on(void);
void radio_receive_on(void);
void radio_off(void);
#endif // RADIO_NON_BLOCKING_CALLS

// Length (bytes) of receive data.  Mainly used in radio_receive() and radio_rx_isr()
UINT8 rx_data_length = 0;

//! Table for the data that configures the radio
const UINT8 radio_init_data[] =
{
    REG_DATA_RATE, RADIO_MODE,
    REG_SERDES_CTL, (SERDES_ENABLE | (EOF_LEN_MASK & EOF_BITS)),
    REG_RX_INT_EN, (RX_FULL_A | RX_EOF_A),
    REG_TX_INT_EN, (TX_EMPTY),  
    REG_VALID_TX, 0xFF,
    REG_THRESHOLD_LO, CHIP_ERROR_THRESHOLD,
    REG_THRESHOLD_HI, (NUM_CHIPS - CHIP_ERROR_THRESHOLD),
    REG_ANALOG_CTL, (AGC_RSSI_CTL | PACTL_EN),
    REG_PA, (DEFAULT_POWER_LEVEL & PA_BIAS_MASK),
#ifdef ADJUST_XTAL
    REG_XTAL_ADJ, (CLOCK_DISABLE | (DEFAULT_XTL & XTAL_ADJ_MASK)),
#else
    REG_XTAL_ADJ, (CLOCK_DISABLE),              // disable 13MHz clock output
#endif
    REG_VCO_CAL, MINUS_5_PLUS_5,                 // use the laser trimmed VCO value
    REG_AGC_CTL, AGC_OFF,                       // turn off AGC circuitry to reduce power consumption
#ifdef RADIO_USE_RSSI
    REG_CARRIER_DETECT, CDET,                   // force carrier detect (automatically triggers an RSSI reading)
#endif    
    REG_CLOCK_ENABLE, 0x41,                     // set per datasheet
    REG_CLOCK_MANUAL, 0x41,                     // set per datasheet
    REG_SYN_LOCK_CNT, SYNTH_SETTLE_COUNT       // time to wait for the synth to settle
};


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_init
//
//!  Description: After a reset and power cycle, the crystal is given time to
//!               settle and then it comes out of reset and then the radio is
//!               configured
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////

void radio_init()
{
    UINT8 i;
    
    RADIO_PORT &= ~(N_PD | N_RESET);  
    RADIO_PORT |= N_PD; 
    TIMER_DELAY_MSEC(RADIO_WAKEUP_TIME * 2);    // crystal settle time

    spi_init();
    RADIO_PORT |= (N_RESET);                // bring radio out of reset...

    // load the configuration data
    for(i = 0; i < sizeof(radio_init_data); i += 2)
    {
        spi_radio_put(radio_init_data[i] , radio_init_data[i + 1]);
    }
 }

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    radio_receive_on
//
//!  Description: Interrupts are disabled and the overflow, channel_EOF, and 
//!               RX data length are all initialized, if RADIO_NON_BLOCKING_CALLS
//!               is defined.  Then sets the radio up to receive.  Enables 
//!               interrupts, if necessary
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_receive_on(void)
{
#ifdef RADIO_NON_BLOCKING_CALLS
    RADIO_ISR_DISABLE();
    overflow     = 0;
    channel_EOF   = 0;  
    rx_data_length = 0;
#endif // RADIO_NON_BLOCKING_CALLS
    
    
    // clear old data and interrupts
    spi_radio_get(REG_DATA_RX_A);
    spi_radio_get(REG_RX_INT_STAT);
    spi_radio_put(REG_CONTROL, (RX_ENABLE | AUTO_SYNTH_COUNT)); 
    
#ifdef RADIO_NON_BLOCKING_CALLS
    RADIO_ISR_ENABLE();
#endif
}

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    radio_transmit_on
//
//!  Description: Sets up the radio to transmit mode.  Disables interrupts if
//!               RADIO_NON_BLOCKING_CALLS is defined.  Interrupts should 
//!               always be disabled while transmitting packets because it is
//!               polled driven. The device knows when it will transmit a packet.
//!               Interrupts should only be used when receiving packets because
//!               the device doesn't know when it receives data, thus more power
//!               is conserved.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_transmit_on(void)
{
#ifdef RADIO_NON_BLOCKING_CALLS
    RADIO_ISR_DISABLE();             // disable ISR
#endif    
    // enable transmit mode
    spi_radio_put(REG_CONTROL, (TX_ENABLE | AUTO_SYNTH_COUNT)); 
}

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    radio_off
//
//!  Description: Turns off transmit and receive modes
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_off(void)
{                                                                     
#ifdef RADIO_NON_BLOCKING_CALLS
    RADIO_ISR_DISABLE();                                              
#endif
    spi_radio_put(REG_CONTROL, 0);
}

//////////////////////////////////////////////////////////////////////////////
//
//   Function:    radio_set_channel
//
//!  Description: Sets the radio channel by passing the channel register 
//!               (REG_CHANNEL), and a valid channel to spi_radio_put().  
//!               CHANNEL_OFFSET is used for 2 MHz guard band. (See radio.h 
//!               for CHANNEL_OFFSET definition) CHANNEL_MASK is to ensure
//!               validity of channel.
//!
//!  Inputs:      
//!               - channel - channel to set the radio to
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////  
void radio_set_channel(UINT8 channel)
{
    spi_radio_put(REG_CHANNEL, (CHANNEL_MASK & (channel + CHANNEL_OFFSET)));
}

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    radio_set_pn_code
//
//!  Description: Load the PN Code. For more information on PN codes please see
//!               the LS Theory of Operation app note.
//!
//!  Inputs:      
//!               - pn_code - pointer to the 8-byte PN code
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_set_pn_code(const UINT8 *pn_code)
{
    UINT8 i;

    for(i = 0; 7 >= i; i++)
    {
        spi_radio_put(REG_PN_CODE + i, *(pn_code + i));
    }
}


///////////////////////////////////////////////////////////////////////////////
//
//   Function:    radio_transmit
//
//!  Description: Send a packet to the radio, one byte at a time. 
//!               If the radio is not already in transmit mode, 
//!               radio_transmit_on() will be called before transmitting data 
//!               and the radio will be turned off ( radio_off() ) after the  
//!               packet transmission is complete.
//!
//!  Inputs:      
//!               - len     - length of packet
//!               - *data   - pointer to the packet
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_transmit(UINT8 buffer_length, UINT8 *tx_buffer)
{
    UINT8 i;
    UINT8 counter;
    UINT8 radio_state;

    radio_state = (spi_radio_get(REG_CONTROL));
    if ((radio_state & TX_ENABLE) != TX_ENABLE)
    {
        radio_transmit_on();
    }
    i = (spi_radio_get(REG_CONTROL));

    spi_radio_put(REG_TX_INT_EN, TX_EMPTY);
    // send the packet
    for(i = 0; i < buffer_length; ++i, ++tx_buffer) 
    {
        for (counter = 0; (counter < DEADMAN_MAX_COUNT) && (!LS_IRQ_ASSERTED); 
             counter++); //dead man timeout

        spi_radio_put(REG_DATA_TX, *tx_buffer);
    }

    // enable the EOF interrupt and disable the EMPTY interrupt
    spi_radio_put(REG_TX_INT_EN, TX_EOF);

    TIMER_DELAY_50_USEC();
    TIMER_DELAY_50_USEC();
    
    for (counter = 0; (counter < DEADMAN_MAX_COUNT) && (!LS_IRQ_ASSERTED); 
          counter++); //dead man timeout

    if ((radio_state & TX_ENABLE) != TX_ENABLE)
    {
        radio_off();
    }
}

#ifndef RADIO_NON_BLOCKING_CALLS     
///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_receive
//
//!  Description: If the radio is not already in receive mode radio_receive_on()
//!               is called.  Data is received from the radio until an end-of-
//!               frame (EOF) interrupt is set or a timeout is reached
//!
//!  Inputs:      
//!               - buffer_length - Maximum number of bytes that can be received
//!                               before a buffer overflow occurs.
//!               - *rx_buffer    - Pointer to the buffer to store received data.              
//!               - *valid_buffer - Pointer to buffer to store received valid bytes.
//!               - timeout       - Length of time to be in receive mode before timing
//!                               out. This value is in 10us units.  Once a byte
//!                               is received the function will not return until
//!                               the EOF interrupt is received, or a buffer overflow
//!                               occurs, in order to prevent the function from
//!                               returning in the middle of receiving a packet.
//!
//!  Returns:     
//!               - rx_data_length - Length of packet received 
//!               - 0 - Otherwise
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 radio_receive(UINT8 buffer_length, UINT8 *rx_buffer, UINT8 *valid_buffer, UINT16 timeout)
{
    UINT8  irq_source;
    UINT8  radio_state;
    UINT16 loop_counter = 0;

    radio_state = (spi_radio_get(REG_CONTROL));
    if ((radio_state & RX_ENABLE) != RX_ENABLE)
    {
        radio_receive_on();
    }
    rx_data_length = 0;    

    // don't break out of the loop mid-packet..
    while ((loop_counter < timeout) || (rx_data_length != 0))    
    {
        // poll for a Radio Module interrupt, interrupts are self clearing
        // when the appropiate status register is read 
        if(LS_IRQ_ASSERTED)
        {   
            // read the Radio Rx Interrupt register
            irq_source = spi_radio_get(REG_RX_INT_STAT);
            
            if(irq_source & RX_FULL_A)
            {
                *rx_buffer = spi_radio_get(REG_DATA_RX_A);
                
                // don't read the valid register if it is all valid
                if( irq_source & RX_VALID_A )
                {
                    *valid_buffer = 0xFF;
                }
                else
                {
                    *valid_buffer = spi_radio_get(REG_VALID_RX_A);
                }
            
                // filter out bad bytes and don't overwrite the globals
                if((*valid_buffer >= 0x0F) || (rx_data_length > 0))
                {
                    if (rx_data_length < buffer_length)
                    {
                        rx_data_length++;
                        loop_counter = 0;

#ifdef RADIO_CHECK_FOR_ACK        
                        if (((data & 0xF0) == 0x50) && (rx_data_length == ACK_LEN)) 
                        {
                            break;
                        }
#endif
                        rx_buffer++;
                        valid_buffer++;
                    }
                    else
                    {
                        // buffer overflow... exit receive mode
                        rx_data_length = 0;
                        break;
                    }
                }
            }
                
            if(irq_source & RX_EOF_A)
            {               
                if (rx_data_length != 0)
                {
                    break;
                }
            }
            if(irq_source & RX_OVER_A)
            {
                loop_counter = 0;
            }            
        }   //if(LS_IRQ_ASSERTED)
        else
        {
            ++loop_counter;            // only increment loop_counter for each 10us delay...
            TIMER_DELAY_10_USEC();
        }
    } // while(loop_counter...)    
    
    if ((radio_state & RX_ENABLE) != RX_ENABLE)
    {
        radio_off();
    }

    return rx_data_length;
}

#else // RADIO_NON_BLOCKING_CALLS

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    radio_receive
//
//!  Description: If the radio is not already in receive mode radio_receive_on()
//!               is called.  Data is received from the radio until an end-of-
//!               frame (EOF) interrupt is set or a timeout is reached. 
//!
//!               - This routine utilizes the radio_rx_isr to handle receiving
//!               each byte of data from the radio from interrupt context.
//!
//!  Inputs:      
//!               - buffer_length - Maximum number of bytes that can be received
//!                               before a buffer overflow occurs.
//!               - *rx_buffer    - Pointer to the buffer to store received data.
//!               - *valid_buffer - Pointer to buffer to store received valid bytes.
//!               - timeout       - Length of time to be in receive mode before timing
//!                               out. This value is in 10us units.  Once a byte
//!                               is received the function will not return until
//!                               the EOF interrupt is received, or a buffer overflow
//!                               occurs, in order to prevent the function from
//!                               returning in the middle of receiving a packet.
//!
//!  Returns:     
//!               - rx_data_length - Length of packet received (if an EOF occurs) 
//!               - 0 - Otherwise
//!            
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 radio_receive(UINT8 buffer_length, UINT8 *rx_buffer, UINT8 *valid_buffer, UINT16 timeout)
{
    UINT16 loop_counter;
    UINT8  radio_state;
    
    rx_buffer_length = buffer_length;
    rx_data_buffer = rx_buffer;
    rx_valid_buffer = valid_buffer;


    radio_state = (spi_radio_get(REG_CONTROL));
    if ((radio_state & RX_ENABLE) != RX_ENABLE)
    {
        radio_receive_on();
    }
    else if (channel_EOF == 1)        // if already in receive mode and already received EOF
    {
        return rx_data_length;
    }

    // do NOT break out of receive mode if radio is in the middle of receiving a packet...
    for(loop_counter = 0; ((loop_counter < timeout) || (rx_data_length > 0)); ++loop_counter)            
    {
        if (channel_EOF == 1)
        {
            break;
        }
        TIMER_DELAY_10_USEC();  
    }
    if ((radio_state & RX_ENABLE) != RX_ENABLE)
    {
        radio_off();
    }

    if (channel_EOF == 1)
    {
        return rx_data_length;
    }
    else
    {
        return 0;
    }
}


///////////////////////////////////////////////////////////////////////////////
//
//   Function:    radio_rx_isr
//
//!  Description: This routine reads the register for the RX interrupt 
//!               (REG_RX_INT_STAT).  If the receive register is full (RX_FULL_A),
//!               and all bits aren't valid, it will read the valid register. 
//!               Then the bad bits are filtered out and the length of the packet
//!               is figured and stored globally.  If an EOF event occurred on
//!               the Channel A receive(RX_EOF_A) and the global overflow flag
//!               is set, then the data is dumped. If the global overflow flag 
//!               is not set, the global channel_EOF flag is set
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_rx_isr()
{
    UINT8 irq_source;
    UINT8 data;
    UINT8 valid;
    
    // poll for a Radio Module interrupt, interrupts are self clearing
    // when the appropriate status register is read 
    while(LS_IRQ_ASSERTED)
    {   
        // read the Radio Rx Interrupt register
        SPI_ISR_RADIO_GET(REG_RX_INT_STAT, irq_source);
        
        if ((irq_source & 0x0F) == 0)
        {
            break;
        }

        if(irq_source & RX_FULL_A)
        {
            SPI_ISR_RADIO_GET(REG_DATA_RX_A, data);
            
            // don't read the valid register if it is all valid
            if( irq_source & RX_VALID_A )
            {
                valid = 0xFF;
            }
            else
            {
                SPI_ISR_RADIO_GET(REG_VALID_RX_A, valid);
            }
        
            // filter out bad bytes and don't overwrite the globals
            if( (0x0F < valid) && (channel_EOF != 1) )
            {
#ifdef RADIO_CHECK_FOR_ACK        // Power Savings Initiative
                if (((data & 0xF0) == 0x50) && (rx_data_length == (ACK_LEN-1))) 
                {
                    channel_EOF = 1;    
                    // shut the radio off like we used too??
                }
#endif // RADIO_CHECK_FOR_ACK

                if (rx_data_length < rx_buffer_length)
                {
                    *((UINT8 *)(rx_data_buffer + rx_data_length)) = data;
                    *((UINT8 *)(rx_valid_buffer + rx_data_length)) = valid;
                    rx_data_length++;
                }
                else
                {
                    overflow = 1;
                }
            }
        }
            
        if(irq_source & RX_EOF_A)
        {               
    
            if( overflow == 0 )
            {

                // only handle good packets  // logic for master ??
                if (rx_data_length != 0) 
                {
                    channel_EOF = 1;    
                }   
            }
            else // overflow packet, dump it
            {
               overflow = 0;
               rx_data_length = 0;
            }
        }   
    }   //while(LS_IRQ_ASSERTED)
}
#endif // RADIO_NON_BLOCKING_CALLS

#ifdef RADIO_SLEEP_ENABLED

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    radio_sleep
//
//!  Description: Powers down the LS radio turns on the pull up for the MISO 
//!               to conserve power.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!            
//!
///////////////////////////////////////////////////////////////////////////////
void radio_sleep(void)
{
#ifdef ADJUST_XTAL
    // Set crystal to zero
    radio_set_crystal(0x00);
#endif

    // Power down radio
    RADIO_PORT &= ~N_PD;

    // Turn on pull up for MISO to reduce current
    PRT1DM1 = PORT_1_DRIVE_1 & ~MISO_MASK;
}

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    radio_wakeup
//
//!  Description: Wakes the radio up from sleep mode turning on the MISO high-z,
//!               powering up (N_PD) the radio, and allowing some delay for wakeup
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!            
//!
///////////////////////////////////////////////////////////////////////////////
void radio_wakeup(void)
{   
    // Turn high-z back on for MISO
    PRT1DM1 = PORT_1_DRIVE_1;

    // Power up radio
    RADIO_PORT |= N_PD;

    TIMER_DELAY_MSEC( RADIO_WAKEUP_TIME );  

}



#endif // RADIO_SLEEP_ENABLED

#ifdef ADJUST_XTAL
///////////////////////////////////////////////////////////////////////////////
//
//   Function:    radio_set_crystal
//
//!  Description: Sets the crystal adjustment, which allows the crystal to be
//!               fine tuned. Normally this is not required if 30ppm or less
//!               crystals are used.
//!
//!  Inputs:      
//!               - xtal - crystal adjustment value.
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_set_crystal(UINT8 xtal)
{
    // set the crystal adjustment value 
    spi_radio_put(REG_XTAL_ADJ,  (CLOCK_DISABLE | (xtal & XTAL_ADJ_MASK)));
}
#endif  // ADJUST_XTAL


#ifdef RADIO_USE_MID
///////////////////////////////////////////////////////////////////////////////
//
//   Function:    radio_get_mid
//
//!  Description: Sets radio up and reads the radio's Manufacturing ID (MID).
//!               Then the radio is set up to not read the MID
//!
//!  Inputs:      
//!               - *mid - pointer to a 4-byte buffer to store the MID 
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_get_mid(UINT8 *mid)
{
    // enable MID
    spi_radio_put( REG_ANALOG_CTL, AGC_RSSI_CTL | PACTL_EN | MID_READ_EN );
    
    // read MID
    *mid = spi_radio_get(REG_MFG_ID_1);
    mid++;
    *mid = spi_radio_get(REG_MFG_ID_2);
    mid++;
    *mid = spi_radio_get(REG_MFG_ID_3);
    mid++;
    *mid = spi_radio_get(REG_MFG_ID_4);
    
    // disable MID
    spi_radio_put( REG_ANALOG_CTL, AGC_RSSI_CTL | PACTL_EN );
}
#endif // RADIO_USE_MID

#ifdef RADIO_USE_RSSI
///////////////////////////////////////////////////////////////////////////////
//
//   Function:    radio_get_rssi
//
//!  Description: Sets radio up and reads Reads the latest RSSI value from
//!                the radio and makes sure it is valid 
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - rssi - Valid RSSI reading from radio
//!               - 0 - otherwise
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 radio_get_rssi(void)
{
    UINT8 rssi;
    
    spi_radio_get(REG_RSSI);
    rssi = spi_radio_get(REG_RSSI);
    
    // check for valid RSSI
    if ((rssi & RSSI_VALID) == 0)
    {
        return 0;
    }
    
    // mask off the valid bit and the reserved bits
    rssi &= RSSI_MASK;
    
    return rssi;
}
#endif // RADIO_USE_RSSI
