//--------------------------------------------------------------------------
//! \file
//!
//! Collection of master protocol and bind prototypes.
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/Radio/master_protocol.c $
// $Modtime: 3/04/04 2:46p $
// $Revision: 1.6 $
//--------------------------------------------------------------------------
//
// Copyright 2003, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

//--------------------------------------
// Included files
//--------------------------------------


#include "master_protocol.h"
#include "radio.h"
SYS_PARAMETERS sys_params;

//! Receive packet
LS_PACKET      rx_packet;

//! Transmit packet
LS_PACKET      tx_packet;

//! Extra transmit packet for ack/data packet
LS_PACKET      tx_packet2;


//! Transmit packet number
UINT8          tx_packet_num;


//! Channel valid array for data packets
UINT8 channel_valid[LS_RX_PACKET_LEN];

//! Data toggle
UINT8 data_toggle;

//! Receive data toggle
UINT8 rx_data_toggle;  


//! Time stamp
TIME_STAMP time_stamp;

//! Corrupt counter for finding corrupt bits
UINT8  corrupt_counter = 0;

//! Receive signal strength indicator counter
UINT8  rssi_counter = 0;

//! Data length for receive data
extern UINT8 rx_data_length;

//! Flag to know whether or not the RSSI was checked
UINT8  rssi_checked = FALSE;


//! Macro for the device type
#define protocol_device_type DEVICE_TYPE

//! Data byte for ping packet
#define PING_DATA 0x31;

//! When pinging for normal reasons
#define PING_REASON_NORMAL    0x00

//! When pinging for RSSI reasons
#define PING_REASON_RSSI      0x01

//! When pinging for corrupt packet reasons
#define PING_REASON_CORRUPT   0x02


//! Noise threshold for the receive signal strength indicator
#define RSSI_NOISE_THRESHOLD  0x08

//! Used as the max in the main control loop in master_ping_mode()
#define PING_NUM_RSSI		  20

//! There are 2 channels per RSSI increment
#define CHANNELS_PER_RSSI_INCREMENT 0x02

//! Timeout when checking RSSI
#define QUICK_RSSI_TIMEOUT        0x02

//! Number of high RSSI readings to limit the RSSI counter
#define NUM_HIGH_RSSI_READINGS    0x05

//! RSSI toggle timeout to make sure radio is ready to setup in data mode
#define RSSI_TOGGLE_TIMEOUT       48

//! PN codes
const UINT8 pn_code_table[] = 
{
    0x6A, 0xE7, 0x01, 0xEA, 0x03, 0xFD, 0x13, 0xD2, 
    0xDC, 0xC0, 0x6B, 0xB8, 0x2B, 0x09, 0xBB, 0xB2, 
    0xA3, 0x1E, 0xF2, 0xA4, 0x31, 0x32, 0x7A, 0xB3, 
    0x44, 0x83, 0x3B, 0xDD, 0x14, 0xCF, 0x8E, 0xC9, 
    0x35, 0x35, 0x4E, 0xC5, 0xF3, 0x52, 0x47, 0xB0, 
    0x7C, 0x23, 0x8A, 0xCE, 0x45, 0x5C, 0x54, 0xD7, 
    0x81, 0xAC, 0xFB, 0x83, 0x7A, 0x9A, 0x61, 0xAC, 
    0x3C, 0x12, 0x5F, 0x9C, 0x39, 0x98, 0xF6, 0x8A, 
};


//--------------------------------------
// Local Function Declarations
//--------------------------------------

extern UINT8 app_load_packet( UINT8 *packet_num );
extern UINT8 app_unload_packet( UINT8 rxLength );



///////////////////////////////////////////////////////////////////////////////
//
//   Function:    master_init
//
//!  Description: Calls radio init, and figures device parameters using 
//!               algorithms to find PN code, channel, pin, and checksum seed  
//!               (based off the MID).  The PN code data toggles, and time stamp 
//!               get set.  Then makes sure a channel is found at startup
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - LS_STATUS_SUCCESS - for successful initialization
//!
///////////////////////////////////////////////////////////////////////////////

UINT8 master_init(void)
{
    // configure the Radio
    radio_init();
           
    //Always get the device parameters, this is how the DVK bridge behaves
    {
        
        // read the radio ID, reusing existing global memory
        radio_get_mid(&sys_params.bridge_mid.mid_1);
        
        // hash out values for PN Code, Base Channel, and Network 
        // PIN using the same algorithm as the bridge
        sys_params.network_id.pn_code = (sys_params.bridge_mid.mid_1 << 2) + 
                                        sys_params.bridge_mid.mid_2 + 
                                        sys_params.bridge_mid.mid_3;
        while( sys_params.network_id.pn_code >= NUM_PN_CODES )
        {
            sys_params.network_id.pn_code -= NUM_PN_CODES;
        }
    
        sys_params.network_id.channel = (sys_params.bridge_mid.mid_2 >> 2) -
                                        (sys_params.bridge_mid.mid_1 << 5) + 
                                        sys_params.bridge_mid.mid_3;
        while( sys_params.network_id.channel >= NUM_CHANNELS )
        {
            sys_params.network_id.channel -= NUM_CHANNELS;
        }
        sys_params.network_id.pin = (((sys_params.bridge_mid.mid_1 - 
                                       sys_params.bridge_mid.mid_2) & 
                                      PIN_MASK) + 
                                     MIN_PIN);
        
        sys_params.network_id.seed = ((sys_params.bridge_mid.mid_2 >> 6)) +
                                     sys_params.bridge_mid.mid_1 + 
                                     sys_params.bridge_mid.mid_3; 
    }

	radio_set_pn_code(&pn_code_table[(8 * sys_params.network_id.pn_code)]);
    
    data_toggle = 1;
    rx_data_toggle = 1;

    time_stamp = TIMER_GET_TIME_STAMP();

    // always find a channel on startup
    master_ping_mode(PING_REASON_NORMAL);
              
    return LS_STATUS_SUCCESS;
}

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    master_ping_mode
//
//!  Description: Sets the channel/pn_code and parameters for the ping, 
//!               transmits a ping packet and turns on receive mode to receive 
//!               the acknowledge, checks RSSI and resets it along with the 
//!               corrupt counter
//!
//!  Inputs:      
//!               - reason - to check if we need to start with the top channel
//!
//!  Returns:     
//!               - LS_STATUS_SUCCESS - Successful ping
//!
///////////////////////////////////////////////////////////////////////////////

UINT8 master_ping_mode(UINT8 reason)
{
    UINT8  packet_received;   
    UINT8  rssi; 
    UINT8  ping_channel_count = 0;
    UINT8  rssi_threshold = RSSI_NOISE_THRESHOLD;
    UINT8  ping_count = 0;
    

    // Ping Operation

    // set the channel and listen for activity, select another channel
    // if the channel is taken  
 
    // use a ping request to find another bridge on the channel, also
    // check for other device/bridge "activity" on the channel
       
    // format the ping packet
    tx_packet.first.byte      = PING_DATA;   // 0x31;
                   

    if (reason == PING_REASON_NORMAL)
    {
        // reinit to top channel
        init_channel();
    }
    else
    {
        // get next channel
        get_next_channel();
    }
    
    radio_set_channel(sys_params.network_id.channel);
	radio_set_pn_code(&pn_code_table[(8 * sys_params.network_id.pn_code)]);

    // send a ping request and wait for response, if no response change
    // the channel and try again

    // setup TX loop control variables
    packet_received = NO_PACKET;

    while ( ping_count++ < PING_NUM_RSSI )
    {
        radio_transmit_on();
    	radio_transmit(PING_LEN, (UINT8 *)&tx_packet);

        radio_receive_on();
        if(radio_receive(LS_RX_PACKET_LEN, (UINT8 *)&rx_packet, (UINT8 *)&channel_valid, ACK_TIMEOUT) > 0)
        {
            packet_received = protocol_verify_packet();
        }
    
        // look for response
        if(packet_received == PING_PACKET) 
        {  
            if( rx_packet.ping.hdr.flag == PING_RESP )
            {
                // Bridge found 
                #ifndef PING_NO_HOP 
                get_next_channel();
                ping_channel_count++;
                ping_count = 0;
                #endif
            }
        }
        else 
        {
            // check RSSI
            rssi = protocol_check_rssi();
            if (rssi > rssi_threshold)
            {
                get_next_channel();
                ping_channel_count++;
                ping_count = 0;
            }
        }
        if (ping_channel_count > CHANNELS_PER_RSSI_INCREMENT)
        {
            rssi_threshold++;
            rssi_threshold &= RSSI_MASK;
            ping_channel_count = 0;
        }
    }    // while (ping_count < PING_NUM_RSSI)

    rssi_counter = 0;
    corrupt_counter  = 0;

    radio_off();
    radio_receive_on();
   
    return LS_STATUS_SUCCESS;
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    master_bind_mode
//
//!  Description: Initially sets the PN code.  Continuously sets the channel, 
//!               looks for a bind request, formats and transmits the bind 
//!               packet, sets up to receive an acknowledge.  Upon a good 
//!               acknowledge, it binds to the device and returns.  It checks 
//!               for a timeout, and if it occurs then it will return.  Before 
//!               any return it goes into ping mode
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - LS_STATUS_SUCCESS - successful bind 
//!               - LS_STATUS_FAILED - unsuccessful bind
///////////////////////////////////////////////////////////////////////////////

UINT8 master_bind_mode(void)
{
    UINT8  status = LS_STATUS_FAILED;
    UINT8  channel_count = 0;
    UINT8  tx_timeout;
    UINT8  packet_received;
    UINT8  bind_channel = BIND_BASE_CHANNEL;
        
    // load the bind pn_code index
	radio_set_pn_code(&pn_code_table[(8 * BIND_PN_CODE)]);

    // setup TX loop control variables
    tx_timeout = 0;

    while( 1 )
    {      
	 	radio_set_channel(bind_channel);
 
        // setup loop control
        packet_received = NO_PACKET;
    
        radio_receive_on();
        if(radio_receive(LS_RX_PACKET_LEN, (UINT8 *)&rx_packet, (UINT8 *)&channel_valid, BIND_TIMEOUT) > 0)
        {
            packet_received = protocol_verify_packet();
        }
        
        if(( packet_received == BIND_REQ ) && (rx_data_length == BIND_REQ_LEN))
        {  
           
            if( rx_packet.bind_request.hdr.dev_type == DEVICE_TYPE )
            {
                radio_transmit_on();   
                
                // Send a Bind response
                // format the bind response packet
                tx_packet.first.byte         = 0;   // clear the packet
                tx_packet.bind_response.hdr.type   = BIND_RESP; 
                tx_packet.bind_response.hdr.pin    = sys_params.network_id.pin;
                tx_packet.bind_response.hdr.parity = protocol_gen_parity(tx_packet.first.byte);
				tx_packet.bind_response.mid_1 = sys_params.bridge_mid.mid_1;
				tx_packet.bind_response.mid_2 = sys_params.bridge_mid.mid_2;
				tx_packet.bind_response.mid_3 = sys_params.bridge_mid.mid_3;
				tx_packet.bind_response.mid_4 = sys_params.bridge_mid.mid_4;

                tx_packet.bind_response.checksum    = protocol_calc_ck_sum((UINT8 *)&tx_packet.bind_response,
                                               BIND_RESP_LEN-1, BIND_SEED);
    
                // send the response      
    	    	radio_transmit(BIND_RESP_LEN, (UINT8 *)&tx_packet);
                
                // set the radio to receive mode        
                radio_receive_on(); 
                if(radio_receive(LS_RX_PACKET_LEN, (UINT8 *)&rx_packet, (UINT8 *)&channel_valid, ACK_TIMEOUT) > 0)
                {
                    packet_received = protocol_verify_packet();
                }
                
                // check for a good ACK
                if(( packet_received == ACK_PACKET ) && (rx_data_length == ACK_LEN))
                { 
                    if( rx_packet.ack.hdr.flag == GOOD_ACK )
					{
                        // ack received, exit bind loop                        
                        status = LS_STATUS_SUCCESS;
                        break;
                    }  // if( rx_packet.ack.hdr.flag == GOOD_ACK )
                }   // if( packet_received == ACK_PACKET )
            }   
        }  // BIND_REQ
        
        // check for Bind timeout after listening on each channel 5 times
        else if( tx_timeout > 5 * NUM_CHANNELS_PER_SUBSET )
        {
            break;
        }
        else if( channel_count++ == 5 )
        {
            // timeout, bump channel
            #ifndef BIND_NO_HOP
            bind_channel += 6;
            if (bind_channel >= NUM_CHANNELS)
            {
            	bind_channel -= NUM_CHANNELS;
            }            
            #endif
            channel_count = 0;
            tx_timeout++;
            radio_off();
        }         
    }  // while( 1 )       
    
    master_ping_mode(PING_REASON_NORMAL);
 
    return status;
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    master_data_mode
//
//!  Description: Checks for idle data and receives a packet.  If no packet 
//!               is received then the radio is turned off.  If a packet is 
//!               received it transmits a response and checks for a receive 
//!               response.  While checking for a receive response, it first 
//!               checks for a regular data packet (lets the app know if there 
//!               is one), then either creates and sends an ack/data packet 
//!               or an ack packet.  If it doesn't check for a receive response 
//!               it checks if the packet is a connect request with an approved 
//!               device or if it is a ping packet and formats and transmits 
//!               the packet. If it is neither of those, it checks to see if it 
//!               a good acknowledge packet and sets up the return value.  If it 
//!               isn't a packet it checks the RSSI
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - DATA_RECEIVED - data packet
//!               - ACK_RECEIVED - valid acknowledge packet
//!               - DATA_IDLE - idle data
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 master_data_mode( void )
{
    UINT8  tx_timeout;	
    UINT8  packet_received = NO_PACKET;
    UINT8  status = DATA_IDLE;
    UINT8  tx_length = 0;
    
    
    if(radio_receive(LS_RX_PACKET_LEN, (UINT8 *)&rx_packet, (UINT8 *)&channel_valid, 0) != 0)
    {
        packet_received = protocol_verify_packet();
        
        if (packet_received != NO_PACKET)
        {
            // get ready to transmit a response
            radio_transmit_on();
        }
        else                     // invalid and/or corrupt packet
        {
            radio_off();        // radio must toggle in/out of receive mode for the RSSI...
        }
    
        // check for receive responses
        if(( packet_received == DATA_PACKET ) || ((packet_received == NULL_PACKET) && (rx_data_length == NULL_LEN)))
        {
            if ((packet_received == DATA_PACKET) && (rx_packet.data.hdr.dt_data == (rx_data_toggle & 0x01)))
            {
                rx_data_toggle++;
                // let App know a data packet was received
                status |= DATA_RECEIVED;
                app_unload_packet(rx_data_length-2);    // strip header and checksum
            }
    
            // check for back channel data
            tx_length = app_load_packet(&tx_packet_num);    
            if( tx_length > 0 )
            {
                if (tx_packet_num == 1)
                {
                    // create ACK/Data packet
                    tx_packet.first.byte = 0;
                    tx_packet.ack_data.hdr.type = DATA_ACK_PACKET;
                    tx_packet.ack_data.hdr.flag = GOOD_ACK; 
               
                    // the RX data toggle field is being mirrored back in the ACK
                    tx_packet.ack_data.hdr.dt_ack = rx_packet.data.hdr.dt_data;
                    tx_packet.ack_data.hdr.dt_data = data_toggle;
                    tx_packet.ack_data.hdr.parity = protocol_gen_parity( 
                                                 (UINT8) tx_packet.first.byte );        
                    
                    // calculate check sum
                    tx_packet.ack_data.app_packet[tx_length] = 
                               protocol_calc_ck_sum((UINT8 *)&tx_packet, tx_length+1,
                                                     sys_params.network_id.seed);  
                               
                    // send the packet, add header and checksum to length 
        	        radio_transmit( tx_length + 2, (UINT8 *)&tx_packet );    
                }
                else
                {
                    // create ACK/Data packet
                    tx_packet2.first.byte = 0;
                    tx_packet2.ack_data.hdr.type = DATA_ACK_PACKET;
                    tx_packet2.ack_data.hdr.flag = GOOD_ACK; 
               
                    // the RX data toggle field is being mirrored back in the ACK
                    tx_packet2.ack_data.hdr.dt_ack = rx_packet.data.hdr.dt_data;
                    tx_packet2.ack_data.hdr.dt_data = data_toggle;
                    tx_packet2.ack_data.hdr.parity = protocol_gen_parity( 
                                                 (UINT8) tx_packet2.first.byte );        
                    
                    // calculate check sum
                    tx_packet2.ack_data.app_packet[tx_length] = 
                               protocol_calc_ck_sum((UINT8 *)&tx_packet2, tx_length+1,
                                                     sys_params.network_id.seed);  
                               
                    // send the packet, add header and checksum to length 
        	        radio_transmit( tx_length + 2, (UINT8 *)&tx_packet2 );    
                }
            } 
            else    // send an ACK for the data packet
            {                
                // create ACK packet
                tx_packet.first.byte = 0;
                tx_packet.ack.hdr.type = ACK_PACKET;
                tx_packet.ack.hdr.flag = GOOD_ACK; 
           
                // the RX data toggle field is being mirrored back in the ACK
                tx_packet.ack.hdr.dt_ack = rx_packet.data.hdr.dt_data;
    
                tx_packet.ack.hdr.parity = protocol_gen_parity( 
                                         (UINT8) tx_packet.first.byte );        
        
                // send the packet
    	        radio_transmit( ACK_LEN, (UINT8 *)&tx_packet );    
            }
    
        }    // if( packet_received == DATA_PACKET)
                
        // check for an "approved" device trying to connect
        else if(( packet_received == CONNECT_REQ ) && (rx_data_length == CONNECT_REQ_LEN))
        {
            // save the connect data and reply
       
            if( (sys_params.bridge_mid.mid_1 == rx_packet.connect_request.mid_1) &&  
                (sys_params.bridge_mid.mid_2 == rx_packet.connect_request.mid_2) &&  
                (sys_params.bridge_mid.mid_3 == rx_packet.connect_request.mid_3) &&  
                (sys_params.bridge_mid.mid_4 == rx_packet.connect_request.mid_4)  )
            {
        
                // Send a Connect response
                
                // format the connect response packet
                tx_packet.first.byte      = 0;   // clear the packet
                tx_packet.ping.hdr.type   = CONNECT_RESP; 
                tx_packet.ping.hdr.flag   = ACK_RESP; 
                tx_packet.ping.hdr.parity = protocol_gen_parity(tx_packet.first.byte);
        
                // send the response      
            	radio_transmit(CONNECT_RESP_LEN, (UINT8 *)&tx_packet);
            }
        }    
                       
        else if((packet_received == PING_PACKET) && (rx_data_length == PING_LEN))
        {  
            if( rx_packet.ping.hdr.flag == PING_REQ )
            {
                // Send a Ping response
                
                // format the ping packet
                tx_packet.first.byte      = 0;   // clear the packet
                tx_packet.ping.hdr.type   = PING_PACKET; 
                tx_packet.ping.hdr.flag   = PING_RESP; 
                tx_packet.ping.hdr.parity = protocol_gen_parity(tx_packet.first.byte);
    
                // send the response      
    	    	radio_transmit(PING_LEN, (UINT8 *)&tx_packet);
            }
        }
        else if ((packet_received == ACK_PACKET) && (rx_data_length == ACK_LEN))
        {
            if ((((tx_packet_num == 1) && (tx_packet.data.hdr.type == DATA_ACK_PACKET))
                 || ((tx_packet_num == 2) && (tx_packet2.data.hdr.type == DATA_ACK_PACKET))) 
                 && (rx_packet.ack.hdr.dt_ack == (data_toggle & 0x01)) &&
                 (rx_packet.ack.hdr.flag == GOOD_ACK))
            {
                data_toggle++;
            
                status |= ACK_RECEIVED;
            }
        }
        radio_receive_on();
        time_stamp = TIMER_GET_TIME_STAMP();
        rssi_checked = FALSE;
    }
    // Check the RSSI
    else 
    {
        // need to check RSSI 2 ms after starting receive, or after a packet..
        if ((time_stamp + QUICK_RSSI_TIMEOUT < TIMER_GET_TIME_STAMP()) && (rssi_checked == FALSE) && (rx_data_length == 0))
        {
            UINT8 rssi;
            
            if (protocol_check_rssi() > RSSI_NOISE_THRESHOLD)
            {
                if (rssi_counter > NUM_HIGH_RSSI_READINGS)
                {
                    // need to go to ping mode
                    master_ping_mode(PING_REASON_RSSI);
                }
            }
            rssi_checked = TRUE;
        }
    
        if (((time_stamp + RSSI_TOGGLE_TIMEOUT) < TIMER_GET_TIME_STAMP()) && (rx_data_length == 0))
        {
            radio_off();
            radio_receive_on();
            time_stamp = TIMER_GET_TIME_STAMP();
            rssi_checked = FALSE;
        }
    }
           
    return( status );
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_check_rssi
//
//!  Description: Check the strength of the signal by counting the number of
//!               corrupted bits after the RSSI_NOISE_THRESHOLD has been
//!               reached via the rssi_counter.
//!
//!  Inputs:     Void 
//!
//!  Returns:    
//!               - rssi - rssi value
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 protocol_check_rssi(void)
{
	UINT8 rssi = 0;
	
	// read RSSI
    rssi = radio_get_rssi();
    
	if (rssi > RSSI_NOISE_THRESHOLD)
	{
		rssi_counter++;
	}
	else 
    {
        rssi_counter = 0;
	}
		
	return rssi;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_calc_ck_sum
//
//!  Description: Calculate Packet Checksum
//!
//!  Inputs:      
//!               - *data - Packet pointer
//!               - len - Length of packet
//!               - seed - Checksum seed
//!
//!  Returns:     
//!               - checksum - Checksum calculated from data
//!
///////////////////////////////////////////////////////////////////////////////

UINT8 protocol_calc_ck_sum(UINT8 *data, UINT8 len, UINT8 seed)
{
    UINT8 checksum;
    
    for(checksum = seed; len > 0; len--, data++) 
    {
        checksum ^= *data;
    }        
    return(checksum);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_gen_parity
//
//!  Description: Calculate odd parity for a byte of data
//!
//!  Inputs:      
//!               - data - data byte
//!
//!  Returns:     
//!               - 1 - even number of ones
//!               - 0 - odd number of ones
//!
///////////////////////////////////////////////////////////////////////////////

UINT8 protocol_gen_parity(UINT8 data)
{
    UINT8 count = 0;

    for (; data; )
    {
        data >>= 1; // shift out the parity bit...

        // check for bits that are set
        if (data & 0x01)
        {
           ++count;
        }   
    }    

    // check for an odd number
    if( count & 0x01 )
        return 0;
    else
        return 1;
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_send_ack
//
//!  Description: Formats the acknowledge packet and transmits it
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void protocol_send_ack(void)
{
    // create ACK packet
    tx_packet.first.byte = 0;
    tx_packet.ack.hdr.type = ACK_PACKET;
    tx_packet.ack.hdr.flag = GOOD_ACK; 

    tx_packet.ack.hdr.dev_id = (protocol_device_type & 0x01);

    // the RX data toggle field is being mirrored back in the ACK
    tx_packet.ack.hdr.dt_ack = rx_packet.data.hdr.dt_data;

    tx_packet.ack.hdr.parity = protocol_gen_parity(
                                                  (UINT8) tx_packet.first.byte );        

#ifdef CHECKSUM_ALL_PACKETS
    tx_packet.data.app_packet[0] = 
    protocol_calc_ck_sum((UINT8 *)&tx_packet, 1, sys_params.network_id.seed );
#endif        

    // send the packet
    radio_transmit( ACK_LEN, (UINT8 *)&tx_packet );    
}    


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_verify_packet
//
//!  Description: Gets packet data and makes sure the bytes are protocol 
//!               perfect(otherwise it will return NO_PACKET).  It checks and
//!               fixes packets with checksums  by testing each column of bits 
//!               (See comments and comment block in code source).  It then 
//!               compares checksums with the calculated checksum.
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - 0x00-0x07 - Packet type (See protocol.h for types of packets)  
//!               - NO_PACKET - no packet or bad packet
//!
///////////////////////////////////////////////////////////////////////////////

UINT8 protocol_verify_packet(void)
{   
    UINT8  i, j;
    UINT8  packet_type;
    UINT8  num_erasures;    // track # of erasures per column
    UINT8  checksum;
    UINT8  bad_bit   = 0; 
    UINT8  bad_row;
    UINT8  bad_bit_count = 0;

    
    // get the packet type, it is the upper nibble in the first byte
    packet_type = rx_packet.ack.hdr.type;

    // check for perfect protocol bytes, requirement for bit correction AND
    // blunder check the parity bit  AND 
    // blunder check the packet type AND
    // blunder check the data types by length
    if ( (rx_data_length == 0) || 
         (channel_valid[0] != 0xFF) || 
         (channel_valid[rx_data_length-1] != 0xFF) ||
         (protocol_gen_parity(rx_packet.first.byte) != (rx_packet.first.byte & 0x01)) ||
         (packet_type >= MAX_NUM_TYPES + 1) ||
         ( ((packet_type == DATA_ACK_PACKET) || (packet_type == DATA_PACKET)) && (rx_data_length <= 2) ) || 
         ((packet_type == ACK_PACKET) && (rx_data_length != ACK_LEN)) ) 
    {
        // packet protocol bytes invalid OR
        // bad parity in packet OR
        // bad format in packet
        return( NO_PACKET );
    }
    
    j = rx_data_length;

    // only check/fix packets with checksums
    if(rx_data_length > 1)
    {
        j = rx_data_length;

        // quick test for bad bits
        bad_bit = 0xFF;
        j = rx_data_length;
        for (i = 1; i < rx_data_length; i++)
        {
            j = rx_data_length;

            // AND the column together to find
            // bad bits
            bad_bit &= channel_valid[i];
            j = rx_data_length;
        }
        
        // check for any bad bits
        if (bad_bit != 0xFF)
        {
            ////////////////////////////////////////////////////////////////////
            //
            //  The checksum can only be used to fix packets with a single error
            //  in a column. The checksum is always the last byte in the packet.
            //
            //                        0 1 2 3 4 5 6 7
            //                       +---------------+
            //  ChannelPacket[0]     |x|x|x|x|x|x|x|x|      First Byte 
            //                       +---------------+
            //                       +---------------+
            //  ChannelValid [0]     |1|1|0|0|0|1|1|1|      0=Bad bit  
            //                       +---------------+
            //
            //                       +---------------+
            //  ChannelPacket[1]     |x|x|x|x|x|x|x|x|      Data 
            //                       +---------------+
            //                       +---------------+
            //  ChannelValid [1]     |1|1|1|1|1|1|1|1|      0=Bad bit  
            //                       +---------------+
            //
            //                       +---------------+
            //  ChannelPacket[2]     |x|x|x|x|x|x|x|x|      Checksum  
            //                       +---------------+
            //                       +---------------+
            //  ChannelValid [2]     |1|1|1|1|1|1|1|1|      0=Bad bit  
            //                       +---------------+ 
            //
            ////////////////////////////////////////////////////////////////////             

            // process the data in the Packet by column
            for (i = 0; 7 >= i; i++)
            {
                num_erasures = 0;

                // test for a column with a bad bit
                if (!(bad_bit & (1 << i)))
                {
                    if ( packet_type == BIND_RESP )
                    {
                        checksum = BIND_SEED;
                    }
                    else
                    {
                        checksum = sys_params.network_id.seed;
                    }

                       // Now that we have the bad column, figure out which row
                    // has the bad bit
                    for (j = 0; j < rx_data_length; j++)
                    {
                        // This is only true if bit j of channel_valid is 0, 
                        // an invalid bit
                        if ((channel_valid[j] & (1 << i)) == 0)
                        {
                            bad_row = j;

                            // Using num_erasures to tell when 2 bits in the
                            // same column are bad, but it could be used for 
                            // stats as well...
                            num_erasures++;
                            bad_bit_count++;
                            if ( (num_erasures > 1) || (3 < bad_bit_count) )
                            {
                                 // discard the packet because
                                 // we cannot recover from two
                                 // erased bits in the same column or
                                 // there are too many bad bits in
                                 // general
                                packet_type = NO_PACKET;    
                                break;
                            }
                        }
                        else  // good data, calculate checksum
                        {
                            checksum ^= *((UINT8 *) (&rx_packet.first.byte) + j);
                        }
                    }  // for ()

                    if (packet_type == NO_PACKET)
                    {
                        break;
                    }

                    if (checksum & (1 << i))
                    {
                        *((UINT8 *)(&rx_packet.first.byte) + bad_row) |= (1 << i);
                    }
                    else
                    {
                        *((UINT8 *)(&rx_packet.first.byte) + bad_row) &= 
                        ~(1 << i);
                    }
                } // if ()
            } // for()
        } // if bad_bit != 0xFF

        // compare the checksum....
        if ( packet_type == BIND_RESP )
        {
            checksum = BIND_SEED;
        }
        else
        {
            checksum = sys_params.network_id.seed;
        }
        // Comparing the received checksum with the calculated checksum    
        if (*((UINT8 *) (&rx_packet.first.byte) + ((rx_data_length)-1)) != 
            protocol_calc_ck_sum((UINT8*)&rx_packet.first.byte, 
                                 (rx_data_length)-1, checksum ))
        {
            // bad packet
            packet_type = NO_PACKET;
        }

    } // if packet_type == packet with checksum

    return(packet_type);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_get_tx_pkt
//
//!  Description: Returns a pointer to the radio transmit packet 
//!
//!  Inputs:      Void
//!
//!  Returns:     Void *
//!
///////////////////////////////////////////////////////////////////////////////

void * protocol_get_tx_pkt(void)
{
    return &tx_packet.data.app_packet;
}


#ifdef TWO_WAY
///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_get_rx_pkt
//
//!  Description: Returns a pointer to the radio receive packet 
//!
//!  Inputs:      Void
//!
//!  Returns:     Void *
//!
///////////////////////////////////////////////////////////////////////////////

void * protocol_get_rx_pkt(void)
{
    return &rx_packet.data.app_packet;
}
#endif  // TWO_WAY

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    get_next_channel
//
//!  Description: Performs the channel selection algorithm. Every sixth channel
//!               will be used.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////

void get_next_channel(void)
{
    sys_params.network_id.channel += (6 * (sys_params.network_id.pin + 1));
    
    if((NUM_CHANNELS) - 1 < sys_params.network_id.channel)
    {
        sys_params.network_id.channel -= NUM_CHANNELS;    
    }
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    init_channel
//
//!  Description: Initialize the channel selection algorithm to the top channel
//!               in the channel subset
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////

void init_channel(void)
{
    while (sys_params.network_id.channel < NUM_CHANNELS - 6) 
    {
        sys_params.network_id.channel += 6;
    }
}
