//--------------------------------------------------------------------------
//! \file
//!
//!  Include file which defines the LS radio packet format
//!
//!  NOTE:If you are porting this code to another processor, the bit fields 
//!       may need to be swapped around, and calls to protocol_gen_parity may need to be 
//!       changed also (casting to find the beginning of the header byte).
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/Radio/protocol.h $
// $Modtime: 3/04/04 3:11p $
// $Revision: 1.5 $
//--------------------------------------------------------------------------
//
// Copyright 2003-2004, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

#ifndef _MASTER_PROTOCOL_H_
#define _MASTER_PROTOCOL_H_

//--------------------------------------
// Included files
//--------------------------------------
#include "ls_config.h"

//--------------------------------------
// API Definitions and Types
//--------------------------------------
// Two way acknowledge timeout
#ifndef ACK_TIMEOUT
    #ifdef DATA_RATE_16
	#define ACK_TIMEOUT  50
    #endif
    #ifdef DATA_RATE_32
	#define ACK_TIMEOUT  50
    #endif
    #ifdef DATA_RATE_64
    //! Timeout for the receiving of a packet ( see radio_receive())
	#define ACK_TIMEOUT  50
    #endif
#endif // ACK_TIMEOUT

//! To ping the board for no activity
#define NO_ACTIVITY     0x00

//! To ping the board for activity
#define ACTIVITY        0x01

//! To ping the board for null activity
#define NULL_ACTIVITY   0x02

//! Number of channels in the 2.4 GHz spread spectrum
#define NUM_CHANNELS            78

//! Mask used in the pin distribution algorithm
#define PIN_MASK                0x03  

//! For allowing certain pins to be used in the pin distribution algorithm
#define MIN_PIN                 0x02

//! Number of channels per channel subset
#define NUM_CHANNELS_PER_SUBSET 13


//! Used for a successful bind, ping, or initialization
#define LS_STATUS_SUCCESS   0x00

//! Used when a bind, ping, or initialization fails
#define LS_STATUS_FAILED    0x01

//  Status codes
//! Flag for successful packet transmission
#define DATA_SENT           41

//! Flag for invalid data packet
#define INVALID_DATA        43  

//! Used when the device is no longer bound or a timeout occurred when sending
//! the packet
#define TX_TIMEOUT_ERR      0x05

//! Flag for successful ack/data packet transmission
#define ACK_DATA            0x07

//! Flag for good ack packet
#define GOOD_ACK            0x01


// Packet types
//! Bind Request Packet Type
#define BIND_REQ            0x00

//! Bind Response Packet Type
#define BIND_RESP           0x00

//! Connect Request Packet Type
#define CONNECT_REQ         0x01

//! Connect Response Packet Type
#define CONNECT_RESP        0x02

//! Ping Packet Type
#define PING_PACKET         0x03

//! Data Packet Type
#define DATA_PACKET         0x04

//! Ack Packet Type
#define ACK_PACKET          0x05

//! Data/Ack Packet Type
#define DATA_ACK_PACKET     0x06

//! Null Packet Type
#define NULL_PACKET         0x07
#ifdef ENCRYPT_DATA         

    //! Key Packet Type when encryption is enabled
    #define KEY_PACKET      0x08

    //! Number of packets is 8 when encryption is enabled
    #define MAX_NUM_TYPES   0x08   // needs to point to last used packet type
#else

    //! Number of packets is 7 when encryption is enabled
    #define MAX_NUM_TYPES   0x07   // needs to point to last used packet type
#endif // ENCRYPT_DATA


//! When there is no packet received
#define NO_PACKET           0xFF


//! Ping Request Packet
#define PING_REQ            0x00

//! Ping Response Packet
#define PING_RESP           0x01

//! Flag used in a connect response packet
#define ACK_RESP            0x01


//! Bitmask of idle data for master_data_mode
#define DATA_IDLE        0x00

//! Bitmask of received data for master_data_mode
#define DATA_RECEIVED    0x02

//! Bitmask of a received acknowledge for master_data_mode
#define ACK_RECEIVED     0x04

//! Checksum seed for bind mode and verifying bind response packets
#define BIND_SEED           0x00

//! PN code for bind mode
#define BIND_PN_CODE		0x00

//! Base channel for bind mode
#define BIND_BASE_CHANNEL   0x00

//! Bind timeout when trying to receive a bind packet in bind mode
#define BIND_TIMEOUT        300

//! To setup the radio with a unique channel, PN code, checksum seed, and pin.
//! It's also used for storing the radio manufacturing ID for the bridge.
typedef struct _SYS_PARAMETERS
{
    //! Data packet parameters for the radio and bridge
    struct 
    {

        //! For storing the channel used in the data packet
        UINT8   channel;

        //! For storing the PN code used in the data packet
        UINT8   pn_code;

        //! For storing the checksum seed used in the data packet
        UINT8   seed;

        //! For storing the pin used in the data packet
        UINT8   pin;

        //       	UINT8	baseChannel;
    } network_id;

    #ifdef PROTOCOL_1_1

    //! Radio manufacturing ID for bridge
    struct 
    {
        //! 1st byte of radio manufacturing ID
        UINT8   mid_1;

        //! 2nd byte of radio manufacturing ID
        UINT8   mid_2;

        //! 3rd byte of radio manufacturing ID

        UINT8   mid_3;

        //! 4th byte of radio manufacturing ID

        UINT8   mid_4;

        //UINT8   signature;
    } bridge_mid;
    #endif // PROTOCOL_1_1

} SYS_PARAMETERS;

//! To set radio parameters
extern SYS_PARAMETERS sys_params;

//! Part of the network Id and is used for checking for un-initialized parameters
#define SIGNATURE_BYTE    0x90

//! To set up a data packet
typedef union _LS_PACKET
{
    //! To hold all data of bind response packet
    struct 
    {  
        //! To hold the header data of bind request packet
        struct
        {
            //! Parity bit of bind request packet
            unsigned parity   : 0x01;

            //! Type of device part of bind request packet
            unsigned dev_type : 0x03;   

            //! Type of bind request packet
            unsigned type     : 0x04;   
        } hdr;

        #ifndef PROTOCOL_1_1

        //! 1st byte of radio manufacturing ID
        UINT8   mid_1;

        //! 2nd byte of radio manufacturing ID
        UINT8   mid_2;

        //! 3rd byte of radio manufacturing ID
        UINT8   mid_3;

        //! 4th byte of radio manufacturing ID
        UINT8   mid_4;

        //! Checksum of bind packet being transmitted
        UINT8 checksum;
        #endif // PROTOCOL_1_1
    } bind_request;

    //! To hold the data of bind response packet
    struct 
    {  
        //! To hold the header data of bind response packet
        struct
        {
            //! Parity bit of bind response packet being transmitted
            unsigned parity  : 0x01;

            //! For storing the pin used in the pind response packet
            unsigned pin     : 0x03;   

            //! Type of bind response packet being transmitted
            unsigned type    : 0x04;   
        } hdr;

        //! 1st byte of radio manufacturing ID
        UINT8 mid_1;

        //! 2nd byte of radio manufacturing ID
        UINT8 mid_2;

        //! 3rd byte of radio manufacturing ID
        UINT8 mid_3;

        //! 4th byte of radio manufacturing ID
        UINT8 mid_4;

        //! Checksum of bind response packet being transmitted
        UINT8 checksum;
    } bind_response;

    //! To hold the data of connect request packet
    struct
    {  
        //! To hold the header data of connect request packet
        struct
        {
            //! Parity bit of connect request packet
            unsigned parity : 0x01;

            //! For storing the pin used in the connect request packet
            unsigned pin    : 0x03;   

            //! Type of connect request packet
            unsigned type   : 0x04;   
        } hdr;

        #ifndef PROTOCOL_1_1

        //! Channel for connect request packet 
	    UINT8 channel;

        //! PN code for connect request packet 
	    UINT8 pn_code;
        #endif // PROTOCOL_1_1

        //! 1st byte of radio manufacturing ID
        UINT8   mid_1;

        //! 2nd byte of radio manufacturing ID
        UINT8   mid_2;

        //! 3rd byte of radio manufacturing ID
        UINT8   mid_3;

        //! 4th byte of radio manufacturing ID
        UINT8   mid_4;
        #ifdef DYNAMIC_PA_SUPPORT

        //! Power level when using dynamic PA support
		UINT8 pa;
		#endif //DYNAMIC_PA_SUPPORT

        //! Checksum of connect request packet
        UINT8 checksum;
    } connect_request;

    //! To hold the data of connect response packet
    struct 
    {  
        //! To hold the header data of connect response packet
        struct
        {
            //! Parity bit of connect response packet
            unsigned parity : 0x01;

            //! Filler for 2 bits of the connect response packet, could be
            //! 0 if NULL_PACKET_DATA is not defined   
            unsigned fill   : 0x02;   

            //! Data packet flag to tell whether it was a good or bad data
            unsigned flag   : 0x01;   

            //! Type of connect response packet
            unsigned type   : 0x04;   
        } hdr;
    } connect_response;

    //! To hold the data of the ping packet
    struct
    {
        //! To hold the header data of the ping packet
        struct
        {
            //! Parity bit of ping packet
            unsigned parity : 0x01;

            // Filler for 2 bits of the ping packet, could be
            // 0 if NULL_PACKET_DATA is not defined   
            unsigned fill   : 0x02;    

            //! Data packet flag to tell whether it was a good or bad data
            unsigned flag   : 0x01;

            //! Type of ping packet
            unsigned type   : 0x04;   
        } hdr;
    } ping;

    //! To hold the data of an data packet
    struct
    {
        //! To hold the header of a data packet
        struct
        {
            //! Parity bit of data packet
            unsigned parity : 0x01;

            //! Type of device part of data packet
            unsigned dev_id  : 0x01;   

            //! Data toggle information of data part of packet
            unsigned dt_data: 0x01;   

            // Filler for 1 bit of the data packet, could be
            // 0 if NULL_PACKET_DATA is not defined   
            unsigned fill   : 0x01;   

            //! Type of data packet
            unsigned type   : 0x04;   
        } hdr;

        //! Buffer to hold data packet
	    UINT8 app_packet[APP_TX_PACKET_SIZE];

        //! To hold calculated checksum of data packet being received
        UINT8 checksum;
    } data;

    //! To hold the data of an acknowledge packet
    struct
    {
        //! To hold the header data of an acknowledge packet
        struct
        {
            //! Parity bit of acknowledge packet
            unsigned parity : 0x01;

            //! Data toggle information of acknowledge packet
            unsigned dt_ack : 0x01;   

            //! Type of device part of acknowledge packet
            unsigned dev_id  : 0x01;   

            //! Acknowledge packet flag to tell whether it was a good or bad ack 
            unsigned flag   : 0x01;   

            //! Type of acknowledge packet
            unsigned type   : 0x04;   
        } hdr;
    } ack;

    //! To hold all data of an acknowledge/data packet
    struct
    {
        //! To hold the receive header data of an acknowledge/data packet
        struct
        {
            //! Parity bit of acknowledge/data packet
            unsigned parity : 0x01;

            //! Data toggle information of acknowledge part of packet
            unsigned dt_ack : 0x01;   

            //! Data toggle information of data part of packet
            unsigned dt_data : 0x01;   

            //! Acknowledge/data packet flag to tell whether it was a good or bad ack
            unsigned flag   : 0x01;   

            //! Type of acknowledge/data packet being received
            unsigned type   : 0x04;   
        } hdr;

        //! Buffer to hold acknowledge/data packet
    	UINT8 app_packet[APP_TX_PACKET_SIZE];

        //! Calculated checksum of acknowledge/data packet
        UINT8 checksum;
    } ack_data;

    //! To hold the transmit data of null packet
    struct
    {
        //! To hold the transmit header data of null packet
        struct
        {
            //! Parity bit of null packet
            unsigned parity : 0x01;

            //! Type of device part of null packet
            unsigned dev_id : 0x01;   

            // Filler for 2 bits of the null packet, could be
            // 0 if NULL_PACKET_DATA is not defined   
            unsigned fill   : 0x02;  

            //! Type of data packet
            unsigned type   : 0x04;   
        } hdr;
    } null;
    #ifdef ENCRYPT_DATA    
    struct
    {
        //! Header for encrypted data packet
        UINT8  header;

        //! Buffer to hold part of encrypted data packet
        UINT32 app_packet[2];

        //! To hold calculated checksum of encrypted data packet
        UINT8  checksum;
    } encrypt_response;
    #endif // ENCRYPT_DATA

    //! To hold the first byte of the receive packet
    struct
    {
        //! First byte of receive packet
        UINT8 byte;
    } first;
   
} LS_PACKET;

//! LS packet for transmitting data
#define LS_TX_PACKET LS_PACKET

//! LS packet for receiving data
#define LS_RX_PACKET LS_PACKET

#ifdef ENCRYPT_DATA

//! To hold the encryption key and signature
typedef struct _ENCRYPT_KEY
{
    //! Buffer for encryption key
    UINT32 key[4];

    //! Signature for encryption
    UINT8  signature;
} ENCRYPT_KEY;

//! Size of encryption key
#define ENCRYPT_KEY_SIZE 17
#endif    // ENCRYPT_DATA

// Lengths are variable due to unions. +1 is for the Checksum, +2 is for
// type byte and checksum

//! Size of an RX data packet.  +2 is for type byte and checksum
#define RX_DATA_PACKET_LEN APP_RX_PACKET_SIZE+2

//! Max size of RX arrays
#define LS_RX_PACKET_LEN   (sizeof(LS_RX_PACKET))     

#ifndef PROTOCOL_1_1

 //! Length of the bind request packet.  +2 is for type byte and checksum
 #define BIND_REQ_LEN            4+2

 //! Length of the bind response packet.  +2 is for type byte and checksum
 #define BIND_RESP_LEN           3+2

 #ifdef DYNAMIC_PA_SUPPORT

    //! Length of the connect request packet.  +2 is for type byte and checksum,
    //! +1 is for type byte (dynamic PA)
    #define CONNECT_REQ_LEN     6+2+1
 #else

    //! Length of the connect request packet.  +2 is for type byte and checksum,
    #define CONNECT_REQ_LEN         6+2
 #endif // DYNAMIC_PA_SUPPORT
#else

 //! Length of the bind request packet
 #define BIND_REQ_LEN        1

 //! Length of the bind response packet.  +2 is for type byte and checksum
 #define BIND_RESP_LEN       4+2
 #ifdef DYNAMIC_PA_SUPPORT

    //! Length of the connect request packet.  +2 is for type byte and checksum,
    //! +1 is for type byte (dynamic PA)
    #define CONNECT_REQ_LEN     4+2+1
 #else

    //! Length of the connect request packet.  +2 is for type byte and checksum
    #define CONNECT_REQ_LEN     4+2
 #endif // DYNAMIC_PA_SUPPORT
#endif //PROTOCOL_1_1

#ifdef ENCRYPT_DATA

//! Length of the key response packet
#define KEY_RESPONSE_LEN   10
#endif ENCRYPT_DATA

#ifdef CHECKSUM_ALL_PACKETS

//! Length of the connect response packet (including check sum)
#define CONNECT_RESP_LEN   0x02

//! Length of the null packet (including check sum)
#define NULL_LEN           0x02

//! Length of the ping packet (including check sum)
#define PING_LEN           0x02

//! Length of the ack packet (including check sum)
#define ACK_LEN            0x02
#else

//! Length of the connect response packet
#define CONNECT_RESP_LEN   0x01

//! Length of the null packet
#define NULL_LEN           0x01

//! Length of the ping packet
#define PING_LEN           0x01

//! Length of the ack packet
#define ACK_LEN            0x01
#endif // CHECKSUM_ALL_PACKETS

//! Max size of TX arrays
#define LS_TX_PACKET_LEN  (sizeof(LS_TX_PACKET))   

//! Size of an TX data packet.  +2 is for type byte and checksum
#define TX_PACKET_LEN     APP_TX_PACKET_SIZE + 2   // TX packet size

//--------------------------------------
// API Function Declarations
//--------------------------------------

void * protocol_get_tx_pkt(void);
#ifdef TWO_WAY
void * protocol_get_rx_pkt(void);
#endif

UINT8  protocol_gen_parity(UINT8 data);
UINT8  protocol_calc_ck_sum(UINT8 *data, UINT8 len, UINT8 seed);
UINT8  protocol_check_rssi(void);
void   protocol_send_ack(void);
UINT8  protocol_verify_packet(void);

UINT8  master_init(void);
UINT8  master_ping_mode(UINT8 reason);
UINT8  master_bind_mode(void);
UINT8  master_data_mode( void );

void init_channel(void);
void get_next_channel(void);

//! Packet for receiving
extern LS_RX_PACKET  rx_packet;

//! Packet for transmitting
extern LS_TX_PACKET  tx_packet;

#endif // _MASTER_PROTOCOL_H_
