//--------------------------------------------------------------------------
//! \file
//!
//!  Collection of routines to perform RS232 data transfer using
//!  the RS-232 port
//
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/DVK RS232/RS232.c $
// $Modtime: 2/12/04 12:33p $
// $Revision: 1.4 $
//--------------------------------------------------------------------------
//
// Copyright 2003, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

//--------------------------------------
// Included files
//--------------------------------------


#include "master_protocol.h"
#include "rx8_1.h"
#include "tx8_1.h"
#include "rs232.h"


//--------------------------------------
// Local Definitions and Types
//--------------------------------------
//! Transmit packet
extern LS_TX_PACKET   tx_packet;

//! Receive packet
extern LS_RX_PACKET   rx_packet;

//! Extra transmit packet for gRx8Buffer2
extern LS_TX_PACKET   tx_packet2;

//! Buffer for data packets
#define gRx8Buffer1   tx_packet.data.app_packet

//! 2nd buffer if the other is not available
#define gRx8Buffer2   tx_packet2.data.app_packet

//! Receive length for buffer 1
UINT8 gRx8Length1 = 0;

//! Receive length for buffer 2
UINT8 gRx8Length2 = 0;

//! To know what buffer to use for sending data (initially buffer 1)
UINT8 gSendBuffer = 1;

//! To know what buffer to use for loading data (initially buffer 1)
UINT8 gLoadBuffer = 1;

//! Buffer for data packet being transmitted
UINT8 gTx8Buffer[MAX_DATA_LEN];

//! Length of data in transmit buffer
UINT8 serialTxLength = 0;

//! Counter for every byte that is transmitted
UINT8 serialTxByteCounter = 0;

//--------------------------------------
// Local Function Declarations
//--------------------------------------

#pragma interrupt_handler rx8_isr

// Needs external linkage, but not a public API
void rx8_isr(void);

#pragma interrupt_handler tx8_isr

// Needs external linkage, but not a public API
void tx8_isr(void);

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_init
//
//!  Description: Initialize the RS232 control variables and sets the clear-
//!               to-send (CTS) flag.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void app_init()
{
    RX8_1_EnableInt();

    // lower CTS flag to load data...
    PRT2DR &= ~DB9_CTSout;      // Assert CTS
}



///////////////////////////////////////////////////////////////////////////////
//Sets the Tx Interrupt Mode bit in the Function Register.
//  Function:    app_unload_packet
//
//!  Description: When there are no packets being sent over RS-232, this
//!               routine disables the transmit interrupt, receives the RS232   
//!               packet, loads the app buffer, sets global packet variables, 
//!               sets the Tx Interrupt Mode bit, reads the Tx Status bits in 
//!               the Control/Status register, then enables the interrupt
//!
//!  Inputs:      
//!               - packet_size - Size of data packet.
//!
//!  Returns:     Void
//!               
///////////////////////////////////////////////////////////////////////////////
void app_unload_packet( UINT8 packet_size )
{
    UINT8 i;
    
    // spin until previous packet has been sent over RS-232
    while(serialTxLength != 0);    
    
    TX8_1_DisableInt();   // should already be disabled, but just to be safe...
    for( i = 0; i < packet_size; i++ )
    {
        // load packet into app buffer so radio doesn't overwrite the data...
        gTx8Buffer[i] = rx_packet.data.app_packet[i];    
    }
    serialTxLength = packet_size;
    serialTxByteCounter = 0;
    TX8_1_SetTxIntMode(TX8_1_TX_BUFFER_EMPTY);
    TX8_1_SendData(gTx8Buffer[serialTxByteCounter++]);
    TX8_1_EnableInt();
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_load_packet
//
//!  Description: After disabling the receive interrupt, this routine either     
//!               sets the global LoadBuffer for loading data or it sets the 
//!               global SendBuffer for sending data, depending on what the value 
//!               of the global SendBuffer and the global RX Lengths are set to. 
//!               Then it sets the RX packet length and packet number then
//!               enables the receive interrupt.
//!
//!  Inputs:      
//!               - *packet_num - Packet number
//!
//!  Returns:     
//!               - rx_packet_length - Length of the data packet being received
//!               
///////////////////////////////////////////////////////////////////////////////
UINT8 app_load_packet( UINT8 *packet_num )
{
    UINT8 rx_packet_length = 0;
    *packet_num = 0;

    RX8_1_DisableInt();
    if ((gSendBuffer == 1) && (gRx8Length1 > 0))
    {
        gLoadBuffer = 2;    // start loading data into the other buffer
        *packet_num = 1;

        rx_packet_length = gRx8Length1;
    }
    else if ((gSendBuffer == 2) && (gRx8Length2 > 0))
    {
        gLoadBuffer = 1;    // start loading data into the other buffer
        *packet_num = 2;

        rx_packet_length = gRx8Length2;
    }
    else 
    {
        if ((gSendBuffer == 1) && (gRx8Length2 > 0))
        {
            gSendBuffer = 2;
        }
        else if ((gSendBuffer == 2) && (gRx8Length1 > 0))
        {
            gSendBuffer = 1;
        }
        rx_packet_length = 0;
        *packet_num = 0;
    }
    RX8_1_EnableInt();
    
    return rx_packet_length;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_packet_transmitted
//
//!  Description: After disabling the receive interrupt this routine will set 
//!               up the right SendBuffer for the next packet and lower the 
//!               clear-to-send (CTS) flag if there is a length to either RX 
//!               packet.  If there is no length to either one, it will enable 
//!               the receive interrupt and exit.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!               
///////////////////////////////////////////////////////////////////////////////
void app_packet_transmitted( void )
{
    RX8_1_DisableInt();

    if ((gSendBuffer == 1) && (gRx8Length1 != 0))
    {
        gRx8Length1 = 0;
        gSendBuffer = 2;    // next packet to send is the other buffer

        if (gRx8Length2 < CTS_ASSERT_LEN)
        {
            // lower CTS flag to load more data...
            PRT2DR &= ~DB9_CTSout;      // Assert CTS
        }
    }
    else if ((gSendBuffer == 2) && (gRx8Length2 != 0))
    {
        gRx8Length2 = 0;
        gSendBuffer = 1;    // next packet to send is the other buffer

        if (gRx8Length1 < CTS_ASSERT_LEN)
        {
            // lower CTS flag to load more data...
            PRT2DR &= ~DB9_CTSout;      // Assert CTS
        }
    }
    RX8_1_EnableInt();

}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    rx8_isr
//
//!  Description: Once the start bit is detected from the radio receiver and
//!               no errors are detected, this routine checks to see what buffer  
//!               is available and loads it byte by byte.  Once the data is 
//!               loaded in the buffer, the clear-to-send flag is set.  If an
//!               error is detected the data is dumped.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void rx8_isr(void)
{
    UINT8 rx_status;
    // check the radio_receiver to detect start bit 
    
    while(1)
    {
        while ( !( rx_status = RX8_1_bReadRxStatus() & RX8_RX_COMPLETE ) )
        {
        }
    
        // data radio_received, now check for errors
       if (( rx_status & RX8_RX_NO_ERROR ) == 0 )
       {
           if (gLoadBuffer == 1)
           {
               if (gRx8Length1 < MAX_DATA_LEN) 
               {
                   gRx8Buffer1[gRx8Length1] = bRX8_1_ReadRxData();
                   ++gRx8Length1;
        
                   if (gRx8Length1 >= CTS_ASSERT_LEN)
                   {
                       PRT2DR |= DB9_CTSout;        // DE-Assert CTS NOW!
                   }         
               }
                else
                {
                    // dump the data, there's nowhere to put it :-(
                    bRX8_1_ReadRxData();    
                }
           }
           else
           {
               if (gRx8Length2 < MAX_DATA_LEN) 
               {
                   gRx8Buffer2[gRx8Length2] = bRX8_1_ReadRxData();
                   ++gRx8Length2;
        
                   if (gRx8Length2 >= CTS_ASSERT_LEN)
                   {
                       PRT2DR |= DB9_CTSout;        // DE-Assert CTS NOW!
                   }         
               }
               else
               {
                   // dump the data, there's nowhere to put it :-(
                   bRX8_1_ReadRxData();    
               }
            }
            return;
        }
        else
        {
            bRX8_1_ReadRxData();    // dump the data on an error... 
        }
    }
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    tx8_isr
//
//!  Description: If a buffer is ready for a packet transmission, the transmit
//!               interrupt mode will be set after the packet has transmitted
//!               successfully.  Then the transmit interrupt will be disabled.  
//!               If there is no data ready to transmit it will return.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void tx8_isr( void )
{
    UINT8 tx_status;

    tx_status = bTX8_1_ReadTxStatus();
    if ( tx_status  & TX8_TX_BUFFER_EMPTY )
    {
        // if more data, load next byte....
        if (serialTxByteCounter < serialTxLength)
        {
            TX8_1_SendData(gTx8Buffer[serialTxByteCounter++]);
            if (serialTxByteCounter == serialTxLength)
            {
                TX8_1_SetTxIntMode(TX8_1_INT_MODE_TX_COMPLETE);
            }
        }
        else if (serialTxByteCounter == serialTxLength)
        {
            serialTxLength = 0;
            TX8_1_DisableInt();      // disable interrupt
            return;
        }
    }
    else
    {
        return;
    }
}
