//--------------------------------------------------------------------------
//! \file
//!
//! Main application control routine
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/DVK RS232/main.c $
// $Modtime: 3/16/04 9:50a $
// $Revision: 1.4 $
//--------------------------------------------------------------------------
//
// Copyright 2003, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//--------------------------------------------------------------------------

//--------------------------------------
// Included files
//--------------------------------------
#include "protocol.h"
#include "RS232.h"
#include "bind.h"


//--------------------------------------
// Local Definitions and Types
//--------------------------------------

//! Transmit packet
extern LS_TX_PACKET tx_packet;

//! Receive packet
extern LS_RX_PACKET rx_packet;


//! Maximum number of transmits
#define MAX_NUM_TRANSMIT    5

//! To know when to check for a bind button press
#define BIND_CHECK_COUNT    40

//! For initializing and resetting poll_gap
#define MIN_POLL_GAP        1

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    main
//
//!  Description: RS232 Slave main routine initializes the interrupt service
//!               routine, timer, platform board, protocol, and RS232 slave,   
//!               gets the time stamp and turns off the LEDs.  Then it 
//!               continuously checks for receive data packets from serial port 
//!               and transmits packets when existent and checks the status of 
//!               the transmitted packet.  If there are no packets to be sent 
//!               or received, it will look for a bind button press and bind if
//!               bind is successful.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void main()
{
    UINT8      status;
    UINT8      length;
    UINT8      num_transmit_tries;
    TIME_STAMP time_stamp;
    UINT16     poll_gap = MIN_POLL_GAP;
    UINT8      no_response_count = 0;
    UINT8      bind_check_counter = 0;

  
    // setup the Radio/GPIO interrupt logic
    isr_init();

    // initialize the timer and its ISR
    timer_init();
    
    // configure the board
    platform_init();

    set_connect_led(OFF); 
    set_bind_led(OFF);
    
    // setup the protocol initialization 
    protocol_init();

    // perform PERT initialization
    app_init();

    time_stamp = TIMER_GET_TIME_STAMP();

    // main infinite control loop
    while(1) 
    {   
        // perform the general routine, check for an RX byte from
        // the serial port.
        length = app_tx_process( );  
        
        num_transmit_tries = 0;
        
        // send packet if enter pressed and not all packets sent   
        while (( length > 0 ) && ( num_transmit_tries < MAX_NUM_TRANSMIT ))
        {          
            // radio_transmit the application report
            status = protocol_send_packet( length );        
            num_transmit_tries++;
            
            // other devices may need to respond shortly after receiving the 
            // transmitted packet
            poll_gap = MIN_POLL_GAP;            
            
            if (status == ACK_DATA)
            {
                // application processes packet contents
#ifdef BACK_CHANNEL_DATA 
                app_rx_process( protocol_get_rx_packet_length() );
#endif // BACK_CHANNEL_DATA
                no_response_count = 0;

                app_packet_transmitted();
                length = 0;
            }  
            else if (status == TX_TIMEOUT_ERR)
            {
            // need to reload the packet, because it might have been overwritten
                app_reload_packet( length );                
            }
            else if (status == DATA_SENT)
            {
                no_response_count = 0;
                app_packet_transmitted();
                length = 0;
            }            
            
            timer_timer_off();    // reset the timer..
            timer_timer_on(); 
        } //   if( length > 0 )
        if (poll_gap < TIMER_GET_TIME_STAMP())
        {
            status = protocol_send_packet(0);
            if (status == ACK_DATA)
            {
                // application processes packet contents
#ifdef BACK_CHANNEL_DATA 
                app_rx_process( protocol_get_rx_packet_length() );
#endif // BACK_CHANNEL_DATA
                no_response_count = 0;
                poll_gap = MIN_POLL_GAP;
            }  
            else
            {
                no_response_count++;
                if ((no_response_count > 3) && (poll_gap < 0x1000))
                {
                    if (poll_gap == 0)
                    {
                        poll_gap = 1;
                    }
                    else
                    {
                        poll_gap <<= 1;        // pollgap * 8
                    }
                    no_response_count = 0;
                }
            }
            
            timer_timer_off();    // reset the timer
            timer_timer_on(); 
        }        
        
        if( bind_check_counter++ > BIND_CHECK_COUNT)
        {
            bind_check_counter = 0;
            
            if(bind_pressed())
            {
                set_bind_led(ON); 

                // turn off the Connect LED
                set_connect_led(OFF); 
                status = protocol_bind();
                set_bind_led(OFF);

                if (status == LS_STATUS_SUCCESS) {
                    set_connect_led(ON);
                }
            }            
        }
    }  // while(1)
}  // end main


