//--------------------------------------------------------------------------
//! \file
//!
//!  Collection of routines to perform General data transfer using
//!  the RS-232 port
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/DVK RS232/RS232.c $
// $Modtime: 3/10/04 12:25p $
// $Revision: 1.4 $
//--------------------------------------------------------------------------
//
// Copyright 2003, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------


//--------------------------------------
// Included files
//--------------------------------------

#include "rx8_1.h"
#include "tx8_1.h"
#include "rs232.h"
#include "protocol.h"


//--------------------------------------
// Local Definitions and Types
//--------------------------------------

//! Transmit packet
extern LS_TX_PACKET   tx_packet;

//! Receive packet
extern LS_RX_PACKET   rx_packet;

//! Buffer for data packet being received
UINT8 serialRxBuffer1[MAX_DATA_LEN];

//! Length of data in receive buffer1
UINT8 serialRxLength1 = 0;


//! Another buffer for data packets being received used if the first one is
//! already in use
UINT8 serialRxBuffer2[MAX_DATA_LEN];

//! Length of data in receive buffer2
UINT8 serialRxLength2 = 0;


//! To know what buffer to use for sending data (initially buffer 1)
UINT8 gSendBuffer = 1;

//! To know what buffer to use for loading data (initially buffer 1)
UINT8 gLoadBuffer = 1;


//! Buffer for data packet being transmitted
UINT8 serialTxBuffer[MAX_DATA_LEN];

//! Length of data in transmit buffer
UINT8 serialTxLength = 0;

//! Counter for every byte that is transmitted
UINT8 serialTxByteCounter = 0;

//--------------------------------------
// Local Function Declarations
//--------------------------------------

#pragma interrupt_handler rx8_isr

// Needs external linkage, but not a public API
void rx8_isr(void);

#pragma interrupt_handler tx8_isr

// Needs external linkage, but not a public API
void tx8_isr(void);

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_init
//
//!  Description: Initialize the RS232 control variables and sets the clear-
//!               to-send (CTS) flag.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void app_init()
{
    RX8_1_EnableInt();
    
    // init CTS
    PRT2DR |= DB9_CTSout;   //DE-Assert the CTS flow-control line to start

}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_tx_process
//
//!  Description: Checks for a transmit packet.  If there is no transmit 
//!               packet, the clear-to-send flag is set and it exits.  When 
//!               valid input data is in the buffer, the receive interrupt
//!               is disabled. The data is loaded and a data packet is generated.  
//!               Then the receive interrupt is enabled.
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - 0 - no packet to send
//!               - serialRxLength - length of buffer that was loaded 
//!               
///////////////////////////////////////////////////////////////////////////////
UINT8 app_tx_process( void )
{
    UINT8 i = 0;
    
    // exit if no input data

    if( gSendBuffer == 1)
    {
        if( serialRxLength1 == 0 ) 
        {
            PRT2DR &= ~DB9_CTSout;      // Assert CTS    
            return( 0 );
        }
        else 
        { 
            RX8_1_DisableInt();
                
            for (i = 0; i < serialRxLength1; i++)
            {
                tx_packet.data.app_packet[i] = serialRxBuffer1[i];
            }
            gLoadBuffer = 2;    // switch to the second buffer for loading new data
    
            RX8_1_EnableInt();
            return( serialRxLength1 );
        }
    }
    else
    {
        if( serialRxLength2 == 0 ) 
        {
            PRT2DR &= ~DB9_CTSout;      // Assert CTS    
            return( 0 );
        }
        else 
        { 
            RX8_1_DisableInt();
                
            for (i = 0; i < serialRxLength2; i++)
            {
                tx_packet.data.app_packet[i] = serialRxBuffer2[i];
            }
            gLoadBuffer = 1;    // switch to the second buffer for loading new data

            RX8_1_EnableInt();                      
            return( serialRxLength2 );
        }
    }
} 
    

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_reload_packet
//
//!  Description: Regenerates the transmit data packet depending on what buffer
//!               is currently being used 
//!
//!  Inputs:      
//!               - length - length of the buffer
//!
//!  Returns:     Void
//!               
///////////////////////////////////////////////////////////////////////////////
void app_reload_packet( UINT8 length )
{
    UINT8 i;

    if (gSendBuffer == 1)
    {
        for (i = 0; i < length; i++)
        {
            tx_packet.data.app_packet[i] = serialRxBuffer1[i];
        }
    }
    else
    {
        for (i = 0; i < length; i++)
        {
            tx_packet.data.app_packet[i] = serialRxBuffer2[i];
        }
    }
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_packet_transmitted
//
//!  Description: After disabling the receive interrupt this routine will set 
//!               up the right SendBuffer for the next packet and lower the 
//!               clear-to-send (CTS) flag if there is a length to either RX 
//!               packet.  If there is no length to either one, it will enable 
//!               the receive interrupt and exit.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!               
///////////////////////////////////////////////////////////////////////////////
void app_packet_transmitted( void )
{
    RX8_1_DisableInt();

    if (gSendBuffer == 1)
    {
        serialRxLength1 = 0;
        gSendBuffer = 2;    // next packet to send is the other buffer

        if (serialRxLength2 < CTS_ASSERT_LEN)
        {
            // lower CTS flag to load more data...
            PRT2DR &= ~DB9_CTSout;      // Assert CTS
        }
    }
    else
    {
        serialRxLength2 = 0;
        gSendBuffer = 1;    // next packet to send is the other buffer

        if (serialRxLength1 < CTS_ASSERT_LEN)
        {
            // lower CTS flag to load more data...
            PRT2DR &= ~DB9_CTSout;      // Assert CTS
        }
    }

    RX8_1_EnableInt();

    // lower CTS flag to load more data into the buffer that was just transmitted
    PRT2DR &= ~DB9_CTSout;      // Assert CTS
}



///////////////////////////////////////////////////////////////////////////////
//
//  Function:    app_rx_process
//
//!  Description: Waits until the previous packet has been loaded into the 
//!               TX8 buffer, disables the transmit interrupt and loads the
//!               transmit data packet and sets it up to send.  Then the packet
//!               is transmitted and the transmit interrupt is enabled.
//!
//!  Inputs:      
//!               - packet_size - Application data packet
//!
//!  Returns:      Void
//!              
//!              
///////////////////////////////////////////////////////////////////////////////

void app_rx_process( UINT8 packet_size )
{
    UINT8 i;
    
    if (serialTxLength != 0)
    {
        // spin until previous packet has been loaded into TX8 buffer
        while(serialTxLength != 0);    
        timer_delay_10_usec();    // delay long enough for byte to be emptied
    }
    
    TX8_1_DisableInt();  // should already be disabled, but just to be safe...
    for( i = 0; i < packet_size; i++ )
    {
        // load packet into app buffer so radio doesn't overwrite the data...
        serialTxBuffer[i] = rx_packet.data.app_packet[i];    
    }
    serialTxLength = packet_size;
    serialTxByteCounter = 0;
    TX8_1_SetTxIntMode(TX8_1_TX_BUFFER_EMPTY);
    TX8_1_SendData(serialTxBuffer[serialTxByteCounter++]);
    TX8_1_EnableInt();
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    rx8_isr
//
//!  Description: Once the start bit is detected from the radio receiver and
//!               no errors are detected, this routine checks to see what buffer  
//!               is available and loads it byte by byte.  Once the data is 
//!               loaded in the buffer, the clear-to-send flag is set.  If an
//!               error is detected the data is dumped.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////

void rx8_isr(void)
{
    UINT8 rx_status;

    while(1)
    {
        //rx_status = bRX8_1_ReadRxStatus();
        while ( !( rx_status = RX8_1_bReadRxStatus() & RX8_RX_COMPLETE ) )
        {
        }

        // data radio_received, now check for errors
       if (( rx_status & RX8_RX_NO_ERROR ) == 0 )
       {
           if (gLoadBuffer == 1)
           {
               if (serialRxLength1 < MAX_DATA_LEN) 
               {
                   serialRxBuffer1[serialRxLength1] = bRX8_1_ReadRxData();
                   ++serialRxLength1;
        
                   if (serialRxLength1 >= CTS_ASSERT_LEN)
                   {
                       PRT2DR |= DB9_CTSout;        // DE-Assert CTS NOW!
                   }         
               }
                else
                {
                    // dump the data, there's nowhere to put it :-(
                    bRX8_1_ReadRxData();    
                }
           }
           else
           {
               if (serialRxLength2 < MAX_DATA_LEN) 
               {
                   serialRxBuffer2[serialRxLength2] = bRX8_1_ReadRxData();
                   ++serialRxLength2;
        
                   if (serialRxLength2 >= CTS_ASSERT_LEN)
                   {
                       PRT2DR |= DB9_CTSout;        // DE-Assert CTS NOW!
                   }         
               }
               else
               {
                   // dump the data, there's nowhere to put it :-(
                   bRX8_1_ReadRxData();    
               }
            }
            return;
        }
        else
        {
            bRX8_1_ReadRxData();    // dump the data on an error... 
        }
    }
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    tx8_isr
//
//!  Description: If a buffer is ready for a packet transmission, the transmit
//!               interrupt mode will be set after the packet has transmitted
//!               successfully.  Then the transmit interrupt will be disabled.  
//!               If there is no data ready to transmit it will return.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void tx8_isr( void )
{
    UINT8 tx_status;

    //while(1)
    {
    tx_status = bTX8_1_ReadTxStatus();
    if ( tx_status  & TX8_TX_BUFFER_EMPTY )
    {
        // if more data, load next byte....
        if (serialTxByteCounter < serialTxLength)
        {
                TX8_1_SendData(serialTxBuffer[serialTxByteCounter++]);
            if (serialTxByteCounter == serialTxLength)
            {
                TX8_1_SetTxIntMode(TX8_1_INT_MODE_TX_COMPLETE);
            }
        }
        else if (serialTxByteCounter == serialTxLength)
        {
            // disable interrupt
            serialTxLength = 0;
            TX8_1_DisableInt();                
            return;
        }
    }
    else
    {
        return;
    }
    }
}
