//--------------------------------------------------------------------------
//!  \file
//!
//!  Collection of functions for FCC, ETSI and TELEC testing
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/EMI Tester/emc_tester.c $
// $Modtime: 3/11/04 3:52p $
// $Revision: 1.4 $
//--------------------------------------------------------------------------
//
// Copyright 2003-2004, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

#include "emc_tester.h"
#include "radio.h"
#include "ls_config.h"

//! For know what mode the emc_tester is in
UINT8 gEMCtesterMode;
//! Current channel the radio is on
UINT8 gChannel  = DEFAULT_CHANNEL;
//! For using the top channel (channel 77)
UINT8 gTopChannel = CHANNEL_TOP;
//! For using the bottom channel (channel 0)
UINT8 gBotChannel = CHANNEL_BOTTOM;
//! Current PN code the radio is on
UINT8 pn_code   = DEFAULT_PN_CODE;
//! Data rate setting for radio
UINT8 gDataRate = DATA_RATE_16;
//! Current PA bias the radio is on
UINT8 gPaBias = PA_BIAS;

//! Packet, data toggle 0
const UINT8 pktData0[] = { 0x43, 0x01, 0x01, 0x43}; // mouse packet, data toggle 0
//! Packet, data toggle 1
const UINT8 pktData1[] = { 0x46, 0x01, 0x01, 0x46}; // mouse packet, data toggle 1
//! Size of packet
      UINT8 pktSize    = sizeof(pktData1);


/*************************** MAIN FUNCTIONS **********************************/


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    start_transmit_carrier
//
//!  Description: Begins transitting on the carrier frequency. Transmission continues
//!               until stop_transmit_carrier is called.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
    
void start_transmit_carrier(void)
{
    stop_test();
    
    gEMCtesterMode = EMC_TESTER_CARRIER;

    SPI_RADIO_PUT(REG_CHANNEL, (CHANNEL_MASK & (gChannel + CHANNEL_OFFSET)));
    SPI_RADIO_PUT(REG_CONTROL, (TX_ENABLE | AUTO_SYNTH_COUNT));	
    SPI_RADIO_PUT(REG_CLOCK_ENABLE, 0x51);
    SPI_RADIO_PUT(REG_CLOCK_MANUAL, 0x41);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    stop_transmit_carrier
//
//!  Description: Stops transmitting on the carrier frequency
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void stop_transmit_carrier(void)
{
    radio_off();
    SPI_RADIO_PUT(REG_CLOCK_ENABLE, 0x41);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    start_transmit_data
//
//!  Description: Starts transmitting random on the current channel. Transmission
//!               continues until stop_transmit_data is called.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void start_transmit_data(void)
{
    stop_test();
    
    gEMCtesterMode = EMC_TESTER_DATA;

    SPI_RADIO_PUT(REG_TX_INT_EN, TX_EMPTY);
    
    radio_transmit_on(gChannel);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    stop_transmit_data
//
//!  Description: Stops transmitting random data on the current channel.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void stop_transmit_data(void)
{
    radio_off();
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    start_receive
//
//!  Description: Starts receiving on the current channel. Receive continues
//!               until stop_receive is called.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void start_receive(void)
{
    stop_test();
    
    gEMCtesterMode = EMC_TESTER_RECEIVE;
    
    radio_receive_on(gChannel);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    stop_receive
//
//!  Description: Stops receiving on the current channel.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void stop_receive(void)
{
    radio_off();
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    packet_test
//
//!  Description: Enables the empty interrupt, transmits a packet of data, then
//!               disables the empty interrupt and enables the transmit 
//!               interrupt. Performs a deadman timout and turns off the radio
//!               transmitter
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void packet_test(void)
{
    // reuse random_filler as a data toggle control
    extern UINT8 random_filler;
    UINT8 tCounter = 0, i;
      
     
    SPI_RADIO_PUT(REG_TX_INT_EN, TX_EMPTY);

    radio_transmit_on(gChannel);     

    // send out a packet of data
    for( i = 0; i < pktSize; i++ )
    {
        tCounter = 0;
        while( !LS_IRQ_ASSERTED )
        {
            if( tCounter++ > 120 ) 
            {
                break;  // dead man timeout, normal count 27-30        
            }
        }
         // reuse random_filler as a data toggle control
        if( random_filler & 0x01 )  // data toggle 1
        {  
            SPI_RADIO_PUT(REG_DATA_TX, pktData1[i]);
        }
        else  // data toggle 0
        {
            SPI_RADIO_PUT(REG_DATA_TX, pktData0[i]);
        }
    }
    
    // toggle data toggle
    if( random_filler & 0x01 )  // set data toggle 0
    {       
        random_filler = 0;
    }
    else  // set data toggle 1
    {
        random_filler = 1;
    }
    // enable the EOF interrupt and disable the EMPTY interrupt
    SPI_RADIO_PUT(REG_TX_INT_EN, TX_EOF);

    tCounter = 0;
    
	TIMER_DELAY_50_USEC();
	TIMER_DELAY_50_USEC();

    
	while(!LS_IRQ_ASSERTED)
    {
        if( tCounter++ > 120 ) 
        {
            break;  // dead man timeout   
        }    
    }                  
    
    radio_off();

}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    stop_test
//
//!  Description: Stops the current test if a test is running.
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!              - gEMCtesterMode - Mode of emc_tester
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 stop_test()
{
    
    switch(gEMCtesterMode)
    {
    case EMC_TESTER_CARRIER:
        stop_transmit_carrier(); 
        break;
        
    case EMC_TESTER_DATA:
        stop_transmit_data();
        break;
      
    case EMC_TESTER_RECEIVE:
        stop_receive();
    }
      
    return gEMCtesterMode;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    restart_test
//
//!  Description: Restarts the test, if a test was running.
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - gEMCtesterMode - Mode of emc_tester
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 restart_test()
{
  
    switch(gEMCtesterMode)
    {
    case EMC_TESTER_CARRIER:
        start_transmit_carrier(); 
        break;
        
    case EMC_TESTER_DATA:
        start_transmit_data();
        break;
      
    case EMC_TESTER_RECEIVE:
        start_receive();
    }

    // ensure the Radio tx interrupt is enabled
    SPI_RADIO_PUT(REG_TX_INT_EN, TX_EMPTY);
    
    return gEMCtesterMode;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    reset_test
//
//!  Description: Stops the current test and moves the EMI Tester to Idle Mode.
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - gEMCtesterMode - Mode of emc_tester
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 reset_test(void)
{
    stop_test();
    gEMCtesterMode = EMC_TESTER_IDLE;
    
    return gEMCtesterMode;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    increment_test
//
//!  Description: Cycles through the test modes
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - gEMCtesterMode - Mode of emc_tester
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 increment_test(void)
{
    stop_test();
    
    switch(gEMCtesterMode)
    {
    case EMC_TESTER_IDLE:
        gEMCtesterMode = EMC_TESTER_CARRIER;
        break;
        
    case EMC_TESTER_CARRIER:
        gEMCtesterMode = EMC_TESTER_DATA;
        break;
        
    case EMC_TESTER_DATA:
        gEMCtesterMode = EMC_TESTER_RECEIVE;
        break;
        
    case EMC_TESTER_RECEIVE:
        gEMCtesterMode = EMC_TESTER_PKT_DATA;
        break;
        
    case EMC_TESTER_PKT_DATA:
        gEMCtesterMode = EMC_TESTER_IDLE;
        break;
    }
    
    restart_test();
    
    return gEMCtesterMode;
}


/************************** HELPER FUNCTIONS *********************************/


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    reset_PN_CODE
//
//!  Description: Resets the PN code to PN Code 0 of the current PN code table.
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - pn_code - PN code that was reset
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 reset_pn_code(void)
{
    stop_test();
    pn_code = 0;
    if (gDataRate == DATA_RATE_16)
    {
        radio_set_pn_code_64(pn_code);
    }
    else 
    {
        radio_set_pn_code_32(pn_code);
    }
    restart_test();
    
    return pn_code;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    increment_pn_code
//
//!  Description: Increments the current PN code of the current PN code table.
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - pn_code - PN code that was incremented
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 increment_pn_code(void)
{
    stop_test();
    
    if( gDataRate == DATA_RATE_16 )
    {
        if (pn_code < (NUM_PN_CODES_64-1))
        {
            pn_code++;
        }
        else
        {
            pn_code = 0;
        }
        radio_set_pn_code_64(pn_code);
    }
    else
    {
        if (pn_code < (NUM_PN_CODES_32-1))
        {
            pn_code++;
        }
        else
        {
            pn_code = 0;
        }
        radio_set_pn_code_64(pn_code);
    }
    
    restart_test();
    
    return pn_code;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    decrement_pn_code
//
//!  Description: Increments the current PN code of the current PN code table.
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - pn_code - PN code that was decremented
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 decrement_pn_code(void)
{
    stop_test();
    
    if( gDataRate == DATA_RATE_16) 
    {
        if (pn_code > 0)
        {
            pn_code--;
        }
        else
        {
            pn_code = NUM_PN_CODES_64-1;
        }
        radio_set_pn_code_64(pn_code);
    }
    else 
    {
        if (pn_code > 0)
        {
            pn_code--;
        }
        else
        {
            pn_code = NUM_PN_CODES_32-1;
        }
        radio_set_pn_code_32(pn_code);
    }
    
    restart_test();
    
    return pn_code;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    reset_channel
//
//!  Description: Rests the channel to CHANNEL_BOTTOM (defined in emc_tester.h)
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - gChannel - Channel that was set to the bottom channel
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 reset_channel(void)
{
    stop_test();
    gChannel = CHANNEL_BOTTOM;
    restart_test();
    
    return gChannel;
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    increment_channel
//
//!  Description: Increments the current channel in the following order:
//!               CHANNEL_BOTTOM -> CHANNEL_MIDDLE -> CHANNEL_TOP -> CHANNEL_BOTTOM
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - gChannel - Channel that it was incremented to
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 increment_channel(void)
{
    stop_test();

#ifndef TELEC_ALL_CHANNELS
    if( gChannel > CHANNEL_MIDDLE )  // top channel
    {
        gChannel = gBotChannel;
    }
    else if  ( gChannel < CHANNEL_MIDDLE )  // bottom channel
    {
        gChannel = CHANNEL_MIDDLE;
    }
    else // Middle
    {
        gChannel = gTopChannel;
    } 
#else
    if (gChannel < NUM_CHANNELS-1) 
    {
        gChannel++;
    }
    else 
    {
        gChannel = 0;
    }
#endif // TELEC_ALL_CHANNELS
    
    restart_test();
    
    return gChannel;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    decrement_channel
//
//!  Description: Decrements the current channel in the following order:
//!               CHANNEL_TOP -> CHANNEL_MIDDLE -> CHANNEL_BOTTOM -> CHANNEL_TOP
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - gChannel - Channel that it was decremented to
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 decrement_channel(void)
{
    stop_test();
    
#ifndef TELEC_ALL_CHANNELS
    if( gChannel > CHANNEL_MIDDLE )
    {
        gChannel = CHANNEL_MIDDLE;
    }
    else if  ( gChannel < CHANNEL_MIDDLE )  
    {
        gChannel = gTopChannel;
    }
    else // Middle
    {
        gChannel = gBotChannel;
    }    
#else
    if (gChannel > 0)
    {    
        gChannel--;
    }
    else
    {
        gChannel = NUM_CHANNELS-1;
    }
#endif // TELEC_ALL_CHANNELS
 
    restart_test();
    
    return gChannel;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    reset_data_rate
//
//!  Description: Resets the data rate to 16kbps
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - gDataRate - Data rate that was reset
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 reset_data_rate(void)
{
    stop_test();
    reset_pn_code();
    gDataRate = DATA_RATE_16;
    radio_set_pn_code_64(pn_code);
    restart_test();
    
    return gDataRate;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    increment_data_rate
//
//!  Description: Increments the data rate in the following order:
//!               16kbps -> 32kbps -> 62.5kbps -> 16kbps
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - gDataRate - Data rate that was incremented
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 increment_data_rate(void)
{
    stop_test();
    
    switch (gDataRate)
    {
    case DATA_RATE_16:
        SPI_RADIO_PUT(REG_DATA_RATE, RADIO_MODE_32);
        SPI_RADIO_PUT(REG_THRESHOLD_LO, THRESHOLD_LO_32);
        SPI_RADIO_PUT(REG_THRESHOLD_HI, THRESHOLD_HI_32);
        if( pn_code >= NUM_PN_CODES_32 )
        {
            pn_code = NUM_PN_CODES_32 - 1;
        }        
        radio_set_pn_code_32(pn_code);
        gDataRate = DATA_RATE_32;
        break;
    
    case DATA_RATE_32:
        SPI_RADIO_PUT(REG_DATA_RATE, RADIO_MODE_DDR);
        SPI_RADIO_PUT(REG_THRESHOLD_LO, THRESHOLD_LO_32);
        SPI_RADIO_PUT(REG_THRESHOLD_HI, THRESHOLD_HI_32);
        if( pn_code >= NUM_PN_CODES_32 )
        {
            pn_code = NUM_PN_CODES_32 - 1;
        }
        radio_set_pn_code_32(pn_code);
        gDataRate = DATA_RATE_64;
        break;
    
    case DATA_RATE_64:
        SPI_RADIO_PUT(REG_DATA_RATE, RADIO_MODE_64);
        SPI_RADIO_PUT(REG_THRESHOLD_LO, THRESHOLD_LO_64);
        SPI_RADIO_PUT(REG_THRESHOLD_HI, THRESHOLD_HI_64);
        radio_set_pn_code_64(pn_code);
        gDataRate = DATA_RATE_16;
        break;
    }
    
    restart_test();
    
    return gDataRate;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    decrement_data_rate
//
//!  Description: Decrements the data rate in the following order:
//!               62.5kbps -> 32kbps -> 16kbps -> 62.5kbps
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - gDataRate - Data rate that was decremented
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 decrement_data_rate(void)
{
    stop_test();
    
    switch (gDataRate)
    {
    case DATA_RATE_16:
        SPI_RADIO_PUT(REG_DATA_RATE, RADIO_MODE_DDR);
        SPI_RADIO_PUT(REG_THRESHOLD_LO, THRESHOLD_LO_32);
        SPI_RADIO_PUT(REG_THRESHOLD_HI, THRESHOLD_HI_32);      
        if( pn_code >= NUM_PN_CODES_32 )
        {
            pn_code = NUM_PN_CODES_32 - 1;
        }        
        radio_set_pn_code_32(pn_code);
        gDataRate = DATA_RATE_64;
        break;
    
    case DATA_RATE_32:
        SPI_RADIO_PUT(REG_DATA_RATE, RADIO_MODE_64);
        SPI_RADIO_PUT(REG_THRESHOLD_LO, THRESHOLD_LO_64);
        SPI_RADIO_PUT(REG_THRESHOLD_HI, THRESHOLD_HI_64);
        radio_set_pn_code_64(pn_code);
        gDataRate = DATA_RATE_16;
        break;
    
    case DATA_RATE_64:
        SPI_RADIO_PUT(REG_DATA_RATE, RADIO_MODE_32);
        SPI_RADIO_PUT(REG_THRESHOLD_LO, THRESHOLD_LO_32);
        SPI_RADIO_PUT(REG_THRESHOLD_HI, THRESHOLD_HI_32);
        if( pn_code >= NUM_PN_CODES_32 )
        {
            pn_code = NUM_PN_CODES_32 - 1;
        }        
        radio_set_pn_code_32(pn_code);
        gDataRate = DATA_RATE_32;
        break;
    }
    
    restart_test();
    
    return gDataRate;
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    increment_pa_bias
//
//!  Description: Increments the PA bias in the following order:
//!               0 -> 1 -> 2 -> 3 -> 4 -> 5 -> 6 -> 7 -> 7 ..
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - gPaBias - PA bias that was incremented
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 increment_pa_bias(void)
{
    if (gPaBias < 7) {
        stop_test();
        gPaBias++;
        SPI_RADIO_PUT(REG_PA, (gPaBias & PA_BIAS_MASK));
        restart_test();
    }
    
    return gPaBias;
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    decrement_pa_bias
//
//!  Description: Decrements the PA bias in the following order:
//!               7 -> 6 -> 5 -> 4 -> 3 -> 2 -> 1 -> 0 -> 0 ..
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - gPaBias - PA bias that was decremented
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 decrement_pa_bias(void)
{
    if (gPaBias > 0) {
        stop_test();
        gPaBias--;
        SPI_RADIO_PUT(REG_PA, (gPaBias & PA_BIAS_MASK));
        restart_test();
    }
    
    return gPaBias;
}