//--------------------------------------------------------------------------
//! \file
//!
//!  This file contains DVK common ISR code
//!
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/Common/isr.c $
// $Modtime: 2/12/04 12:33p $
// $Revision: 1.4 $
//--------------------------------------------------------------------------
//
// Copyright 2003, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

//--------------------------------------
// Included files
//--------------------------------------

#include "globalparams.h"
#include "psocgpioint.h"
#include "pdc9075.h"
#include "isr.h"
#include "radio.h"



//--------------------------------------
// Local Definitions and Types
//--------------------------------------
//! Number of port shadows
#define NUM_PORTS 0x03


//--------------------------------------
// Local Function Declarations
//--------------------------------------

static void isr_set_port_ie(void);


//--------------------------------------
// Local Definitions
//--------------------------------------
//! Mainly used when enabling and disabling ISR and checking the ISR state
static DVK_ISR enabled_isrs;
//! Port shadows
static UINT8 shadow[NUM_PORTS];



///////////////////////////////////////////////////////////////////////////////
//
//  Function:    isr_gpio
//
//!  Description: Interrupt service routine for general purpose I/O (GPIO).  It
//!               Calls radio_isr() if interrupts are enabled. 
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void isr_gpio(void)
{
    if( isr_is_enabled(INT_RADIO) )
    {
        radio_isr();
    }
}
///////////////////////////////////////////////////////////////////////////////
//
//  Function:    isr_init
//
//!  Description: Disables general purpose I/O (GPIO) interrupts via the 
//!               interrupt enable registers.  Enables GPIO pin interrupts,
//!               interrupt mask and the global interrupt
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void isr_init(void)
{
    enabled_isrs = 0;

    shadow[1] = PORT_1_INTENABLE;
    shadow[2] = PORT_2_INTENABLE;

    // Enable/disable GPIO pin interrupts based upon enabled_isrs
    isr_set_port_ie();

    // Enable GPIO interrupt mask
    M8C_EnableIntMask(INT_MSK0, INT_MSK0_GPIO);
    
    // Enable global interrupt
    M8C_EnableGInt;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    isr_enable
//
//!  Description: Sets the port to enable interrupts.  Used in 
//!               radio_transmit_on() and radio_receive_on()
//!
//!  Inputs:      
//!               - isr - DVK Interrupt Service Routine
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void isr_enable(DVK_ISR isr)
{
#ifndef GPIO_ONLY_ISR
    enabled_isrs |= isr;
    isr_set_port_ie();
#else
	PRT1IE |= IRQ_MASK;         
	INT_MSK0 |= INT_MSK0_GPIO;  
	M8C_EnableGInt;
#endif
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    isr_disable
//
//!  Description: Sets the port to disable interrupts.  Used in 
//!               radio_transmit_on(), radio_receive_on(), and spi_radio_on()
//!
//!  Inputs:      
//!               - isr - DVK Interrupt Service Routine
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void isr_disable(DVK_ISR isr)
{
#ifndef GPIO_ONLY_ISR
    enabled_isrs &= ~isr;
    isr_set_port_ie();
#else
    M8C_DisableGInt;
#endif
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    isr_is_enabled
//
//!  Description: Checks to see if isr is enabled
//!
//!  Inputs:      
//!               - isr - DVK Interrupt Service Routine
//!
//!  Returns:     
//!               - TRUE - If ISR is enabled
//!               - FALSE - If ISR is not enabled
//!
///////////////////////////////////////////////////////////////////////////////
BOOL isr_is_enabled(DVK_ISR isr)
{
    if(enabled_isrs & isr)
    {
        return TRUE;
    }

    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    isr_set_port_ie
//
//!  Description: Sets the ports interrupt enable registers according to what
//!               port shadows are set to.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
static void isr_set_port_ie(void)
{


    // Radio interrupt
    if( enabled_isrs & INT_RADIO )
    {
        shadow[1] |= ( IRQ_MASK );
    }
    else
    {
        shadow[1] &= ~( IRQ_MASK );
    }


    // Set interrupt enable registers
    PRT0IE = shadow[0];
    PRT1IE = shadow[1];
    PRT2IE = shadow[2];
}


