//--------------------------------------------------------------------------
//! \file
//!
//!  Collection of routines to initialize and access the PSOC
//!  ports specifically for the PDC-9075 board
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/DVK Listener/pdc9075.c $
// $Modtime: 3/04/04 2:42p $
// $Revision: 1.4 $
//--------------------------------------------------------------------------
//
// Copyright 2003, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

#include "TX8_1.h"
#ifdef RX_SERIAL_ENABLE
#include "RX8_1.h"
#endif
#include "FlashBlock.h"
#include "spi_clk.h"
#include "radio.h"
#include "pdc9075.h"
#include "timer.h"
#include "debug.h"

//! Pattern array for LED's
const  UINT8 LEDpattern[] = {0xFE, 0xFD, 0xFB, 0xF7, 0xEF, 0xDF, 0xBF, 0x7F,
                             0xBF, 0xDF, 0xEF, 0xF7, 0xFB, 0xFD};
//! For moving to different elements of the LED array
static UINT8 ledIndex;



///////////////////////////////////////////////////////////////////////////////
//
//   Function:    platform_init
//
//!  Description: Initialize the PSOC ports for the PDC-9075 board by setting
//!               up the serial port, initializing ports 4 and 5 for switches,
//!               and by clearing the green and red LEDs
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void platform_init()
{
    SPI_Clk_Start();
    
    // setup the serial port for transmitting
	TX8_1_Start(TX8_PARITY_NONE);
    #ifdef RX_SERIAL_ENABLE
	RX8_1_Start(RX8_PARITY_NONE);
	#endif
    
    // initialize port 4 and 5 for reading switches
    sw_ports_init();
    
    // clear red and green LEDS
    LED_PORT_DR = LEDS_OFF;
}


///////////////////////////////////////////////////////////////////////////////
//
//   Function:    sw_ports_init
//
//!  Description: Initialize ports 4 and 5 for reading switches on
//!               the keypad, joystick, and the selector switch.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void sw_ports_init()
{

    // enable the row pullups
    PRT5DR |= (ROW_0 | ROW_1 | ROW_2 | ROW_3);
    // drive all the columns high
    COL_PORT_DR = (COL_0 | COL_1 | COL_2 | COL_3 | COL_4 | COL_5 | COL_6 | COL_7);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    get_channel
//
//!  Description: Reads the value of the selector switch
//!
//!  Inputs:      None
//!
//!  Returns:     
//!               - result - Value of selector switch position, 0 to 15
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 get_channel(void)
{
	UINT8 result;
	UINT8 colPortShadow;

    // save port 4 configuration
	colPortShadow = COL_PORT_DR;
    
	// drive the common pins low, COL_5/6
	COL_PORT_DR = (COL_0 | COL_1 | COL_2 | COL_3 | COL_4 | COL_7);

    // read the selector selector switch
	result = PRT5DR & (ROW_0 | ROW_1 | ROW_2 | ROW_3);

    // restore port 4 configuration
	COL_PORT_DR = colPortShadow;

    result = (15-result) * 5;

	return result;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    check_buttons_active
//
//!  Description: Scan the gamepad button ports for activity via each column/
//!               row combination and sets the button state dependent upon 
//!               which buttons are held down.  Then restores the port used by
//!               the column
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - 0 - No activity
//!               - 1 - Button down
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 check_buttons_active()
{
	UINT16 buttonState = 0;
	UINT8 index;
	UINT8 row;
	UINT8 col;
	UINT8 mask;
	UINT8 rowBit;
	UINT8 colPortShadow;

	colPortShadow = COL_PORT_DR;
	for(col = 0; col < COLUMNS; ++col)
	{
		// drive the column
		COL_PORT_DR = ~(0x01 << col);
		// read the row
		row = PRT5DR;
		row &= (ROW_0 | ROW_1 | ROW_2 | ROW_3);
		mask = 0x01;
		for(mask = 0x01, rowBit = 0;rowBit < ROWS; ++rowBit, mask <<= 1)
        {
        	index = (col * ROWS) + rowBit;           
            if(!(row & mask))   // down key
            {
             	COL_PORT_DR = colPortShadow;
             	return 1;
            }
 		}
	}
	COL_PORT_DR = colPortShadow;
	
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    activity_check
//
//!  Description: Check for POT activity
//!
//!  Inputs:      
//!               - this - Current POT value 
//!               - last - Previous POT value
//!
//!  Returns:     
//!               - 0 - No activity
//!               - 1 - Activity
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 activity_check(UINT8 this, UINT8 last)
{
    if(this > last)
    {
        if((this - last) > DEADBAND)
        {
            return(1);
        }
    }
    else
    {
        if((last - this) > DEADBAND)
        {
            return(1);
        }
    }
    return(0);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    do_leds
//
//!  Description: Increment and display activity LED's for TX/RX
//!
//!  Inputs:      Void 
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void do_leds(void)
{
	ledIndex++;
	if(ledIndex >= sizeof(LEDpattern))
	{
		ledIndex = 0;
	}
    // light the LED's    
	COL_PORT_DR = LEDpattern[ledIndex];
}

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    bind_pressed
//
//!  Description: Check to see if the bind button is pressed.  
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - 1 - Bind button pressed           
//!               - 0 - Bind button not pressed
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 bind_pressed(void)
{
    UINT8  led_port_shadow;

    led_port_shadow = LED_PORT_DR;        
    if(led_port_shadow &= BUTTON_BIND)
    {
        // clear the button
        led_port_shadow &= ~BUTTON_BIND; 
        LED_PORT_DR = led_port_shadow;
        return 1;
    }
    return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    set_bind_led
//
//!  Description: Turn the bind LED on or off
//!
//!  Inputs:      
//!               - state - ON or OFF.
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void set_bind_led(UINT8 state)
{
    UINT8  led_port_shadow;

    led_port_shadow = LED_PORT_DR;        
    if( state == ON )
    {
        // clear the button
        led_port_shadow &= ~BUTTON_BIND; 

        // turn the LED on
        LED_PORT_DR = led_port_shadow & ~BIND_LED;
    }
    else  // turn it off
    {
        // clear the button
        led_port_shadow &= ~BUTTON_BIND; 

        // turn off the LED
        LED_PORT_DR = led_port_shadow | BIND_LED;
    }
}  

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    set_connect_led
//
//!  Description: Turn the connect LED on or off
//!
//!  Inputs:      
//!               - state - ON or OFF.
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void set_connect_led(UINT8 state)
{
    UINT8  led_port_shadow;

    led_port_shadow = LED_PORT_DR;        
    if( state == ON )
    {
        // clear the button
        led_port_shadow &= ~BUTTON_CONNECT; 

        // turn the LED on
        LED_PORT_DR = led_port_shadow & ~CONNECT_LED;
    }
    else  // turn it off
    {
        // clear the button
        led_port_shadow &= ~BUTTON_CONNECT; 

        // turn off the LED
        LED_PORT_DR = led_port_shadow | CONNECT_LED;
    }
} 

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    xtal_scan_buttons
//
//!  Description: Scans the buttons by first saving the port configuration,
//!               looping through each row column combination and check for a
//!               button that is down.  Before it returns the button index, it 
//!               restores the original port configuration.
//!
//!  Inputs:      None
//!
//!  Returns:     
//!               - index - Index of button that is pressed
//!               - 0xFF  - If no button was pressed
//!
///////////////////////////////////////////////////////////////////////////////
static UINT16 xtal_scan_buttons()
{
	UINT16 buttonState = 0;
	UINT8 index;
	UINT8 row;
	UINT8 col;
	UINT8 mask;
	UINT8 rowBit;
	UINT8 colPortShadow;

	// save port 4's configuration
	colPortShadow = COL_PORT_DR;
    
    //loop through the buttons
	for(col = 0; col < COLUMNS; ++col)
	{
		// drive the column
		COL_PORT_DR = ~(0x01 << col);
		// read the row
		row = PRT5DR;
		row &= (ROW_0 | ROW_1 | ROW_2 | ROW_3);
		mask = 0x01;
		for(mask = 0x01, rowBit = 0;rowBit < ROWS; ++rowBit, mask <<= 1)
        {
        	index = (col * ROWS) + rowBit;           
            if(!(row & mask))   // down key
            {
	            // restore Port 4 config
	            COL_PORT_DR = colPortShadow;
            	return index;
            }
 		}
	}
	// restore Port 4 config
	COL_PORT_DR = colPortShadow;
	return 0xFF;
}

