//--------------------------------------------------------------------------
//!  \file
//!
//!  Collection of functions to access the LS Radio
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/Radio/radio.c $
// $Modtime: 3/11/04 3:52p $
// $Revision: 1.4 $
//--------------------------------------------------------------------------
//
// Copyright 2003-2004, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

//--------------------------------------
// Included files
//--------------------------------------

#include "string.h"
#include "ls_config.h"
#include "radio.h"
#include "globalparams.h"
#include "emc_tester.h"

//--------------------------------------
// Local Definitions and Types
//--------------------------------------
//! Value of gLFSR if gLFSR (Linear Feedback Shift Register) is 0 when
//! generating the 15 bit random value
#define LFSR_SEED 0x1021

//! Initial value of LFSR (Linear Feedback Shift Register) for generating the
//! random value
UINT16 gLFSR = 0;

#ifdef GPIO_ONLY_ISR
#pragma interrupt_handler   radio_isr
#endif


#ifdef PRINT_TX_PACKET
//! Current PN code the radio is on
UINT8 pn_code;
#endif

#ifdef ADJUST_XTAL
//! Current crystal adjust value for the radio
UINT8 gXtal = 0;
#endif

//! Used as a data toggle and to fill unused bytes with random data
UINT8 random_filler = 0;
//! For know what mode the emc_tester is in
extern UINT8 gEMCtesterMode;


//! For keeping track of the last channel used
static UINT8 gLastChannel = 0xFF;


//! Table for the data that configures the radio
const UINT8 radio_init_data[] =
{
    REG_CONTROL, 0,
    REG_DATA_RATE, RADIO_MODE_64,
    REG_CONFIG, IRQ_ACTIVE_HI, 
    REG_THRESHOLD_LO, THRESHOLD_LO_64,
    REG_THRESHOLD_HI, THRESHOLD_HI_64,
#ifdef ADJUST_XTAL
    REG_XTAL_ADJ, (CLOCK_DISABLE | (DEFAULT_XTL & XTAL_ADJ_MASK)),
#else
    REG_XTAL_ADJ, (CLOCK_DISABLE),
#endif
    REG_PA, (PA_BIAS & PA_BIAS_MASK),
    REG_RX_INT_EN, (RX_FULL_A | RX_EOF_A),
    REG_TX_INT_EN, (TX_EMPTY),  
    REG_SERDES_CTL, (SERDES_ENABLE | (EOF_LEN_MASK & EOF_BITS)),
    REG_VALID_TX, 0xFF,
    REG_ANALOG_CTL, (AGC_DISABLE | PACTL_EN),
    REG_AGC_CTL, AGC_OFF,
    REG_SYN_LOCK_CNT, SYNTH_SETTLE_COUNT, // wait for the synth to settle
    REG_CLOCK_ENABLE, 0x41, // set per datasheet
    REG_CLOCK_MANUAL, 0x41 // set per datasheet
};

//! 48 PN codes for a data rate of 16kbps
const UINT8 pn_code_table64[] = 
{
    0x83, 0xF7, 0xA8, 0x2D, 0x7A, 0x44, 0x64, 0xD3,
    0x3F, 0x2C, 0x4E, 0xAA, 0x71, 0x48, 0x7A, 0xC9,
    0x17, 0xFF, 0x9E, 0x21, 0x36, 0x90, 0xC7, 0x82,
    0xA6, 0x46, 0xB5, 0x9A, 0x3A, 0x30, 0xB6, 0xAD,
    0xBC, 0x5D, 0x9A, 0x5B, 0xEE, 0x7F, 0x42, 0xEB,
    0x24, 0xF5, 0xDD, 0xF8, 0x7A, 0x77, 0x74, 0xE7,
    0x3D, 0x70, 0x7C, 0x94, 0xDC, 0x84, 0xAD, 0x95,
    0x1E, 0x6A, 0xF0, 0x37, 0x52, 0x7B, 0x11, 0xD4,
    0x62, 0xF5, 0x2B, 0xAA, 0xFC, 0x33, 0xBF, 0xAF,
    0x40, 0x56, 0x32, 0xD9, 0x0F, 0xD9, 0x5D, 0x97,
    0x8E, 0x4A, 0xD0, 0xA9, 0xA7, 0xFF, 0x20, 0xCA,
    0x38, 0xB3, 0x31, 0xAB, 0x24, 0x78, 0xA6, 0xBD,
    0x4C, 0x97, 0x9D, 0xBF, 0xB8, 0x3D, 0xB5, 0xBE,
    0x0C, 0x5D, 0x24, 0x30, 0x9F, 0xCA, 0x6D, 0xBD,
    0x29, 0x29, 0x7B, 0xB5, 0xC8, 0xF4, 0x4D, 0x8A,
    0x50, 0x14, 0x33, 0xDE, 0xF1, 0x78, 0x95, 0xAD,
    0xD6, 0xA6, 0xAA, 0x10, 0x96, 0xB3, 0xA6, 0xA7,
    0xF6, 0xCE, 0x0D, 0x12, 0xE3, 0x66, 0xA6, 0x94,
    0x0C, 0x3C, 0xFA, 0xF9, 0xF0, 0xF2, 0x10, 0xC9,
    0xF4, 0xDA, 0x06, 0xDB, 0xBF, 0x4E, 0x6F, 0xB3,
    0x93, 0x9C, 0x4E, 0x14, 0x1A, 0x39, 0xCF, 0xE6,
    0xC9, 0x2C, 0x06, 0x93, 0x86, 0xB9, 0x9E, 0xD7,
    0x9E, 0x08, 0xD1, 0xAE, 0x59, 0x5E, 0xE8, 0xF0,
    0xC0, 0x90, 0x8F, 0xBB, 0x7C, 0x8E, 0x2B, 0x8E,
    0x80, 0x69, 0x26, 0x80, 0x08, 0xF8, 0x49, 0xE7,
    0x7D, 0x2D, 0x49, 0x54, 0xD0, 0x80, 0x40, 0xC1,
    0xB6, 0xF2, 0xE6, 0x1B, 0x80, 0x5A, 0x36, 0xB4,
    0x42, 0xAE, 0x9C, 0x1C, 0xDA, 0x67, 0x05, 0xF6,
    0x9B, 0x75, 0xF7, 0xE0, 0x14, 0x8D, 0xB5, 0x80,
    0xBF, 0x54, 0x98, 0xB9, 0xB7, 0x30, 0x5A, 0x88,
    0x35, 0xD1, 0xFC, 0x97, 0x23, 0xD4, 0xC9, 0x88,
    0xE1, 0xD6, 0x31, 0x26, 0x5F, 0xBD, 0x40, 0x93,
    0xDC, 0x68, 0x08, 0x99, 0x97, 0xAE, 0xAF, 0x8C,
    0xC3, 0x0E, 0x01, 0x16, 0x0E, 0x32, 0x06, 0xBA,
    0xE0, 0x83, 0x01, 0xFA, 0xAB, 0x3E, 0x8F, 0xAC,
    0x5C, 0xD5, 0x9C, 0xB8, 0x46, 0x9C, 0x7D, 0x84,
    0xF1, 0xC6, 0xFE, 0x5C, 0x9D, 0xA5, 0x4F, 0xB7,
    0x58, 0xB5, 0xB3, 0xDD, 0x0E, 0x28, 0xF1, 0xB0,
    0x9A, 0xD6, 0x95, 0xBA, 0xA4, 0xC5, 0x32, 0x9C,
    0x5F, 0x30, 0x3B, 0x56, 0x96, 0x45, 0xF4, 0xA1,
    0x03, 0xBC, 0x6E, 0x8A, 0xEF, 0xBD, 0xFE, 0xF8,
    0x88, 0x17, 0x13, 0x3B, 0x2D, 0xBF, 0x06, 0xD6,
    0xF1, 0x94, 0x30, 0x21, 0xA1, 0x1C, 0x88, 0xA9,
    0xD0, 0xD2, 0x8E, 0xBC, 0x82, 0x2F, 0xE3, 0xB4,
    0x8C, 0xFA, 0x47, 0x9B, 0x83, 0xA5, 0x66, 0xD0,
    0x07, 0xBD, 0x9F, 0x26, 0xC8, 0x31, 0x0F, 0xB8,
    0xD7, 0xA1, 0x54, 0xB1, 0x5E, 0x89, 0xAE, 0x86,
    0xEF, 0x03, 0x95, 0x89, 0xB4, 0x71, 0x61, 0x9D,
    0x40, 0xBA, 0x97, 0xD5, 0x86, 0x4F, 0xCC, 0xD1,
};

//! 8 PN codes for a data rate of 32kbps or 64kbps
const UINT8 pn_code_table32[] = 
{
    0x6A, 0xE7, 0x01, 0xEA, 0x03, 0xFD, 0x13, 0xD2, 
    0xDC, 0xC0, 0x6B, 0xB8, 0x2B, 0x09, 0xBB, 0xB2, 
    0xA3, 0x1E, 0xF2, 0xA4, 0x31, 0x32, 0x7A, 0xB3, 
    0x44, 0x83, 0x3B, 0xDD, 0x14, 0xCF, 0x8E, 0xC9, 
    0x35, 0x35, 0x4E, 0xC5, 0xF3, 0x52, 0x47, 0xB0, 
    0x7C, 0x23, 0x8A, 0xCE, 0x45, 0x5C, 0x54, 0xD7, 
    0x81, 0xAC, 0xFB, 0x83, 0x7A, 0x9A, 0x61, 0xAC, 
    0x3C, 0x12, 0x5F, 0x9C, 0x39, 0x98, 0xF6, 0x8A, 
};


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_init
//
//!  Description: Reset and initialize the LS radio
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_init()
{
    UINT8 i;

    SPI_RADIO_ON();

    // load the configuration data
    for(i = 0; i < sizeof(radio_init_data); i+=2)
    {
        SPI_RADIO_PUT(radio_init_data[i], radio_init_data[i+1]);
    }
    
    // use the laser trimmed VCO value 
    SPI_RADIO_PUT( REG_VCO_CAL, MINUS_5_PLUS_5 );
      
#ifdef ADJUST_XTAL
    radio_set_crystal(gXtal);
#endif
    radio_set_pn_code_64( DEFAULT_PN_CODE );
}

#ifdef DEBUG
///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_dump_registers
//
//!  Description: Dump the registers
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_dump_registers()
{
    UINT8 i;
    for(i = 0; i <= 0x3F; i++)
    {
        DEBUG_HEX(SPI_RADIO_GET(i));
        if ((i & 0x0F) == 0x0F) {
            DEBUG_STR("\r\n");
        }
        else {
            DEBUG_CHAR(' ');    
        }
    }
}

#endif

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_receive_on
//
//!  Description: Set up the radio to radio_receive
//!
//!  Inputs:      
//!               - channel - To set the channel for the radio, and to know the
//!               last channel the radio was on
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_receive_on(UINT8 channel)
{
    RADIO_ISR_DISABLE();
    
    // clear the Radio control register
    if (channel != gLastChannel) {
        gLastChannel = channel;
        SPI_RADIO_PUT(REG_CHANNEL, (CHANNEL_MASK & (channel + CHANNEL_OFFSET)));
    }

    SPI_RADIO_GET(REG_DATA_RX_A);

    SPI_RADIO_PUT(REG_CONTROL, (RX_ENABLE | AUTO_SYNTH_COUNT));

    RADIO_ISR_ENABLE();
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_transmit_on
//
//!  Description: Set up the radio to transmit
//!
//!  Inputs:      
//!               - channel - Channel used for transmitting
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_transmit_on(UINT8 channel)
{
    SPI_RADIO_GET(REG_TX_INT_STAT);
    
    // clear the radio control bits
    if (channel != gLastChannel) {
        gLastChannel = channel;
        SPI_RADIO_PUT(REG_CHANNEL, (CHANNEL_MASK & (channel + CHANNEL_OFFSET)));
    }

    SPI_RADIO_PUT(REG_CONTROL, (TX_ENABLE | AUTO_SYNTH_COUNT)); 

    RADIO_ISR_ENABLE();
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_off
//
//!  Description: Turn off the radio Transmitter
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
/////////////////////////////////////////////////////////////////////////////// 
void radio_off()
{
    // disable the PSoC GPIO interrupt
    RADIO_ISR_DISABLE();

    // turn off tx and rx
    SPI_RADIO_PUT(REG_CONTROL, (0));
      
    // clear the tx and rx interrupt status registers
    SPI_RADIO_GET(REG_TX_INT_STAT);
    SPI_RADIO_GET(REG_RX_INT_STAT);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_set_pn_code_64
//
//!  Description: Load the Channel Access Code based on the PN Code index (from
//!               the table used with a data rate of 64). By convention, the 
//!               index must be an even number.
//!
//!  Inputs:      
//!               - pn_code_index - index into the PN Code table, must be an 
//!               even number
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_set_pn_code_64(int pn_code_index)
{
    UINT8 i;

    pn_code_index %= NUM_PN_CODES_64;
    for(i = 0; i < 8; i++)
    {
        SPI_RADIO_PUT(REG_PN_CODE+i, pn_code_table64[(pn_code_index*8)+i]);
    }
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_set_pn_code_32
//
//!  Description: Load the Channel Access Code based on the PN Code index (from
//!               the table used with a data rate of 32). By convention, the 
//!               index must be an even number.
//!
//!  Inputs:      
//!               - pn_code_index - index into the PN Code table, must be an 
//!               even number
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_set_pn_code_32(int pn_code_index)
{
    UINT8 i;

    pn_code_index %= NUM_PN_CODES_32;
    for(i = 0; i < 8; i++)
    {
        SPI_RADIO_PUT(REG_PN_CODE+i, pn_code_table32[(pn_code_index*8)+i]);
    }
}

#ifdef ADJUST_XTAL
///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_set_crystal
//
//!  Description: Sets the crystal adjust value with xtal that is passed in
//!
//!  Inputs:      
//!               - xtal - The crystal adjust value that will be set   
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_set_crystal(UINT8 xtal)
{
    // set the crystal adjustment value 
    SPI_RADIO_PUT(REG_XTAL_ADJ,  (CLOCK_DISABLE | (xtal & XTAL_ADJ_MASK)));
    gXtal = xtal;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_get_crystal
//
//!  Description: Returns the current crystal adjust value
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////

UINT8 radio_get_crystal(void)
{
    return SPI_RADIO_GET(REG_XTAL_ADJ);
}
#endif  // ADJUST_XTAL



///////////////////////////////////////////////////////////////////////////////
//
//  Function:    gen_LFSR
//
//!  Description: Generate a 15 bit LFSR based pseudo random data based on ITU O.151
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - Random Data ranging between 0x02 and 0x7FFF
//!
///////////////////////////////////////////////////////////////////////////////
UINT16 gen_LFSR()
{   
    if( gLFSR != 0 )
    {
        gLFSR = (gLFSR<<1);
        
        // Add stage 14 & 15 and then add in to SR if mod2 is true
        if( ((gLFSR & 0x6000) == 0x2000) ||
            ((gLFSR & 0x6000) == 0x4000) )
        {
            gLFSR |= 0x01;
        }
    }
    else
    {
        gLFSR = LFSR_SEED;
    }
    return(gLFSR & 0x7FFF);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_isr
//
//!  Description: Process the radio interrupt
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void radio_isr()
{
    if( gEMCtesterMode == EMC_TESTER_DATA)
    {             
        // Fill unused bytes with random data
        random_filler = gen_LFSR();
     
        SPI_RADIO_PUT(REG_DATA_TX, random_filler);
    }
    else if( gEMCtesterMode == EMC_TESTER_RECEIVE)
    {
    
        UINT8 irq_source, data, valid;
        
        // poll for a Radio Module interrupt, interrupts are self clearing
        // when the appropiate status register is read 
        while(LS_IRQ_ASSERTED)
        {   
            // read the Radio Rx Interrupt register
            irq_source = SPI_RADIO_GET(REG_RX_INT_STAT);
            
            if((irq_source & 0x0F) == 0) 
            {
                break;
            }
    
            if(irq_source & RX_FULL_A)
            {
                data  = SPI_RADIO_GET(REG_DATA_RX_A);
            }
            
            if(irq_source & RX_EOF_A)
            {               
            }
        }   //while(LS_IRQ_ASSERTED)
    }
}       

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    radio_get_mid
//
//!  Description: Sets radio up and reads the radio's Manufacturing ID (MID).
//!               Then the radio is set up to not read the MID
//!
//!  Inputs:      
//!               - *mid - pointer to a 4-byte buffer to store the MID 
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 radio_get_id( UINT8 id )
{
    UINT8 mid;
    
    // enable the Radio ID registers
    SPI_RADIO_PUT( REG_ANALOG_CTL, AGC_DISABLE | PACTL_EN | MFG_ID_EN );
    
    // read the selected Radio ID register
    mid = SPI_RADIO_GET( REG_MFG_ID_1 + id );
    
    // disable the Radio ID registers
    SPI_RADIO_PUT( REG_ANALOG_CTL, AGC_DISABLE | PACTL_EN );

    return (mid);
}
