//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/Common/timer.c $
// $Modtime: 2/12/04 12:33p $
// $Revision: 1.3 $
//--------------------------------------------------------------------------
//
// Copyright 2003, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------


//--------------------------------------------------------------------------
//!  \file
//!
//!  This file contains DVK common Timer API code.
//!
//!
//! This module provides a timer interface for general purpose timing
//! requirements.
//!
//! Tick requires one 8 bit timer block named "Tick" and configured with the 
//! following parameters:
//!
//!   Clock:            CPU_32_KHz
//!
//!   Capture:          Low
//!
//!   Period:           32
//!
//!   CompareValue:     0
//!
//!   InterruptType:    Terminal Count
//!
//!   Output:        None
//!
//! This will provide a 1ms tick to be used for millisecond resolution 
//! delays.  If the timer calibration feature is enabled, then the period 
//! will be automatically set by the algorithm to meet 1ms timing.
//!
//! tickint.asm should have the following code:
//!
//! TickINT:
//!   ljmp _Tick
//!   reti
//!
//--------------------------------------------------------------------------


//--------------------------------------
// Included files
//--------------------------------------

#include "timer.h"
#include "tick.h"


//--------------------------------------
// Local Function Declarations
//--------------------------------------

#pragma interrupt_handler tick_isr
// Needs external linkage, but not a public API
void tick_isr(void);


//--------------------------------------
// Local Definitions
//--------------------------------------

//! System tick, updated every Tick ISR ( See Tick_ISR in tickint.asm and 
//! tick_ISR() )
TIME_STAMP tick;


///////////////////////////////////////////////////////////////////////////////
//
//   Function:    tick_isr
//
//!  Description: Interrupt service routine for the timer to tick
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void tick_isr(void)
{
    tick++;
}


///////////////////////////////////////////////////////////////////////////////
//
//   Function:    timer_init
//
//!  Description: Initialize the timer by reseting the tick count,  starting the
//!               timer, and enabling the timer interrupt
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void timer_init()
{
    tick = 0;
    Tick_Start();
    Tick_EnableInt();
}


///////////////////////////////////////////////////////////////////////////////
//
//   Function:    timer_delay_10_usec
//
//!  Description: Delays 10 usecs by calling by delaying 1 usec 10 times
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void timer_delay_10_usec(void)
{
    delay1usec();
    delay1usec();
    delay1usec();
    delay1usec();
    delay1usec();
    delay1usec();
    delay1usec();
    delay1usec();
    delay1usec();
    delay1usec();
}

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    timer_delay_50_usec
//
//!  Description: Delays 50 usecs by calling by delaying 10 usecs 5 times
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void timer_delay_50_usec(void)
{
    timer_delay_10_usec();
    timer_delay_10_usec();
    timer_delay_10_usec();
    timer_delay_10_usec();
    timer_delay_10_usec();
}

///////////////////////////////////////////////////////////////////////////////
//
//   Function:    timer_delay_msec
//
//!  Description: Delays the number of milliseconds specified by the 
//!               num_milliseconds parameter. First it takes a time stamp and
//!               adds the number of milliseconds to delay (num_milliseconds).
//!               Then it enters a while loop where each iteration puts the  
//!               PSoC in sleep mode and is equivalent to one millisecond.  Or
//!               if GPIO_ONLY_ISR is defined, then it will delay 1 msec by
//!               delaying 50 usecs 20 times.
//!
//!  Inputs:      
//!               - num_milliseconds - Number of milliseconds to delay
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void timer_delay_msec(UINT16 num_milliseconds)
{
#ifndef GPIO_ONLY_ISR
    TIME_STAMP cv;
    
    cv = tick + MS_TO_TICK(num_milliseconds);
    
    while( tick <= cv )
    {
        //M8C_Sleep;
        timer_sleep_psoc();  // new macro for HW workaround
    }
#else
    UINT8 i;
    
    for(i=0; i<num_milliseconds; i++)
    {
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
        TIMER_DELAY_50_USEC();
    }
#endif
}


///////////////////////////////////////////////////////////////////////////////
//
//   Function:    timer_get_time_stamp
//
//!  Description: Returns the number of ticks the timer has gone through
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - tick - Number of ticks
//!
///////////////////////////////////////////////////////////////////////////////
TIME_STAMP timer_get_time_stamp(void)
{
    return tick;
}


///////////////////////////////////////////////////////////////////////////////
//
//   Function:    timer_delay_incremental 
//
//!  Description: To delay the timer for a specific type of device (mouse, 
//!                     keypad, gamepad, etc.)
//!
//!  Inputs:      
//!               - from_timestamp - Timestamp to help figure the correct delta
//!                     for the delay
//!               - delay_ms - The base amount of time to figure delta
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void timer_delay_incremental( TIME_STAMP from_timestamp, UINT16 delay_ms )
{

    UINT16 delta;
    
    
    delta = TICK_TO_MS( MS_TO_TICK(delay_ms) - (tick - from_timestamp) );
    
    if( (INT16)delta > 0 )
    {
        timer_delay_msec( delta );
    }
    else
    {
        delta = 0; // breakpoint location for bench test
    }
}


///////////////////////////////////////////////////////////////////////////////
//
//   Function:    timer_time_elapsed
//
//!  Description: Another kind of delay that is generally called within a
//!                     conditional statement  
//!               (i.e. while(!timer_time_elapsed(current_time, MFG_RX_TIMEOUT) ); ) 
//!
//!  Inputs:      
//!               - from_timestamp - number of milliseconds to start with
//!               - elapsed_time_ms - number of milliseconds needed to reach
//!                     the elapsed time
//!               
//!
//!  Returns:     
//!               - True - If elapsed time has been reached 
//!               - False - If elapsed time has NOT been reached
//!
///////////////////////////////////////////////////////////////////////////////
BOOL timer_time_elapsed(TIME_STAMP from_timestamp, UINT16 elapsed_time_ms)
{
    if( tick > (from_timestamp + MS_TO_TICK(elapsed_time_ms)) )
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}


