;****************************************************************************
;
;   file:           regulator.asm
;   Description:    This file contains the regulator firmware for Telec, FCC, 
;                   and ETSI certification
;
;   External Functions:
;                   start_transmit_carrier
;                   stop_transmit_carrier
;                   start_transmit_data
;                   stop_transmit_data
;					start_receive
;                   stop_receive
;
;   Target:         Cypress CY7C63723/CY7C63743 Encore Chips
;   $Header:
;   Version:        1.3.0000
;
;****************************************************************************
; Copyright (2003), Cypress Semiconductor Corporation
; This software is owned by Cypress Semiconductor Corporation (Cypress) and  is
; protected by United States copyright laws and international treaty provisions.
; Cypress hereby grants to Licensee a personal, non-exclusive, non-transferable
; license to copy, use, modify, create derivative works of, and compile the
; Cypress Source Code and derivative works for the sole purpose of creating
; custom software in support of Licensee product ("Licensee Product") to be used
; only in conjunction with a Cypress integrated circuit. Any reproduction,
; modification, translation, compilation, or representation of this software
; except as specified above is prohibited without the express written permission
; of Cypress.
; Disclaimer: Cypress makes no warranty of any kind, express or implied, with
; regard to this material, including, but not limited to, the implied warranties
; of merchantability and fitness for a particular purpose. Cypress reserves the
; right to make changes without further notice to the materials described herein.
; Cypress does not assume any liability arising out of the application or use of
; any product or circuit described herein. Cypress products described herein are
; not authorized for use as components in life-support devices.
; This software is protected by and subject to worldwide patent coverage,
; including U.S. and foreign patents. Use may be limited by and subject to the
; Cypress Software License Agreement.
;
;****************************************************************************
REGULATOR_IDLE:     equ 0
REGULATOR_CARRIER:  equ 1
REGULATOR_DATA:     equ 2
REGULATOR_RECEIVE:  equ 3

CHANNEL_BOTTOM:     equ 0
CHANNEL_MIDDLE:     equ 38
CHANNEL_TOP:        equ 77

DATA_RATE_16:       equ 1
DATA_RATE_32:       equ 2
DATA_RATE_64:       equ 3   

MAX_PA_BIAS:        equ 7
DEBUG_PACKET_SIZE:  equ 5
DEBUG_PACKET_DELAY: equ 5

regulator_state:    equ C0h
lfsr_high:          equ C1h
lfsr_low:           equ C2h
data_rate:          equ C3h
pa_bias:            equ C4h
;temp_lfsr_high:     equ C3h

interface_report:   equ C5h
usb_data:           equ C6h
USB_REPORT_RX:      equ FFh
NO_USB_REPORT:      equ 00h

USB_DATA_NONE:                  equ 00
USB_DATA_TRANSMIT_CARRIER:      equ 01
USB_DATA_TRANSMIT_DATA:         equ 02
USB_DATA_RECEIVE:               equ 03
USB_DATA_STOP_TEST:             equ 04
USB_DATA_INCREMENT_CHANNEL:     equ 05
USB_DATA_DECREMENT_CHANNEL:     equ 06
USB_DATA_INCREMENT_PN_CODE:     equ 07
USB_DATA_DECREMENT_PN_CODE:     equ 08
USB_DATA_INCREMENT_DATA_RATE:   equ 09
USB_DATA_DECREMENT_DATA_RATE:   equ 10
USB_DATA_INCREMENT_PA_BIAS:     equ 11
USB_DATA_DECREMENT_PA_BIAS:     equ 12



;**********************************************************
;
;       regulator_init
;
;**********************************************************
regulator_mode:
    MOV  A, CHANNEL_BOTTOM
    MOV  [channel], A

    MOV  A, DATA_RATE_64
    MOV  [data_rate], A

    MOV  A, PA_BIAS
    MOV  [pa_bias], A

    MOV  A, NO_USB_REPORT
    MOV  [interface_report], A
    MOV  A, USB_DATA_NONE
    MOV  [usb_data], A

    CALL radio_init
    
    MOV  A, 0
    MOV  [pn_code], A
    CALL radio_load_pn_code

    regulator_loop:
    CALL process_test

    ; check for USB commands
    DI
    CALL process_input
    EI

    CALL regulator_leds
    CALL delay50us

    JMP  regulator_loop


;**********************************************************
;
;       process_input
;
;**********************************************************
process_input:
IFNDEF TELEC_USB_CONTROL
    CALL check_for_bind         ; if bind button is pushed, check_for_bind returns A non-zero A
    AND  A, ffh
    JZ   process_input_complete

    CALL increment_channel

process_input_complete:
ELSE
    MOV  A, [usb_data]
    CMP  A, USB_DATA_NONE
    JZ   process_input_complete

    ; USB command has been received... process command  (but first clear it)
    MOV  [ls_temp], A
    MOV  A, 0
    MOV  [usb_data], A
    MOV  A, [ls_temp]

    CMP  A, USB_DATA_TRANSMIT_CARRIER
    JZ   start_transmit_carrier

    CMP  A, USB_DATA_TRANSMIT_DATA
    JZ   start_transmit_data

    CMP  A, USB_DATA_RECEIVE
    JZ   start_receive

    CMP  A, USB_DATA_STOP_TEST
    JZ   reset_test

    CMP  A, USB_DATA_INCREMENT_CHANNEL
    JZ   increment_channel

    CMP  A, USB_DATA_DECREMENT_CHANNEL
    JZ   decrement_channel

    CMP  A, USB_DATA_INCREMENT_PN_CODE
    JZ   increment_pn_code

    CMP  A, USB_DATA_DECREMENT_PN_CODE
    JZ   decrement_pn_code

    CMP  A, USB_DATA_INCREMENT_DATA_RATE
    JZ   increment_data_rate

    CMP  A, USB_DATA_DECREMENT_DATA_RATE
    JZ   decrement_data_rate

    CMP  A, USB_DATA_INCREMENT_PA_BIAS
    JZ   increment_pa_bias

    CMP  A, USB_DATA_DECREMENT_PA_BIAS
    JZ   decrement_pa_bias

process_input_complete:
ENDIF
    RET


;**********************************************************
;
;       start_transmit_carrier
;
;**********************************************************
start_transmit_carrier:
    CALL stop_test

    MOV  A, REGULATOR_CARRIER
    MOV  [regulator_state], A

    CALL setup_tx
    
    ;SPI_RADIO_PUT(REG_CLOCK_ENABLE, 0x51);
    MOV  A, REG_CLOCK_ENABLE
    MOV  [reg_addr], A
    MOV  A, 51h
    MOV  [reg_data], A
    CALL SPI_write

    ;SPI_RADIO_PUT(REG_CLOCK_MANUAL, 0x41);
    MOV  A, REG_CLOCK_MANUAL
    MOV  [reg_addr], A
    MOV  A, 41h
    MOV  [reg_data], A
    CALL SPI_write

    RET

;**********************************************************
;
;       stop_transmit_carrier
;
;**********************************************************
stop_transmit_carrier:
    CALL radio_idle

    MOV  A, REG_CLOCK_ENABLE
    MOV  [reg_addr], A
    MOV  A, 41h
    MOV  [reg_data], A
    CALL SPI_write

    RET


;**********************************************************
;
;       start_transmit_data
;
;**********************************************************
start_transmit_data:
    CALL stop_test

    MOV  A, REGULATOR_DATA
    MOV  [regulator_state], A

    CALL setup_tx
    CALL generate_lfsr              ; generate random data for the next transmission

    MOV  A, 0
    MOV  [tx_sys_counter], A

    RET

;**********************************************************
;
;       process_transmit_data
;
;**********************************************************
process_transmit_data:
IFDEF DEBUG_TRANSMIT_DATA
    MOV  A, [tx_sys_counter]
    CMP  A, DEBUG_PACKET_SIZE+2
    JNC  process_transmit_data_timer         ; reset app_timer and increment tx_sys_counter
    MOV  A, 0
    MOV  [app_timer], A
    JMP  process_transmit_data_debug_complete

process_transmit_data_timer:
    MOV  A, [app_timer]
    CMP  A, DEBUG_PACKET_DELAY
    JC   process_transmit_data_complete

    MOV  A, 0
    MOV  [tx_sys_counter], A

process_transmit_data_debug_complete:
ENDIF

    ; if EOF, load a byte
    IORD port0
    AND  A, IRQ
    JZ   process_transmit_data_complete
    
process_transmit_data_load:
    MOV  A, REG_TX_INT_STAT
    MOV  [reg_addr], A
    CALL SPI_read

    MOV  A, REG_TX_DATA             
    MOV  [reg_addr], A              
    MOV  A, [lfsr_low]
    MOV  [reg_data], A
    CALL SPI_write

    CALL generate_lfsr              ; generate random data for the next transmission
    INC  [tx_sys_counter]    

process_transmit_data_complete:
    RET

;**********************************************************
;
;       stop_transmit_data
;
;**********************************************************
stop_transmit_data:
    CALL radio_idle
    RET


;**********************************************************
;
;       start_receive
;
;**********************************************************
start_receive:
    CALL stop_test
   
    MOV  A, REGULATOR_RECEIVE
    MOV  [regulator_state], A

    CALL setup_rx
    RET


;**********************************************************
;
;       process_receive
;
;**********************************************************
process_receive:
    ; if EOF receive a byte
    IORD port0
    AND  A, IRQ
    JZ   process_receive_complete
    CALL process_rx_int                  

process_receive_complete:
    RET

;**********************************************************
;
;       stop_receive
;
;**********************************************************
stop_receive:
    CALL radio_idle
    RET



;**********************************************************
;
;       restart_test
;
;**********************************************************
restart_test:
    MOV  A, [regulator_state]
    CMP  A, REGULATOR_CARRIER
    JZ   start_transmit_carrier
    CMP  A, REGULATOR_DATA
    JZ   start_transmit_data
    CMP  A, REGULATOR_RECEIVE
    JZ   start_receive

    RET


;**********************************************************
;
;       process_test
;
;**********************************************************
process_test:
    MOV  A, [regulator_state]
    CMP  A, REGULATOR_DATA
    JZ   process_transmit_data
    CMP  A, REGULATOR_RECEIVE
    JZ   process_receive

    RET



;**********************************************************
;
;       stop_test
;
;**********************************************************
stop_test:
    MOV  A, [regulator_state]
    CMP  A, REGULATOR_CARRIER
    JZ   stop_transmit_carrier
    CMP  A, REGULATOR_DATA
    JZ   stop_transmit_carrier
    CMP  A, REGULATOR_RECEIVE
    JZ   stop_receive

    RET

;**********************************************************
;
;       reset_test
;
;**********************************************************
reset_test:
    CALL stop_test
    MOV  A, REGULATOR_IDLE
    MOV  [regulator_state], A

    RET


;**********************************************************
;
;       increment_channel
;
;**********************************************************
increment_channel:
    CALL stop_test
IFNDEF TELEC_ALL_CHANNELS
    MOV  A, [channel]

increment_channel_bottom:
    CMP  A, CHANNEL_BOTTOM
    JNZ  increment_channel_middle
    MOV  A, CHANNEL_MIDDLE
    MOV  [channel], A
    JMP  increment_channel_complete

increment_channel_middle:
    CMP  A, CHANNEL_MIDDLE
    JNZ  increment_channel_top
    MOV  A, CHANNEL_TOP
    MOV  [channel], A
    JMP  increment_channel_complete

increment_channel_top:
    CMP  A, CHANNEL_TOP
    JNZ  increment_channel_top
    MOV  A, CHANNEL_BOTTOM
    MOV  [channel], A
ELSE
    INC  [channel]
    MOV  A, [channel]
    CMP  A, NUM_CHANNELS
    JC   increment_channel_complete
    MOV  A, 0
    MOV  [channel], A
ENDIF   ; TELEC_ALL_CHANNELS

increment_channel_complete:
    CALL restart_test
    RET


;**********************************************************
;
;       decrement_channel
;
;**********************************************************
decrement_channel:
    CALL stop_test
IFNDEF TELEC_ALL_CHANNELS
    MOV  A, [channel]

decrement_channel_bottom:
    CMP  A, CHANNEL_BOTTOM
    JNZ  decrement_channel_middle
    MOV  A, CHANNEL_TOP
    MOV  [channel], A
    JMP  decrement_channel_complete

decrement_channel_middle:
    CMP  A, CHANNEL_MIDDLE
    JNZ  decrement_channel_top
    MOV  A, CHANNEL_BOTTOM
    MOV  [channel], A
    JMP  decrement_channel_complete

decrement_channel_top:
    CMP  A, CHANNEL_TOP
    JNZ  decrement_channel_top
    MOV  A, CHANNEL_MIDDLE
    MOV  [channel], A
ELSE
    DEC  [channel]
    JNC  decrement_channel_complete
    MOV  A, NUM_CHANNELS-1
    MOV  [channel], A
ENDIF ; TELEC_ALL_CHANNELS

decrement_channel_complete:
    CALL restart_test
    RET

;**********************************************************
;
;       increment_pn_code
;
;**********************************************************
increment_pn_code:
    CALL stop_test
    MOV  A, [data_rate]
    CMP  A, DATA_RATE_16
    JNZ  increment_pn_code_32
    INC  [pn_code]
    MOV  A, [pn_code]
    CMP  A, NUM_PN_CODES_64
    JC   increment_pn_code_complete
    ; code is too large, wrap to 0
    MOV  A, 0
    MOV  [pn_code], A
    JMP  increment_pn_code_complete

increment_pn_code_32:
    INC  [pn_code]
    MOV  A, [pn_code]
    CMP  A, NUM_PN_CODES_32
    JC   increment_pn_code_complete
    ; code is too large, wrap to 0
    MOV  A, 0
    MOV  [pn_code], A

increment_pn_code_complete:
    ; load the PN code
    CALL radio_load_pn_code
    CALL restart_test
    RET

;**********************************************************
;
;       decrement_pn_code
;
;**********************************************************
decrement_pn_code:
    CALL stop_test
    DEC  [pn_code]
    JNC  decrement_pn_code_complete

    MOV  A, [data_rate]
    CMP  A, DATA_RATE_16
    JNZ  decrement_pn_code_32

    MOV  A, NUM_PN_CODES_64-1
    MOV  [pn_code], A

decrement_pn_code_32:
    MOV  A, NUM_PN_CODES_32-1
    MOV  [pn_code], A

decrement_pn_code_complete:
    ; load the PN code
    CALL radio_load_pn_code
    CALL restart_test
    RET

;**********************************************************
;
;       increment_pa_bias
;
;**********************************************************
increment_pa_bias:
    MOV  A, [pa_bias]
    CMP  A, MAX_PA_BIAS
    JZ   increment_pa_bias_complete

    CALL stop_test
    INC  [pa_bias]

    MOV  A, REG_PA
    MOV  [reg_addr], A
    MOV  A, [pa_bias]
    AND  A, mPA_BIAS
    MOV  [reg_data], A
    CALL SPI_write
    CALL restart_test

increment_pa_bias_complete:
    RET

;**********************************************************
;
;       decrement_pa_bias
;
;**********************************************************
decrement_pa_bias:
    MOV  A, [pa_bias]
    CMP  A, 0
    JZ   decrement_pa_bias_complete

    CALL stop_test
    DEC  [pa_bias]

    MOV  A, REG_PA
    MOV  [reg_addr], A
    MOV  A, [pa_bias]
    AND  A, mPA_BIAS
    MOV  [reg_data], A
    CALL SPI_write
    CALL restart_test

decrement_pa_bias_complete:
    RET

;**********************************************************
;
;       increment_data_rate
;
;**********************************************************
increment_data_rate:
    CALL stop_test
    MOV  A, [data_rate]

increment_data_rate_16:
    CMP  A, DATA_RATE_16
    JNZ  increment_data_rate_32
    MOV  A, DATA_RATE_32
    MOV  [data_rate], A
    MOV  A, [pn_code]
    CMP  A, NUM_PN_CODES_32
    JC   increment_data_rate_16_reset_pn_code
    MOV  A, NUM_PN_CODES_32-1
    MOV  [pn_code], A

increment_data_rate_16_reset_pn_code:
    CALL radio_load_pn_code
    MOV  A, REG_DATA_RATE
    MOV  [reg_addr], A
    MOV  A, RADIO_MODE_32
    MOV  [reg_data], A
    CALL SPI_write
    JMP  increment_data_rate_complete

increment_data_rate_32:
    CMP  A, DATA_RATE_32
    JNZ  increment_data_rate_64
    MOV  A, DATA_RATE_64
    MOV  [data_rate], A
    MOV  A, REG_DATA_RATE
    MOV  [reg_addr], A
    MOV  A, RADIO_MODE_64
    MOV  [reg_data], A
    CALL SPI_write
    JMP  increment_data_rate_complete

increment_data_rate_64:
    CMP  A, DATA_RATE_64
    JNZ  increment_data_rate_complete
    MOV  A, DATA_RATE_16
    MOV  [data_rate], A
    CALL radio_load_pn_code
    MOV  A, REG_DATA_RATE
    MOV  [reg_addr], A
    MOV  A, RADIO_MODE_16
    MOV  [reg_data], A
    CALL SPI_write

increment_data_rate_complete:
    CALL restart_test
    RET

;**********************************************************
;
;       decrement_data_rate
;
;**********************************************************
decrement_data_rate:
    CALL stop_test
    MOV  A, [data_rate]

decrement_data_rate_16:
    CMP  A, DATA_RATE_16
    JNZ  decrement_data_rate_32
    MOV  A, DATA_RATE_64
    MOV  [data_rate], A
    MOV  A, [pn_code]
    CMP  A, NUM_PN_CODES_32
    JC   decrement_data_rate_16_reset_pn_code
    MOV  A, NUM_PN_CODES_32-1
    MOV  [pn_code], A

decrement_data_rate_16_reset_pn_code:
    CALL radio_load_pn_code
    MOV  A, REG_DATA_RATE
    MOV  [reg_addr], A
    MOV  A, RADIO_MODE_64
    MOV  [reg_data], A
    CALL SPI_write
    JMP  decrement_data_rate_complete

decrement_data_rate_32:
    CMP  A, DATA_RATE_32
    JNZ  decrement_data_rate_64
    MOV  A, DATA_RATE_16
    MOV  [data_rate], A
    CALL radio_load_pn_code
    MOV  A, REG_DATA_RATE
    MOV  [reg_addr], A
    MOV  A, RADIO_MODE_16
    MOV  [reg_data], A
    CALL SPI_write
    JMP  decrement_data_rate_complete

decrement_data_rate_64:
    CMP  A, DATA_RATE_64
    JNZ  decrement_data_rate_complete
    MOV  A, DATA_RATE_32
    MOV  [data_rate], A
    MOV  A, REG_DATA_RATE
    MOV  [reg_addr], A
    MOV  A, RADIO_MODE_32
    MOV  [reg_data], A
    CALL SPI_write

decrement_data_rate_complete:
    CALL restart_test
    RET


;**********************************************************
;
;       generate_lfsr
;
;**********************************************************
generate_lfsr:
    MOV   A, 0
    CMP   A, [lfsr_high]
    JNZ   generate_lfsr_shift
    CMP   A, [lfsr_low]
    JNZ    generate_lfsr_shift

generate_lfsr_first_use:
    MOV   A, 21h
    MOV   [lfsr_low], A
    MOV   A, 10h
    MOV   [lfsr_high], A
    RET

generate_lfsr_shift:
    MOV   A, [lfsr_low]
    ASL   A
    MOV   [lfsr_low], A
    MOV   A, [lfsr_high]
    RLC   A
    MOV   [lfsr_high], A

generate_lfsr_check1:
    MOV   A,[lfsr_high]
    AND   A, 60h
    JZ    generate_lfsr_complete
    CMP   A, 60h
    JZ    generate_lfsr_complete

generate_lfsr_or1:
    MOV   A, [lfsr_low]
    OR    A, 1
    MOV   [lfsr_low], A

generate_lfsr_complete:
    MOV   A,[lfsr_high]
    AND   A,7Fh
    MOV   [lfsr_high],A
    RET   


;**********************************************************
;
;       increment_test
;
;**********************************************************
increment_test:
    CALL stop_test
    MOV  A, [regulator_state]

increment_test_carrier:
    CMP  A, REGULATOR_CARRIER
    JNZ  increment_test_data
    MOV  A, REGULATOR_DATA
    MOV  [regulator_state], A
    JMP  increment_test_complete

increment_test_data:
    CMP  A, REGULATOR_DATA
    JNZ  increment_test_data
    MOV  A, REGULATOR_RECEIVE
    MOV  [regulator_state], A
    JMP  increment_test_complete

increment_test_receive:
    CMP  A, REGULATOR_RECEIVE
    JNZ  increment_test_data
    MOV  A, REGULATOR_CARRIER
    MOV  [regulator_state], A

increment_test_complete:
    CALL restart_test
    RET






;**********************************************************
;
;       increment_test
;
;**********************************************************

regulator_leds:
    MOV  A, [regulator_state]
    CMP  A, REGULATOR_IDLE
    JZ   regulator_leds_idle
    CMP  A, REGULATOR_CARRIER
    JZ   regulator_leds_carrier
    CMP  A, REGULATOR_DATA
    JZ   regulator_leds_data
    CMP  A, REGULATOR_RECEIVE
    JZ   regulator_leds_receive
    RET

regulator_leds_data:
    IOWR watchdog
    MOV  A, [timer]
    AND  A, 80h
    JNZ  regulator_leds_data_toggle

    set_port_0 nDATA_LED
    clr_port_1 nBIND_LED
    RET

regulator_leds_data_toggle:
    clr_port_0 nDATA_LED
    set_port_1 nBIND_LED
    RET

regulator_leds_idle:
    clr_port_0 nDATA_LED
    set_port_1 nBIND_LED
    RET


regulator_leds_carrier:
    set_port_0 nDATA_LED
    clr_port_1 nBIND_LED
    RET

regulator_leds_receive:
    clr_port_0 nDATA_LED
    clr_port_1 nBIND_LED
    RET