;****************************************************************************
;
;   file: wusb-ls-main.asm 
;   Description: This code acts as a bridge for a WirelessUSB mouse and keyboard
;   Target: Cypress CY7C63743
;   Version:        1.3.0000
;
;   This code is based on the logo.asm fw 
;
;   Overview:  
;       This firmware executes A single simple program loop.
;       On reset, the firmware initializes the radio and then
;       waits until enumerated by the USB host.
;       It then continuously polls the radio "data ready" 
;       input, and when active it reads and processes
;       radio received data. If valid, the received data
;       is loaded into the EP1 FIFO, and EP1 is armed
;       for transmission.
;                  
;   USB:
;       At bus reset the USB interface is re-initialized,
;       AND  the firmware soft reboots.  We are then able to
;       handle the standard chapter nine requests on 
;       endpoint zero (the control endpoint).  After this
;       device enumerates as A HID mouse on the USB, the
;       requests come to endpoint one (the data endpoint).
;       Endpoint one is used to send mouse displacement AND 
;       button status information.  
;
;   Pin Connections:
;
;                -------------------
;               | P0[0]     P0[4]   |
; RX Data Rdy   | P0[1]     P0[5]   |
;               | P0[2]     P0[6]   |
;               | P0[3]     P0[7]   |
;               | P1[0]     P1[1]   |   
;               | P1[2]     P1[3]   |
;               | P1[4]     P1[5]   |
;               | P1[6]     P1[7]   |
;       GND     | VSS       D+/SCLK |   USB D+ / PS2 SCLK
;       GND     | VPP       D-/SDATA|   USB D- / PS2 SDATA
;   PULLUP      | VREG      VCC     |   +5V
;               | XTALIN    XTALOUT |
;                -------------------
;
; Revisions:
;
;****************************************************************************
; Copyright (2003), Cypress Semiconductor Corporation
; This software is owned by Cypress Semiconductor Corporation (Cypress) and  is
; protected by United States copyright laws and international treaty provisions.
; Cypress hereby grants to Licensee a personal, non-exclusive, non-transferable
; license to copy, use, modify, create derivative works of, and compile the
; Cypress Source Code and derivative works for the sole purpose of creating
; custom software in support of Licensee product ("Licensee Product") to be used
; only in conjunction with a Cypress integrated circuit. Any reproduction,
; modification, translation, compilation, or representation of this software
; except as specified above is prohibited without the express written permission
; of Cypress.
; Disclaimer: Cypress makes no warranty of any kind, express or implied, with
; regard to this material, including, but not limited to, the implied warranties
; of merchantability and fitness for a particular purpose. Cypress reserves the
; right to make changes without further notice to the materials described herein.
; Cypress does not assume any liability arising out of the application or use of
; any product or circuit described herein. Cypress products described herein are
; not authorized for use as components in life-support devices.
; This software is protected by and subject to worldwide patent coverage,
; including U.S. and foreign patents. Use may be limited by and subject to the
; Cypress Software License Agreement.
;
;****************************************************************************

;---------------- assembler directives -------------------- 

INCLUDE "config.inc"            ; includes protocol configuration

IFDEF PDC_9075
    CPU 63743
ENDIF

IFDEF PDC_9168
    CPU 63723
ENDIF

XPAGEON

INCLUDE "637xx.inc"
INCLUDE "wusb-ls-headers.inc"


;--------------- interrupt vector table -------------------

ORG 00h         

JMP  reset               ; reset vector      
JMP  bus_reset           ; bus reset interrupt
JMP  error               ; 128us interrupt
JMP  ms_timer            ; 1.024ms interrupt
JMP  endpoint0_stub      ; endpoint 0 interrupt
JMP  error			     ; endpoint 1 interrupt
JMP  error               ; endpoint 2 interrupt
JMP  error               ; reserved
JMP  error               ; Capture timer A interrupt Vector
JMP  error               ; Capture timer B interrupt Vector
JMP  error               ; GPIO interrupt
JMP  wakeup_isr          ; Wake-up interrupt vector 


;-------------- program listing ---------------------------

ORG  1Ah
error: JMP reset

;-------------------------------------------------------
;   MACROS
;-------------------------------------------------------
MACRO set_port_0 mask
    MOV  A, mask
    OR   [port0Shadow], A
    MOV  A, [port0Shadow]
    IOWR port0
ENDM

MACRO clr_port_0 mask
    MOV  A, ~mask
    AND  [port0Shadow], A
    MOV  A, [port0Shadow]
    IOWR port0
ENDM

MACRO set_port_1 mask
    MOV  A, mask
    OR   [port1Shadow], A
    MOV  A, [port1Shadow]
    IOWR port1
ENDM

MACRO clr_port_1 mask
    MOV  A, ~mask
    AND  [port1Shadow], A
    MOV  A, [port1Shadow]
    IOWR port1
ENDM


endpoint0_stub:                     ; cannot CALL directly from the interrupt table
    CALL endpoint0                  ; into the upper 4K block
    RETI
    
;----------------------------------------------------------
;
;   Interrupt handler: wakeup_isr
;
;----------------------------------------------------------
wakeup_isr:
RETI

;----------------------------------------------------------
;
;   Interrupt handler: bus_reset
;   Purpose: The program jumps to this routine when
;        the microcontroller has A bus reset.
;
;----------------------------------------------------------

bus_reset:
    MOV  A, STALL_IN_OUT             ; set to STALL INs&OUTs
    IOWR ep0_mode

    MOV  A, ADDRESS_ENABLE           ; enable USB address 0
    IOWR usb_address
    MOV  A, DISABLE                  ; disable endpoint1
    IOWR ep1_mode
    IOWR ep2_mode

    MOV  A, 00h                      ; reset program stack pointer
    MOV  psp,A   

    JMP  reset


;----------------------------------------------------------
;
;   Interrupt: ms_timer
;   Purpose: Every 1ms this interrupt handler clears
;       the watchdog timer.
;
;----------------------------------------------------------

ms_timer:
    PUSH A
    PUSH X

    IOWR watchdog
    
    INC  [app_timer]
    INC  [timer]                ; INC  1ms counter
	INC  [led_timer]
	JNC  ms_suspend_timer
	MOV  A, f0h					; prevent led_timer from overflowing
	MOV  [led_timer], A

ms_suspend_timer:                  
    IORD usb_status              ; read bus activity bit
    AND  A, BUS_ACTIVITY         ; mask off activity bit
    JNZ  bus_activity
        
    INC  [suspend_count]         ; increment suspend counter
    MOV  A, [suspend_count]
    CMP  A, 04h                  ; no bus activity for 4ms, suspend
    JNC  prepare_suspend
    JMP  ms_timer_done

prepare_suspend:
	; We get here if we need to go into suspend mode
    CALL radio_suspend
    CALL encore_suspend

	; Put the enCoRe into suspend    
usb_suspend:
IFDEF USE_GPIO_INT
    MOV  A, 00h                      ; disable radio IRQ P0.1
    IOWR port0_int
ENDIF
                                    ; Put the enCoRe into suspend    
    MOV  A, [remote_wakeup]         ; if remote_wakeup enabled
    CMP  A, 01h
    JNZ  no_wake_up_int

    MOV  A, (USB_RESET_INT | WAKEUP_INT)      ; wake on USB_RESET_INT
	JMP  global_i
    
no_wake_up_int:
    MOV  A, (USB_RESET_INT)      ; wake on USB_RESET_INT

	global_i:
    IOWR global_int    

    IORD control
    OR   A, (SUSPEND|RUN)        ; set suspend bit
    AND  A, 11001111b            ; clear the RESET bits
    EI
    IOWR control

    ;;; IN SUSPEND STATE HERE ;;;

    NOP      ; This is first instruction executed after woken up

	; Stay here if waking up while RESET
    IORD    control
    AND     A, 00110000b          ; check for reset and brownout reset (just in case)
    JZ      check_bus_activity

    MOV  A, 00h
    MOV  [remote_wakeup], A       ; disable remote wakeup

wait_reset:
    IOWR    watchdog
    JMP     wait_reset



;**************************************************************
; This section gets called during suspend due to the wake up
; timer. The bridge initiates a resume under the following
; conditions:
; 1. resume from system (bus activity)
; 2. resume from the bind button (force resume functionality)
; 3. resume from wireless device
;**************************************************************

check_bus_activity:                 ; if bus activity, exit suspend
    IORD usb_status
    AND  A, BUS_ACTIVITY
    JNZ  prepare_resume
        
while_host_is_suspended:
;DEFINE RESUME_FROM_BIND_BUTTON
IFDEF RESUME_FROM_BIND_BUTTON        
    CALL check_for_bind
    AND  A, ffh
    JNZ  prepare_resume
ENDIF

    CALL rwu_poll_radio
    CMP  A, 01h
    JNZ  do_rwu_resume				   
    JMP  usb_suspend

prepare_resume:
    CALL encore_resume
    CALL radio_resume
    JMP  do_rwu_resume           		; saw wake up event

do_rwu_resume:
    MOV  A, [remote_wakeup]      		; if remote_wakeup enabled
    CMP  A, 01h
    JNZ  back_in_business
    CALL signal_usb_resume

    MOV   A,NAK_IN                  
    IOWR  ep1_mode


back_in_business:        
    MOV  A, (1MS_INT|USB_RESET_INT)
    IOWR global_int    

bus_activity:
    MOV  A, 00h                      	; reset suspend counter
    MOV  [suspend_count], A

    IORD usb_status                 
    AND  A, ~BUS_ACTIVITY            	; clear bus activity bit
    IOWR usb_status                 

ms_timer_done:
    POP  X
    POP  A
    RETI

;----------------------------------------------------------
;
;   Interrupt handler: reset
;   Purpose: The program jumps to this routine when
;        the microcontroller has A power on reset.
;
;----------------------------------------------------------

reset:
    MOV  A, (LVR_ENABLE | INTERNAL_CLK)  ; set for use with internal oscillator  
    IOWR clock_config
    
    MOV  A, DSP_TOP                  ; set data stack pointer
    swap A, dsp                  
    MOV  A, PSP_BASE                 ; set PC stack pointer
    MOV  psp,A   
    
    IOWR watchdog
        
    CALL radio_interface_init       ; configure the GPIOs

    ; clear variables
    MOV  A, 00h                      

    MOV  [ep0_in_machine], A     
    MOV  [configuration], A              
    MOV  [ep1_stall], A                  
    MOV  [idle], A
    MOV  [suspend_count], A
    MOV  [ep1_dmabuff0], A
    MOV  [ep1_dmabuff1], A
    MOV  [ep1_dmabuff2], A
    MOV  [int_temp], A
    MOV  [idle_timer], A
    MOV  [idle_prescaler], A
    MOV  [event_machine], A
    MOV  [ep0_transtype], A

    MOV  [pc_has_desc], A
    MOV  [remote_wakeup], A          ; remote wake up is disabled by default

    MOV  [c0h], A
    MOV  [c1h], A
    MOV  [c2h], A
    MOV  [c3h], A
    MOV  [c4h], A
    MOV  [c5h], A
    MOV  [c6h], A
    MOV  [c7h], A
    MOV  [c8h], A
    MOV  [c9h], A
    MOV  [cah], A
    MOV  [cbh], A
    MOV  [cch], A
    MOV  [cdh], A
    MOV  [ceh], A
    MOV  [cfh], A

    MOV  [button_pushed], A				   
    MOV  [rssi_a], A				   
    MOV  [rssi_b], A   
IFDEF AGC_SUPPORT
    MOV  [corrupt_rssi], A   			  
    MOV  [agc_control], A
ENDIF ; AGC_SUPPORT
    			   
    ;initialize application variables
    MOV  [rx_counter], A
    MOV  [packet_ready], A
    MOV  [sequence_id_a], A
    MOV  [app_timer], A
IFDEF USE_DEVICE_ID_BIT
    MOV  [sequence_id_b], A
ENDIF ; USE_DEVICE_ID_BIT
    MOV  [timer], A
    
IFDEF BIND_AUTO
    MOV  [corrupt_counter], A
    MOV  [tx_ack_pending], A
ENDIF

IFDEF TWO_WAY_DATA
    MOV  [tx_app_counter], A
ENDIF

IFDEF ENCRYPT_DATA_A
    MOV  [encrypt_signature], A
ENDIF

    MOV  [led_state], A
    MOV  [ep1_dmabuff + 0], A
    MOV  [ep1_dmabuff + 1], A
    MOV  [ep1_dmabuff + 2], A
    MOV  [ep1_dmabuff + 3], A
    MOV  [ep1_dmabuff + 4], A
    MOV  [ep1_dmabuff + 5], A
    MOV  [ep1_dmabuff + 6], A
    MOV  [ep1_dmabuff + 7], A
    MOV  [ep2_dmabuff + 0], A
    MOV  [ep2_dmabuff + 1], A
    MOV  [ep2_dmabuff + 2], A
    MOV  [ep2_dmabuff + 3], A
    MOV  [ep2_dmabuff + 4], A
    MOV  [ep2_dmabuff + 5], A
    MOV  [ep2_dmabuff + 6], A
    MOV  [ep2_dmabuff + 7], A
    
    MOV  A, 01h
    MOV  [protocol], A

    MOV  A, WAKEUP_ADJUST2|WAKEUP_ADJUST1|WAKEUP_ADJUST0|PRECISION_CLK_ENABLE|INTERNAL_CLK ;128*tWAKE
    IOWR clock_config                 ; set wakeup timer interval AND  disable XTALOUT

    ; enable global interrupts
    MOV  A, (1MS_INT | USB_RESET_INT)
    IOWR global_int

    ; enable endpoint  0 interrupt
    MOV  A, EP0_INT          
    IOWR endpoint_int

    ; enable USB address for endpoint 0
    MOV  A, ADDRESS_ENABLE
    IOWR usb_address

	MOV A,ffh
    MOV  [led_timer], A
    
IFDEF TWO_WAY_DATA
    MOV  A, 50h                     ; THIS IS DEBUG CODE FOR TWO_WAY_DATA
    MOV  [tx_app_buffer], A
ENDIF   ; TWO_WAY_DATA

    EI                              ; enable all interrupts

    MOV  A, VREG_ENABLE             ; enable USB pullup resistor
    IOWR    usb_status              ; to signal connection to host

enum_wait:        
    IOWR watchdog
    MOV  A, [configuration]          ; wait until configured
    CMP  A, 01h                      
    JNZ  enum_wait

IFDEF ENDPOINT_1                      
    ;send A null USB report on EP1
    MOV  A, NUM_EP1_BYTES            ; set endpoint 1 to send defined number of bytes
    OR   A, [ep1_data_toggle]         
    IOWR ep1_count                  
    MOV  A, ACK_IN                   ; set to ack on endpoint 1
    IOWR ep1_mode                    ; AND  flag that data is waiting

IFDEF STARTUP_WAIT_ON_EP                      
.desc_wait1:        
    IOWR watchdog                   ; wait until PC takes that first null report
    IORD ep1_mode                   ; which indicates that PCHasDesc
    AND  A, 0Fh
    CMP  A, ACK_IN
    JZ   .desc_wait1
    CMP  A, NAK_IN
    JZ   .desc_wait_end1
    MOV  A, ACK_IN
    IOWR ep1_mode
    JMP  .desc_wait1
.desc_wait_end1:
    MOV  A, 01h
    MOV  [pc_has_desc], A
ENDIF ; STARTUP_WAIT_ON_EP                     
ENDIF ; ENDPOINT_1
        

    JMP emc_tester_mode


;****************************************************************************
; encore_suspend
;
; Processing:
; The radio is put into Suspend state
;
;****************************************************************************
encore_suspend :

; Set all pins to resistive mode if possible to prevent current flow during suspend. 

; Set Port 1.0 to Hiz, others to resistive (data=1!) for low power
    MOV  A, P1_MODE_1_SUSPEND
    IOWR port1_mode1
    MOV  A, P1_MODE_0_SUSPEND
    IOWR port1_mode0
    MOV  A, P1_DATA_SUSPEND
    IOWR port1

; Set Port 0 to resistive mode
    MOV  A, P0_MODE_1_SUSPEND 
    IOWR port0_mode1
    MOV  A, P0_MODE_0_SUSPEND 
    IOWR port0_mode0
    MOV  A, P0_DATA_SUSPEND 
    IOWR port0
  
    RET 

;****************************************************************************
; encore_resume
;
; Processing:
; The radio is put into Suspend state
;
;****************************************************************************
encore_resume :

    ;Put the GPIOs back to their normal state 
    MOV  A, [port0Shadow]
    IOWR port0
    MOV  A, P0_MODE_0_DEFAULT
    IOWR port0_mode0
    MOV  A, P0_MODE_1_DEFAULT
    IOWR port0_mode1
        
    MOV  A, [port1Shadow]
    IOWR port1
    MOV  A, P1_MODE_0_DEFAULT
    IOWR port1_mode0
    MOV  A, P1_MODE_1_DEFAULT
    IOWR port1_mode1

    RET 
    
;****************************************************************************
; signal_usb_resume
;
; Processing:
; The USB bus is forced into resume state
;
;****************************************************************************
signal_usb_resume:

    MOV  A, VREG_ENABLE | CONTROL1   ; force J	 (D+ Low, D- High)
    IOWR usb_status
    MOV  A, VREG_ENABLE | CONTROL0   ; force K	 (D+ High, D- Low)
    IOWR usb_status
            
    MOV  A, 10h                      ; wait (USB Spec is 1ms - 15ms)
	MOV  X, A

usb_res_timer_1:
    IOWR watchdog
    MOV  A, FFh                     
usb_res_timer_0:                       
    DEC  A                            
    JNZ  usb_res_timer_0             
    DEC  X                            
    JNZ  usb_res_timer_1             

    MOV  A, VREG_ENABLE | CONTROL1  ; force J - to prevent SE1 on the bus
    IOWR usb_status

    MOV  A, VREG_ENABLE             ; disable forcing 
    IOWR usb_status       

    MOV	 A, 0h						; Flag that we have signalled resume on USB
	MOV  [remote_wakeup], A			; Keeps us from re-signalling resume (for ill-behaved USB boxes)

    MOV  [sequence_id_a], A			; Ensures that the wakeup packet actually wakes-up the host
    MOV  [sequence_id_b], A
									
    RET 

;****************************************************************************
; rwu_poll_radio
;
; Processing:
; The radio is polled during suspend to allow remote wakeup from HID device
;
; Output:
; A == 1 ==> radio was resuspended (no remote wakeup)
; A == 0 ==> radio was not resuspended (remote wakeup activated)
;
;****************************************************************************
rwu_timer_0:          equ C8h
rwu_timer_1:          equ C9h
rwu_resuspend:        equ CAh
            
rwu_poll_radio:
    CALL encore_resume
    CALL radio_resume

    MOV  A,00h
    MOV  [packet_ready],A          ; packet_ready = 0;
    MOV  [rx_counter],A            ; rx_counter = 0;
         
    MOV  A,01h
    MOV  [rwu_resuspend],A         ; rwu_resuspend = 1;
                  
    MOV  A,06h                     ; ~ ms ON time of radio
    MOV  [rwu_timer_1],A     	   ; Period is ~ 270 ms
								   ; 0A = 10 ms ON, 260 ms OFF
                                   
.rwu_timer_1_loop:
    IOWR watchdog                  ; WATCHDOG = AC;
    MOV  A,ffh
    MOV  [rwu_timer_0],A           ; rwu_timer_0 = 0xFF;
                                   ; do
.rwu_timer_0_loop:
    IORD port0                     ; AC = PORT0;
    AND  A,IRQ                     ; if(!(AC & IRQ)) 
    JZ   .dec_rwu_timer_0 
    
.rwu_poll_radio:
    CALL process_rx_int
    MOV  A,[rx_counter]            ; if(rx_counter) 
    CMP  A,00h       
    JZ   .dec_rwu_timer_0                              
    MOV  A,00h
    MOV  [rwu_resuspend],A         ; rwu_resuspend = 0;

.dec_rwu_timer_0:
    DEC  [rwu_timer_0]             ; rwu_timer_0--;
    MOV  A,[rwu_timer_0]           ; while(rwu_timer_0 && rwu_resuspend);
    CMP  A,00h       
    JZ   .dec_rwu_timer_1       
    MOV  A,[rwu_resuspend]     
    CMP  A,00h       
    JNZ  .rwu_timer_0_loop       
    
.dec_rwu_timer_1:
    DEC  [rwu_timer_1]             ; rwu_timer_1--;
    MOV  A,[rwu_timer_1]           ; while(rwu_timer_1 && rwu_resuspend);
    CMP  A,00h       
    JZ   .if_rwe_resuspend       
    MOV  A,[rwu_resuspend]     
    CMP  A,00h       
    JNZ  .rwu_timer_1_loop       
                  
.if_rwe_resuspend:
    MOV  A,[rwu_resuspend]         ; if(rwu_resuspend) 
    CMP  A,00h       
    JZ   .radio_poll_return_val                            

.radio_poll_resuspend:
    CALL radio_suspend
    CALL encore_suspend

.radio_poll_return_val:
    MOV  A,[rwu_resuspend]         ; AC = rwu_resuspend;

    RET
    
;-------------- end program listing -----------------------------------------

    INCLUDE "utilities.asm"
    INCLUDE "radio.asm"    

    INCLUDE "emc_tester.asm"

 ; Place USB code in upper 4K block
ORG 1000h                          
    INCLUDE "usbcode.asm"       

ORG 1800h
    INCLUDE "dvk_hardware.asm"
