//--------------------------------------------------------------------------
//! \file
//!
//! Collection of the bind functions for basic, sem-auto, and automatic bind.
//
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/Radio/bind.c $
// $Modtime: 3/19/04 1:34p $
// $Revision: 1.1 $
//--------------------------------------------------------------------------
//
// Copyright 2003-2004, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

//--------------------------------------
// Included files
//--------------------------------------
#include "bind.h"
#include "protocol.h"
#include "radio.h"
#include "flash.h"

//--------------------------------------
// Local Definitions and Types
//--------------------------------------
//! Receive packet
extern LS_RX_PACKET  rx_packet;
//! Transmit packet
extern LS_TX_PACKET  tx_packet;
//! Channel valid array for data packets
extern UINT8         channel_valid[];
//! Length of receive data
extern UINT8         rx_data_length;
//! To tell whether the device is in a connected state or not
UINT8  bind_mode = 0;

const UINT8 pn_code_table[] = 
{
    0x6A, 0xE7, 0x01, 0xEA, 0x03, 0xFD, 0x13, 0xD2, 
    0xDC, 0xC0, 0x6B, 0xB8, 0x2B, 0x09, 0xBB, 0xB2, 
    0xA3, 0x1E, 0xF2, 0xA4, 0x31, 0x32, 0x7A, 0xB3, 
    0x44, 0x83, 0x3B, 0xDD, 0x14, 0xCF, 0x8E, 0xC9, 
    0x35, 0x35, 0x4E, 0xC5, 0xF3, 0x52, 0x47, 0xB0, 
    0x7C, 0x23, 0x8A, 0xCE, 0x45, 0x5C, 0x54, 0xD7, 
    0x81, 0xAC, 0xFB, 0x83, 0x7A, 0x9A, 0x61, 0xAC, 
    0x3C, 0x12, 0x5F, 0x9C, 0x39, 0x98, 0xF6, 0x8A, 
};


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    get_next_channel
//
//!  Description: 
//!               - Performs the channel selection algorithm. Every sixth channel
//!               will be used.
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////

void get_next_channel(void)
{
    sys_params.network_id.channel += (6 * (sys_params.network_id.pin + 1));

    if((NUM_CHANNELS) - 1 < sys_params.network_id.channel)
    {
        sys_params.network_id.channel -= NUM_CHANNELS;    
    }
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    init_channel
//
//!  Description: 
//!               - Initialize the channel selection algorithm to the top channel
//!               in the channel subset
//!
//!  Inputs:      None
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////

void init_channel(void)
{
    while (sys_params.network_id.channel < NUM_CHANNELS - 6) 
    {
        sys_params.network_id.channel += 6;
    }
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_bind_init
//
//!  Description: 
//!               - Perform the basic bind channel and pn_code setup
//!
//!  Inputs:      None
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////

void protocol_bind_init()    // Bind Auto
{
    init_channel();

    bind_mode = DISCONNECTED_STATE;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_bind
//
//!  Description: 
//!               - Formats the bind packet, reads radio information, sets up
//!               the radio and reads/sets the power level.  Then as it runs
//!               through the channels it sends bind requests until an 
//!               acknowledge is received (or the maximum amount of requests 
//!               have been sent).  If all channels have been run through 
//!               without an ack, then a bind timeout occurs which would result
//!               to an unsuccessful bind and all bind parameters are 
//!               reinitialized.  If a good ack packet is received, it then 
//!               creates and sends an ack packet back 3 times to ensure
//!               delivery.  Then the settings are saved in NVRAM and the power 
//!               settings are restored.
//!                                 
//!
//!  Inputs:      None
//!
//!  Returns:     
//!               - status - LS_STATUS_SUCCESS for a successful bind and
//!               LS_STATUS_FAILED for an unsuccessful bind    
//!
///////////////////////////////////////////////////////////////////////////////

UINT8 protocol_bind()    // Bind Auto
{
    UINT8  channel_timeout;
    UINT8  status = LS_STATUS_SUCCESS;
    UINT8  packet_received = NO_PACKET;
    UINT8  tx_timeout;
    UINT8  pa_setting;
    UINT8  led_flash;
                 
    bind_mode = DISCONNECTED_STATE;
    
    // format the bind packet
    tx_packet.bind.hdr.type    = BIND_REQ; 
    tx_packet.bind.hdr.dev_type = DEVICE_TYPE; 
    tx_packet.bind.hdr.parity  = protocol_gen_parity(tx_packet.first.byte);
    
    // load the bind pn_code index
    radio_set_pn_code(&pn_code_table[(8 * BIND_PN_CODE)]);
     
    // Read PA setting
    pa_setting = radio_get_power_level();    
    
    // load Bind PA
    radio_set_power_level(BIND_POWER_LEVEL);

    // seed the channel algorithm with bind parameters 0,0
    sys_params.network_id.channel = BIND_BASE_CHANNEL;
       
    // send a bind request and wait for response, if no response change
    // the channel and try again
    for( channel_timeout = 0; channel_timeout < BIND_TIMEOUT; ++channel_timeout)
    {   
        // setup mode
        radio_set_channel(sys_params.network_id.channel);
        
        // loop until ACK radio_received or TX retries exhausted
        for(tx_timeout = 0; tx_timeout <= BIND_MAX_TRANSMITS; ++tx_timeout)
        {                           
	        radio_transmit(BIND_REQ_LEN, (UINT8 *)&tx_packet);

            // wait for Bind Response 
            if (radio_receive(LS_RX_PACKET_LEN, (UINT8 *)&rx_packet, (UINT8 *)&channel_valid, BIND_ACK_TIMEOUT) > 0)
            {
                packet_received = protocol_verify_packet();
            }

    	    if (packet_received == BIND_RESP) 
            {
                break;                   
            }
            else
    	    {
    	        //	radio_off();
                timer_delay_msec(8);
    
            }
        }
       
        // check for Bind timeout
        if(tx_timeout >= BIND_MAX_TRANSMITS)
        {
	        // Clear Watchdog and Sleep Timer
	        M8C_ClearWDTAndSleep;

            //try the next channel in the subset of channels
            get_next_channel();
        }
        else if( (packet_received == BIND_RESP) && (rx_data_length == BIND_RESP_LEN) ) 
        {  
            // Good Ack
            
             // send an ACK
            tx_packet.first.byte = 0;
            tx_packet.ack.hdr.type   = ACK_PACKET;                    
            tx_packet.ack.hdr.flag   = GOOD_ACK;
            tx_packet.ack.hdr.parity = protocol_gen_parity(
                                      (UINT8) tx_packet.first.byte);
                  
            // send the ACK to the bind response multiple times to
            // ensure delivery
            radio_transmit(ACK_LEN, (UINT8 *)&tx_packet);
            
            TIMER_DELAY_50_USEC();
            TIMER_DELAY_50_USEC();
            TIMER_DELAY_50_USEC();
            TIMER_DELAY_50_USEC();          
            radio_transmit(ACK_LEN, (UINT8 *)&tx_packet);
            
            TIMER_DELAY_50_USEC();
            TIMER_DELAY_50_USEC();
            TIMER_DELAY_50_USEC();
            TIMER_DELAY_50_USEC();
            radio_transmit(ACK_LEN, (UINT8 *)&tx_packet);
                                    
            // extract Bridge MID from BIND_RESP 
            sys_params.bridge_mid.mid_1        = rx_packet.bind.mid_1;
            sys_params.bridge_mid.mid_2        = rx_packet.bind.mid_2;
            sys_params.bridge_mid.mid_3        = rx_packet.bind.mid_3;
            sys_params.bridge_mid.mid_4        = rx_packet.bind.mid_4;

            status = LS_STATUS_SUCCESS;
            break; // exit the for loop
        }
        
        set_bind_led (led_flash);
        
        if (led_flash)
        	led_flash = 0;
        else
        	led_flash = 1;
        	
    }  // for()
    
    // Restore PA setting
    radio_set_power_level(pa_setting);
    
    // check the results of the while loop
    if(channel_timeout > BIND_TIMEOUT - 1)
    {
        // can't Bind, reset to original bind parameters
        protocol_bind_init();
        
        status = LS_STATUS_FAILED;
    }
    else if(packet_received == BIND_RESP)   // ack sent OK
    {
        // store parameters in NVRAM
        NVRAM_WRITE( &sys_params );
        
        // calculate channel, PN code, etc. from the Bridge MID
        calculate_network_id();
     
        // give the dongle time to ping before trying to reconnect
        TIMER_DELAY_MSEC(BRIDGE_PING_TIME);
        
        init_channel();
        
        status = protocol_reconnect();
    }
        
    return status;
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_reconnect
//
//!  Description: 
//!               - Checks for an unbound device, resets radio (and restore
//!               transmit valid bits if necessary), formats a connect data 
//!               packet, reads radio information and sets up the radio.  It 
//!               then reads and adjusts power level (if DYNAMIC_PA enabled).
//!               Then as it runs through the channels it sends connect requests
//!               until an good response is received (or the maximum amount of
//!               requests have been sent). If all channels have been run through 
//!               without a response, then it tries the original channel a few
//!               times.  If no response is received on any channel the 
//!               connection is unsuccessful.
//!               
//!
//!  Inputs:      None
//!
//!  Returns:     
//!               - status - LS_STATUS_SUCCESS for a successful connection and
//!               LS_STATUS_FAILED for an unsuccessful connection
//!
///////////////////////////////////////////////////////////////////////////////

UINT8 protocol_reconnect()
{
    UINT8  timeout;
    UINT8  status;
    UINT8  packet_received;
        
    // check for unbound device
    if (sys_params.network_id.pin == 0) 
    {
        return LS_STATUS_FAILED;
    }
    
    bind_mode = DISCONNECTED_STATE;
    status = LS_STATUS_FAILED;

    // check for radio resetting due to a glitch on the N_RESET
    // pin. The TX Valids bit should be 0xFF. If not that value the 
    // radio is corrupted.
    if( radio_get_power_level() == 0x00)
    {
        // reset the radio
        radio_init();       
    }
    
    // select a channel, send a connect request, and
    // wait for a connect response
    // format the bind packet
    tx_packet.connect.hdr.type   = CONNECT_REQ; 
    tx_packet.connect.hdr.pin    = sys_params.network_id.pin;
    tx_packet.connect.hdr.parity = 
        protocol_gen_parity((UINT8) tx_packet.first.byte);

    // load Bridge MID from sys_params into the tx_packet...
    NVRAM_READ( &tx_packet.connect.mid_1, 4 );
       
    // calculate checksum
    tx_packet.connect.checksum =
        protocol_calc_ck_sum( (UINT8 *)&tx_packet.connect, 
                              CONNECT_REQ_LEN-1, sys_params.network_id.seed );
    
    // load the bind pnCode index
    //radio_set_pn_code(sys_params.network_id.pn_code);
    radio_set_pn_code(&pn_code_table[(8 * sys_params.network_id.pn_code)]);
   
    // send a bind request and wait for response, if no response change
    // the channel and try again
    for(timeout=0; timeout<CONNECT_TIMEOUT; timeout++)
    {                        
        // setup radio_transmit mode
        radio_set_channel(sys_params.network_id.channel);
    	radio_transmit(CONNECT_REQ_LEN, (UINT8 *)&tx_packet);

        // wait for Connect Response 
        if (radio_receive(LS_RX_PACKET_LEN, (UINT8 *)&rx_packet, 
                          (UINT8 *)&channel_valid, CONNECT_ACK_TIMEOUT) > 0)
        {
            packet_received = protocol_verify_packet();
        }
       
        if(packet_received == CONNECT_RESP)
        {
	        if ((rx_packet.connect.hdr.flag == GOOD_ACK ) &&
                (rx_data_length == CONNECT_RESP_LEN))
	        {
                bind_mode = CONNECTED_STATE;
	          
                // return a success status...
                status = LS_STATUS_SUCCESS;
	            
                break; // good response, exit the loop
	        } 
	        else
	        {
                // bad response, bump channel
                get_next_channel();	        
            }                    
        }
        else
        {
            // Try a few times on the original channel
            if (timeout > ORIGINAL_CHAN_RECONNECT_COUNT)
            {
                // timeout, bump channel
                get_next_channel();
                
            }
        }
    }  // end for(timeout = 0; timeout < CONNECT_TIMEOUT; timeout++)
    
    return status;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    set_bind_led
//
//  Description: Turn the Bind LED on or off
//
//  Inputs:      desired state, ON or OFF.
//
//  Returns:     void
//
///////////////////////////////////////////////////////////////////////////////

void set_bind_led(UINT8 state)
{
    UINT8  led_port_shadow;

    led_port_shadow = COL_PORT_DR;        
    if( state == ON )
    {
        // turn the LED on
        COL_PORT_DR = led_port_shadow & ~BIND_LED;
    }
    else  // turn it off
    {
        // turn off the LED
        COL_PORT_DR = led_port_shadow | BIND_LED;
    }
} 