//--------------------------------------------------------------------------
//
//  Main application control routine
//
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/DVK General/main.c $
// $Modtime: 3/16/04 9:50a $
// $Revision: 1.1 $
//--------------------------------------------------------------------------
//
// Copyright 2003, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

#include "cypdef.h"
#include "ls_config.h"
#include "radio.h"
#include "bind.h"
#include "protocol.h"

void platform_init(void);
void sw_ports_init(void);
UINT8 bind_pressed(void);
UINT8 connect_pressed(void);
void set_data_led(UINT8);

void main()
{
    UINT8 led_state = 0;
    UINT16 led_timer = 0;
    UINT8 *pTxPkt;
    
    // configure the board
    platform_init();
    
    // initialize the protocol
    protocol_init();
        
    // get the packet pointers
    pTxPkt = protocol_get_tx_pkt();

    // main infinite control loop    
    while(1) 
    {                   
		if (led_timer)
		{
			led_timer--;
			
			if (!led_state)
			{
				led_state = 1;
				set_data_led(led_state);
			}
		}
		else
		{
			led_state = 0;
			set_data_led(led_state);
		}
		
        // send packet if needed   
        if(connect_pressed())
        {
            // radio_transmit the application report
            pTxPkt[0] = 0x00;
    		pTxPkt[1] = 0x55;
    		pTxPkt[2] = 0xAA;
    		pTxPkt[3] = 0xFF;

            if(protocol_send_packet(APP_TX_PACKET_SIZE) == DATA_SENT)
            {
                set_bind_led(ON);
                led_timer = 16384;
            }
        }
	     
        if(bind_pressed())
        {
            // perform the bind routine
            if (!protocol_bind()) 
            {
                set_bind_led(ON);
            }
        }            
    }  // while(1)

}  // end main

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    platform_init
//
//  Description: Initialize the PSOC ports for the PDC-9075 board
//
//  Inputs:      None
//
//  Returns:     Void
//
///////////////////////////////////////////////////////////////////////////////

void platform_init()
{
    // initialize port 4 and 5 for reading switches
    sw_ports_init();
    
    // clear red and green LEDS
    LED_PORT_DR = LEDS_OFF;
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    sw_ports_init
//
//  Description: Initialize ports 4 and 5 for reading switches on
//               the keypad, joystick, and the selector switch.
//
//  Inputs:      None
//
//  Returns:     Void
//
///////////////////////////////////////////////////////////////////////////////

void sw_ports_init()
{

	// enable the row pullups
	PRT5DR |= (ROW_0 | ROW_1 | ROW_2 | ROW_3);
	// drive all the columns high
	COL_PORT_DR = (COL_0 | COL_1 | COL_2 | COL_3 | COL_4 | COL_5 | COL_6 | COL_7);
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    bind_pressed
//
//  Description: Check to see if the Bind button is pressed
//
//  Inputs:      void
//
//  Returns:     1 = Bind pressed
//               0 = Bind not pressed
//
///////////////////////////////////////////////////////////////////////////////

UINT8 bind_pressed(void)
{
    UINT8  led_port_shadow;

    led_port_shadow = LED_PORT_DR;        
    if(led_port_shadow &= BUTTON_BIND)
    {
        // clear the button
        led_port_shadow &= ~BUTTON_BIND; 
        
        // wait for the button to be released
        while(LED_PORT_DR &= BUTTON_BIND)
        {
            // clear the button
            led_port_shadow &= ~BUTTON_BIND; 
            LED_PORT_DR = led_port_shadow | CONNECT_LED | DATA_LED;
           
            // wait to check for the button release
            timer_delay_msec( 20 );
        }

        // clear the button
        led_port_shadow &= ~BUTTON_BIND;                  
        LED_PORT_DR = led_port_shadow | CONNECT_LED | DATA_LED;
        return 1;
    }
    return 0;
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    connect_pressed
//
//  Description: Check to see if the Connect button is pressed
//
//  Inputs:      void
//
//  Returns:     1 = Connect pressed
//               0 = Connect not pressed
//
///////////////////////////////////////////////////////////////////////////////

UINT8 connect_pressed(void)
{
    UINT8  led_port_shadow;

    led_port_shadow = LED_PORT_DR;        
    if(led_port_shadow &= BUTTON_CONNECT)
    {
        // clear the button
        led_port_shadow &= ~BUTTON_CONNECT; 
        
        // wait for the button to be released
        while(LED_PORT_DR &= BUTTON_CONNECT)
        {
            // clear the button
            led_port_shadow &= ~BUTTON_CONNECT; 
        	LED_PORT_DR = led_port_shadow | CONNECT_LED | DATA_LED;
           
            // wait to check for the button release
            timer_delay_msec( 20 );
        }

        // clear the button
        led_port_shadow &= ~BUTTON_CONNECT;                  
        LED_PORT_DR = led_port_shadow | CONNECT_LED | DATA_LED;
        return 1;
    }
    return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    set_data_led
//
//  Description: Turn the DATA LED on or off
//
//  Inputs:      desired state, ON or OFF.
//
//  Returns:     void
//
///////////////////////////////////////////////////////////////////////////////

void set_data_led(UINT8 state)
{
    UINT8  led_port_shadow;

    led_port_shadow = LED_PORT_DR;        
    if( state == ON )
    {
        // clear the button
        led_port_shadow &= ~BUTTON_BIND; 

        // turn the LED on
        LED_PORT_DR = led_port_shadow & ~DATA_LED;
    }
    else  // turn it off
    {
        // clear the button
        led_port_shadow &= ~BUTTON_BIND; 

        // turn off the LED
        LED_PORT_DR = led_port_shadow | DATA_LED;
    }
} 

