//--------------------------------------------------------------------------
//! \file
//!
//! Collection of functions which support the WSUB LS Protocol for 
//! HID and HID like devices
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/Radio/protocol.c $
// $Modtime: 3/04/04 2:46p $
// $Revision: 1.1 $
//--------------------------------------------------------------------------
//
// Copyright 2003-2004, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

//--------------------------------------
// Included files
//--------------------------------------
#include "protocol.h"
#include "bind.h"
#include "spi_clk.h"
#include "radio.h"
#include "flash.h"


//--------------------------------------
// Local Definitions and Types
//--------------------------------------
//! To know what state the bind is in
extern UINT8 bind_mode;

//! To set up the device ID when generating a transmit packet
UINT8 protocol_device_type = DEVICE_TYPE;
//! Transmit data toggle
UINT8  tx_data_toggle      = 1;  
//! Receive data toggle
UINT8  rx_data_toggle      = 1; 
//! Receive packet
LS_RX_PACKET rx_packet;
//! Transmit packet
LS_TX_PACKET tx_packet;
//! Channel valid array for data packets
UINT8 channel_valid[LS_RX_PACKET_LEN];

//! Data length for receive data
extern UINT8 rx_data_length;

//! For accessing and formatting data packets
SYS_PARAMETERS sys_params;

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_get_tx_pkt
//
//!  Description: 
//!               - Returns a pointer to a radio_transmit buffer pool 
//!               controlled by the protocol, currently in the PSoC only 
//!               1 buffer is used due to memory limitations
//!
//!  Inputs:      Void
//!
//!  Returns:     Void *
//!
///////////////////////////////////////////////////////////////////////////////
void * protocol_get_tx_pkt(void)
{
    return &tx_packet.data.app_packet;
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_get_rx_pkt
//
//!  Description: 
//!               - Returns a pointer to a protocol_receive buffer pool 
//!               controlled by the protocol, currently in the PSoC only 
//!               1 buffer is used due to memory limitations
//!
//!  Inputs:      Void
//!
//!  Returns:     Void *
//!
///////////////////////////////////////////////////////////////////////////////
void * protocol_get_rx_pkt(void)
{
    return &rx_packet.data.app_packet;
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_init
//
//!  Description: 
//!               - Sets up first byte of transmit data packet, and reads nvram.
//!               Gets or initializes, if necessary, device parameters.  Initializes 
//!               the LS Radio and bind protocol
//!
//!  Inputs:      None
//!
//!  Returns:     
//!               - LS_STATUS_SUCCESS - Successful initialization
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 protocol_init()
{
    tx_packet.first.byte = 0x50;    // Data packet
    protocol_device_type = DEVICE_TYPE;

    // get nvram
    NVRAM_READ( &sys_params, sizeof(sys_params) );

    // check for uninitialized parameters
    if (sys_params.network_id.signature != SIGNATURE_BYTE)
    {
        // set channel and pn_code to a value that will allow the different
        // devices to coexist and interfere less with each other
        sys_params.network_id.signature   = SIGNATURE_BYTE;
        sys_params.network_id.channel     = DEFAULT_CHANNEL;
        sys_params.network_id.pn_code     = DEFAULT_PN_CODE;
        sys_params.network_id.seed        = DEFAULT_CHECKSUM_SEED;
        sys_params.network_id.pin         = 0;
    }
    else
    {
        // calculate channel, PN code, etc. from Bridge MID and store 
        // values in sys_params
        calculate_network_id();
    }
    
    SPI_Clk_Start();
       
    // configure the Radio
    radio_init();

    // set up the channel and PN Code
    protocol_bind_init();

    return LS_STATUS_SUCCESS;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    calculate_network_id
//
//!  Description: 
//!               - Figures device parameters using algorithms to find PN code,
//!               channel, pin, and checksum seed (based off the MID).               
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void calculate_network_id(void)
{
    UINT8 channel;
    UINT8 pn_code; 
    UINT8 seed; 
    UINT8 pin;

    pn_code = (sys_params.bridge_mid.mid_1 << 2) + sys_params.bridge_mid.mid_2 + 
              sys_params.bridge_mid.mid_3;
    while ( pn_code >= NUM_PN_CODES )
    {
        pn_code -= NUM_PN_CODES;
    }

    channel = (sys_params.bridge_mid.mid_2 >> 2) -
              (sys_params.bridge_mid.mid_1 << 5) + sys_params.bridge_mid.mid_3;
    while ( channel >= NUM_CHANNELS )
    {
        channel -= NUM_CHANNELS;
    }

    pin = (((sys_params.bridge_mid.mid_1 - sys_params.bridge_mid.mid_2) & 
            PIN_MASK) + MIN_PIN);

    seed = ((sys_params.bridge_mid.mid_2 >> 6)) + sys_params.bridge_mid.mid_1 + 
           sys_params.bridge_mid.mid_3; 

#ifdef TUTORIAL_MODE        
    channel = 0x07;
    pn_code = 0x00;
#endif

    sys_params.network_id.channel         = channel;
    sys_params.network_id.pn_code         = pn_code;
    sys_params.network_id.seed            = seed;
    sys_params.network_id.pin             = pin;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_send_packet
//
//!  Description: 
//!               - Makes sure that that the device is connected. If there is
//!               no data to send, it will prepare a null packet to keep things
//!               alive.  If there is data, a packet is prepared with actual
//!               data.  Once the packet is prepared it is transmitted.  Once
//!               an acknowledge (DATA/ACK or ACK) is received from the master
//!               device, then it sends an ACK back(stripping the checksum and 
//!               header from packet first).
//!                 
//!
//!  Inputs:      
//!               - data_length - Length of packet, 0 = send NULL packet
//!
//!  Returns:     
//!               - TX_TIMEOUT_ERR - If not bound or when a timeout occurs on 
//!                                the transmit
//!               - INVALID_DATA   - When data_length is >= APP_TX_PACKET_SIZE 
//!               - ACK_DATA       - If the data toggle parameter is set in the 
//!                                receive packet
//!               - DATA_SENT      - Data transmitted successfully
///////////////////////////////////////////////////////////////////////////////
UINT8 protocol_send_packet( UINT8 data_length )   // Two Way
{
    UINT8  tx_timeout;   
    UINT8  rx_packet_type;

    if (bind_mode != CONNECTED_STATE)
    {
        protocol_reconnect();
        return TX_TIMEOUT_ERR;
    }

    tx_packet.first.byte = 0;           // clear the header byte

    // send a Null Packet to poll for back channel data or as a keep alive packet...
    if (data_length == 0)
    {
        tx_packet.null.hdr.type   = NULL_PACKET;
        tx_packet.null.hdr.fill = 0;

        tx_packet.null.hdr.dev_id  = (protocol_device_type & 0x01);

        tx_packet.null.hdr.parity = protocol_gen_parity(tx_packet.first.byte);
        data_length = NULL_LEN;

        tx_packet.data.app_packet[0] = 
        protocol_calc_ck_sum((UINT8 *)&tx_packet, 1, 
                             sys_params.network_id.seed );
    }
    // setup for a Data Packet and bump the data toggle
    else if (data_length < APP_TX_PACKET_SIZE + 1)
    {
        tx_packet.data.hdr.type    = DATA_PACKET;
        tx_packet.data.hdr.dev_id  = (protocol_device_type & 0x01);
        tx_packet.data.hdr.dt_data = tx_data_toggle;
        tx_packet.data.hdr.parity  = protocol_gen_parity(tx_packet.first.byte);

        // calculate check sum
        tx_packet.data.app_packet[data_length] = 
            protocol_calc_ck_sum((UINT8 *)&tx_packet, data_length + 1, 
                                 sys_params.network_id.seed);

        data_length += 2;   // length = header + data + checksum
    }
    else
    {
        return( INVALID_DATA );   
    }

    // setup radio_transmit mode, give more time to settle
    //radio_set_channel(sys_params.network_id.channel);

    // initialize loop control
    //rx_packet_type = NO_PACKET;

    //loop until ACK radio_received or retries exhausted
    for (tx_timeout = 0; tx_timeout < TX_TIMEOUT; tx_timeout++)
    {
        // send the packet
        radio_transmit(data_length, (UINT8 *)&tx_packet);

        // get an ACK 
        //if(radio_receive(ACK_TIMEOUT) > 0)
        if (radio_receive(LS_RX_PACKET_LEN, (UINT8 *)&rx_packet, (UINT8 *)&channel_valid, ACK_TIMEOUT) > 0)
        {
            rx_packet_type = protocol_verify_packet();
        }

        if (rx_packet_type == ACK_PACKET)
        {
            if ( (rx_packet.ack.hdr.flag == GOOD_ACK) && (rx_data_length == ACK_LEN) && 
                 ((rx_packet.ack.hdr.dev_id & 0x01) == 
                  (protocol_device_type & 0x01)) &&
                 (((tx_packet.null.hdr.type != NULL_PACKET) &&
                   (rx_packet.ack.hdr.dt_ack == (tx_data_toggle & 0x01))) || 
                  (tx_packet.null.hdr.type == NULL_PACKET)) )
            {
                if (tx_packet.null.hdr.type != NULL_PACKET)
                {
                    // flip the data toggle bit to signal an 
                    // acknowledged packet transmission
                    tx_data_toggle++;    
                }

                return( DATA_SENT );
            }
        }
    } // for()

    protocol_reconnect();
    //radio_idle();

    return( TX_TIMEOUT_ERR );   
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_calc_ck_sum
//
//!  Description: 
//!               - Calculate Packet Checksum
//!
//!  Inputs:      
//!               - *data - Packet pointer
//!               - len - Length of packet
//!               - seed - Checksum seed
//!
//!  Returns:     
//!               - checksum - Checksum calculated from data
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 protocol_calc_ck_sum(UINT8 *data, UINT8 len, UINT8 seed)
{
    UINT8 checksum;

    for (checksum = seed; len > 0; len--, data++)
    {
        checksum ^= *data;
    }        
    return(checksum);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_gen_parity
//
//!  Description: 
//!               - Calculate odd parity for a byte of data
//!
//!  Inputs:      
//!               - data - data byte
//!
//!  Returns:     
//!               - 1 - even number of ones
//!               - 0 - odd number of ones
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 protocol_gen_parity(UINT8 data)
{
    UINT8 count = 0;

    for (; data; )
    {
        data>>=1; // shift out the parity bit...

        // check for bits that are set
        if (data & 0x01)
        {
            ++count;
        }
    }    

    return((count + 1) & 0x01);
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_send_ack
//
//!  Description: 
//!               - Formats the acknowledge packet and transmits it
//!
//!  Inputs:      Void
//!
//!  Returns:     Void
//!
///////////////////////////////////////////////////////////////////////////////
void protocol_send_ack(void)
{
    // create ACK packet
    tx_packet.first.byte = 0;
    tx_packet.ack.hdr.type = ACK_PACKET;
    tx_packet.ack.hdr.flag = GOOD_ACK; 

    tx_packet.ack.hdr.dev_id = (protocol_device_type & 0x01);

    // the RX data toggle field is being mirrored back in the ACK
    tx_packet.ack.hdr.dt_ack = rx_packet.data.hdr.dt_data;

    tx_packet.ack.hdr.parity = protocol_gen_parity(
                                                  (UINT8) tx_packet.first.byte );        

    // send the packet
    radio_transmit( ACK_LEN, (UINT8 *)&tx_packet );    
}    


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    protocol_verify_packet
//
//!  Description: 
//!               - Gets packet data and makes sure the bytes are protocol 
//!               perfect(otherwise it will return NO_PACKET).  It checks and
//!               fixes packets with checksums  by testing each column of bits 
//!               (See comments and comment block in code source).  It then 
//!               compares checksums with the calculated checksum.
//!
//!  Inputs:      Void
//!
//!  Returns:     
//!               - packet_type - 0x00-0x0F (dependent upon type of Packet 
//!               received) or 0xFF for no packet 
//!
///////////////////////////////////////////////////////////////////////////////
UINT8 protocol_verify_packet(void)
{
    UINT8  i;
    UINT8  packet_type;
    UINT8  checksum;
    UINT8  bad_bit; 

    // get the packet type, it is the upper nibble in the first byte
    packet_type = rx_packet.ack.hdr.type;

    // check for perfect protocol bytes, requirement for bit correction AND
    // blunder check the parity bit  AND 
    // blunder check the packet type AND
    // blunder check the data types by length
    if ( (channel_valid[0] != 0xFF) || 
         (channel_valid[rx_data_length-1] != 0xFF) ||
         ( protocol_gen_parity(rx_packet.first.byte) != 
           (rx_packet.first.byte & 0x01)) || 
         (packet_type >= MAX_NUM_TYPES + 1) ||
         ( (packet_type == DATA_ACK_PACKET) && (rx_data_length <= 2) ) || 
         (packet_type == ACK_PACKET && rx_data_length != ACK_LEN) )
    {
        // packet protocol bytes invalid OR
        // bad parity in packet OR
        // bad format in packet
        return( NO_PACKET );
    }

    // only check/fix packets with checksums
    if ( packet_type == BIND_RESP ||
         packet_type == DATA_ACK_PACKET)
    {

        // quick test for bad bits
        bad_bit = 0xFF;
        for (i = 1; i < rx_data_length; i++)
        {
            // AND the colums togethers to find
            // bad bits
            bad_bit &= channel_valid[i];
        }

        if (bad_bit != 0xFF)
        {
            return NO_PACKET;
        }

        // compare the checksum....
        if ( packet_type == BIND_RESP )
        {
            checksum = BIND_SEED;
        }
        else
        {
            checksum = sys_params.network_id.seed;
        }
        // Comparing the received checksum with the calculated checksum    
        if (*((UINT8 *) (&rx_packet.first.byte) + ((rx_data_length)-1)) != 
            protocol_calc_ck_sum((UINT8*)&rx_packet.first.byte, 
                                 (rx_data_length)-1, checksum ))
        {
            // bad packet
            packet_type = NO_PACKET;
        }

    } // if packet_type == packet with sumcheck

    return(packet_type);
}
