//--------------------------------------------------------------------------
//! \file
//!
//!Include file which defines the LS radio packet format
//!
//!  NOTE:If you are porting this code to another processor, the bit fields 
//!       may need to be swapped around, and calls to protocol_gen_parity may need to be 
//!       changed also (casting to find the beginning of the header byte).
//!
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/Radio/protocol.h $
// $Modtime: 3/04/04 3:11p $
// $Revision: 1.1 $
//--------------------------------------------------------------------------
//
// Copyright 2003-2004, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------

#ifndef _PROTOCOL_H_
#define _PROTOCOL_H_

//--------------------------------------
// Included files
//--------------------------------------
#include "ls_config.h"

//--------------------------------------
// API Definitions and Types
//--------------------------------------
// Two way acknowledge timeout
#ifndef ACK_TIMEOUT
	#ifdef DATA_RATE_16
	#define ACK_TIMEOUT  80
	#endif
	#ifdef DATA_RATE_32
	#define ACK_TIMEOUT  80
	#endif
	#ifdef DATA_RATE_64
    //! Timeout for the receiving of a packet ( see radio_receive())
	#define ACK_TIMEOUT  255
	#endif
#endif // ACK_TIMEOUT

//! Number of channels in the 2.4 GHz spread spectrum
#define NUM_CHANNELS 	        78
//! Mask used in the pin distribution algorithm
#define PIN_MASK       	        0x03  
//! To allow certain pins to be used in the pin distribution algorithm
#define MIN_PIN                 0x02
// Connect and data seed
#define DEFAULT_CHECKSUM_SEED   0x00

//! Used for a successful bind, ping, or initialization
#define LS_STATUS_SUCCESS   0x00
//! Used when a bind, ping, or initialization fails
#define LS_STATUS_FAILED    0x01

//  Status codes
//! To know that a valid data packet was transmitted
#define DATA_SENT      		41
//! To know that it was an invalid data packet
#define INVALID_DATA    	43  
//! Used when the device is no longer bound or a timeout occurred when sending
//! the packet
#define TX_TIMEOUT_ERR      0x05
//! To know that a valid ack/data packet was transmitted
#define ACK_DATA            0x07
//! To confirm a good ack packet
#define GOOD_ACK            0x01

// Packet types
//! Bind Request Packet Type
#define BIND_REQ            0x00
//! Bind Response Packet Type
#define BIND_RESP           0x00
//! Connect Request Packet Type
#define CONNECT_REQ         0x01
//! Connect Response Packet Type
#define CONNECT_RESP        0x02
//! Ping Packet Type
#define PING_PACKET         0x03
//! Data Packet Type
#define DATA_PACKET         0x04
//! Ack Packet Type
#define ACK_PACKET          0x05
//! Data/Ack Packet Type
#define DATA_ACK_PACKET     0x06
//! Null Packet Type
#define NULL_PACKET         0x07
//! Number of packets is 7 when encryption is enabled
#define MAX_NUM_TYPES   0x07   // needs to point to last used packet type

//! When there is no packet received
#define NO_PACKET           0xFF

//! Ping Request Packet
#define PING_REQ	        0x00
//! Ping Response Packet
#define PING_RESP	        0x01
//! To set the flag in a connect response packet
#define ACK_RESP            0x01

//! To setup the radio
typedef union _SYS_PARAMETERS
{
    //! To data packet parameters for the radio and bridge
    struct 
    {
        //! For storing the channel used in the data packet
    	UINT8	channel;
        //! For storing the PN code used in the data packet
    	UINT8	pn_code;
        //! For storing the checksum seed used in the data packet
    	UINT8   seed;
        //! For storing the pin used in the data packet
       	UINT8	pin;
        //! For storing the base channel used for connect request packets
       	UINT8	base_channel;
        //! For storing the signature when parameters are first initialized
        UINT8   signature;
    } network_id;

    //! Radio manufacturing ID for bridge
    struct 
    {
        //! 1st byte of radio manufacturing ID
        UINT8   mid_1;
        //! 2nd byte of radio manufacturing ID
        UINT8   mid_2;
        //! 3rd byte of radio manufacturing ID
        UINT8   mid_3;
        //! 4th byte of radio manufacturing ID
        UINT8   mid_4;
        //UINT8   signature;
    } bridge_mid;

} SYS_PARAMETERS;

//! To set radio parameters
extern SYS_PARAMETERS sys_params;

//! Part of the network Id and is used for checking for un-initialized parameters
#define SIGNATURE_BYTE    0x90

//! To set up a Transmit data packet
typedef union _LS_TX_PACKET
{
    //! To hold the transmit data of packet
    struct
    {
        //! To hold the transmit header data of packet
        struct
        {
            //! Parity bit of data packet being transmitted
            unsigned parity : 0x01;
            //! Type of device part of data packet being transmitted
            unsigned dev_id : 0x01;   
            //! Data toggle information of data packet being transmitted
            unsigned dt_data: 0x01;
            //! Filler for 1 bit of the data packet being transmitted, could
            //! be 0 if NULL_PACKET_DATA is not defined   
            unsigned fill   : 0x01;  
            //! Type of data packet being transmitted
            unsigned type   : 0x04;   
        } hdr;
        //! Buffer to hold data packet being transmitted
	    UINT8 app_packet[APP_TX_PACKET_SIZE];
        //! To hold calculated checksum of data packet being transmitted
        UINT8 checksum;
    } data;


    //! To hold the transmit data of an acknowledge data packet
    struct
    {
        //! To hold the transmit header data of an acknowledge packet
        struct
        {
            //! Parity bit of acknowledge packet being transmitted
            unsigned parity : 0x01;
            //! Data toggle information of acknowledge packet being transmitted
            unsigned dt_ack : 0x01;   
            //! Type of device part of acknowledge packet being transmitted
            unsigned dev_id : 0x01;   
            //! Acknowledge packet flag to tell whether it was a good or bad ack
            //! being transmitted
            unsigned flag   : 0x01;   
            //! Type of acknowledge packet being transmitted
            unsigned type   : 0x04;   
        } hdr;
    } ack;

    //! To hold the transmit data of bind packet
    struct 
    {  
        //! To hold the transmit header data of bind packet
        struct
        {
            //! Parity bit of bind packet being transmitted
            unsigned parity   : 0x01;
            //! Type of device part of bind packet being transmitted
            unsigned dev_type : 0x03;   
            //! Type of bind packet being transmitted
            unsigned type     : 0x04;   
        } hdr;

    } bind;
    
    //! To hold the transmit data of connect packet
    struct
    {  
        //! To hold the transmit header data of connect packet
        struct
        {
            //! Parity bit of connect packet being transmitted
            unsigned parity : 0x01;
            //! For storing the pin used in the connect packet being transmitted
            unsigned pin    : 0x03;   
            //! Type of connect packet being transmitted
            unsigned type   : 0x04;   
        } hdr;

        //! 1st byte of radio manufacturing ID
        UINT8   mid_1;
        //! 2nd byte of radio manufacturing ID
        UINT8   mid_2;
        //! 3rd byte of radio manufacturing ID
        UINT8   mid_3;
        //! 4th byte of radio manufacturing ID
        UINT8   mid_4;
        //! Checksum of connect request packet being transmitted
        UINT8 checksum;
    } connect;

    //! To hold the transmit data of null packet
    struct
    {
        //! To hold the transmit header data of null packet
        struct
        {
            //! Parity bit of data packet being transmitted
            unsigned parity : 0x01;
            //! Type of device part of data packet being transmitted
            unsigned dev_id : 0x01;   
            // Filler for 2 bits of the null packet being transmitted, could be
            // 0 if NULL_PACKET_DATA is not defined   
            unsigned fill   : 0x02;  
            //! Type of data packet being transmitted
            unsigned type   : 0x04;   
        } hdr;
    } null;

    //! To hold the first byte of the transmit packet
    struct
    {
        //! First byte of transmit packet
        UINT8 byte;
    } first;
  
} LS_TX_PACKET;

//! To set up a receive data packet
typedef union _LS_RX_PACKET
{
    //! To hold the data of the receive packet
    struct
    {
        //! To hold the header data of the receive packet
        struct
        {
            //! Parity bit of data packet being received
            unsigned parity : 0x01;
            // Filler for 1 bit of the data packet being transmitted, could be
            // 0 if NULL_PACKET_DATA is not defined   
            unsigned fill   : 0x01;   
            //! Data toggle information of data packet being received
            unsigned dt_data: 0x01;
            //! Data packet flag to tell whether it was a good or bad data
            //! being received
            unsigned flag   : 0x01;   
            //! Type of data packet being received
            unsigned type   : 0x04;   
        } hdr;

        //! Buffer to hold data packet being received
    	UINT8 app_packet[APP_RX_PACKET_SIZE];
        //! To hold calculated checksum of data packet being received
        UINT8 checksum;
    } data;

    //! To hold the receive data of an acknowledge/data packet
    struct
    {
        //! To hold the receive header data of an acknowledge/data packet
        struct
        {
            //! Parity bit of acknowledge/data packet being received
            unsigned parity : 0x01;
            //! Data toggle information of acknowledge part of packet being received
            unsigned dt_ack : 0x01;   
            //! Data toggle information of data part of packet being received
            unsigned dt_data : 0x01;   
            //! Acknowledge packet flag to tell whether it was a good or bad 
            //! ack being received
            unsigned flag   : 0x01;   
            //! Type of acknowledge packet being received
            unsigned type   : 0x04;   
        } hdr;
        //! Buffer to hold data packet being received
    	UINT8 app_packet[APP_RX_PACKET_SIZE];
        //! To hold calculated checksum of data packet being received
        UINT8 checksum;
    } ack_data;

    //! To hold the receive data of an acknowledge data packet
    struct
    {
        //! To hold the receive header data of an acknowledge packet
        struct
        {
            //! Parity bit of acknowledge packet being received
            unsigned parity : 0x01;
            //! Data toggle information of acknowledge packet being received
            unsigned dt_ack : 0x01;   
            //! Type of device part of acknowledge packet being received
            unsigned dev_id : 0x01;   
            //! Acknowledge packet flag to tell whether it was a good or bad ack 
            //! being received
            unsigned flag   : 0x01;   
            //! Type of acknowledge packet being received
            unsigned type   : 0x04;   
        } hdr;
    } ack;

    //! To hold the receive data of an connect data packet
    struct
    {
        //! To hold the receive header data of an connect packet
        struct
        {
            //! Parity bit of connect packet being received
            unsigned parity : 0x01;
            //! Filler for 2 bits of the data packet being received, could be 0
            //! if NULL_PACKET_DATA is not defined   
            unsigned fill   : 0x02;   
            //! Connect packet flag to tell whether it was a good or bad ack
            //! being received
            unsigned flag   : 0x01;   
            //! Type of connect packet being received (i.e. CONNECT_REQ)
            unsigned type   : 0x04;   
        } hdr;
    } connect;

    //! To hold the receive data of an bind data packet
    struct
    {
        //! To hold the receive header data of an bind packet
        struct
        {
            //! Parity bit of bind packet being received
            unsigned parity   : 0x01;
            //! For storing the pin used in the bind packet being received
            unsigned pin      : 0x03;   
            //! Type of bind packet being received
            unsigned type     : 0x04;   
        } hdr;

        // Bridge MID
        //! 1st byte of radio manufacturing ID
        UINT8   mid_1;
        //! 2nd byte of radio manufacturing ID
        UINT8   mid_2;
        //! 3rd byte of radio manufacturing ID
        UINT8   mid_3;
        //! 4th byte of radio manufacturing ID
        UINT8   mid_4;
        //! Checksum of bind packet being received
        UINT8 checksum;
    } bind;

    //! To hold the first byte of the receive packet
    struct
    {
        //! First byte of receive packet
        UINT8 byte;
    } first;

} LS_RX_PACKET;

// Lengths are variable due to unions. +1 is for the Checksum, +2 is for
// type byte and checksum

//! Max size of RX arrays
#define LS_RX_PACKET_LEN   (sizeof(LS_RX_PACKET))     // Max size of RX arrays

//! Length of the bind request packet
#define BIND_REQ_LEN        1
//! Length of the bind response packet.  +2 is for type byte and checksum
#define BIND_RESP_LEN       4+2
//! Length of the connect request packet
#define CONNECT_REQ_LEN     4+2
//! Length of the connect response packet
#define CONNECT_RESP_LEN   0x01
//! Length of the null packet
#define NULL_LEN           0x01
//! Length of the ping packet
#define PING_LEN           0x01
//! Length of the ack packet
#define ACK_LEN            0x01

//--------------------------------------
// API Function Declarations
//--------------------------------------

UINT8  protocol_init(void);
void * protocol_get_tx_pkt(void);
void * protocol_get_rx_pkt(void);
UINT8  protocol_gen_parity(UINT8 data);
UINT8  protocol_calc_ck_sum(UINT8 *data, UINT8 len, UINT8 seed);
UINT8  protocol_send_packet(UINT8 data_length);
void   protocol_send_ack(void);
UINT8  protocol_verify_packet(void);

void set_bind_led(UINT8);

void   calculate_network_id(void);

//! Packet for receiving
extern LS_RX_PACKET  rx_packet;
//! Packet for transmitting
extern LS_TX_PACKET  tx_packet;
//! To set up the device ID when generating a transmit packet
extern UINT8 protocol_device_type;
 
#endif // _PROTOCOL_H_
