//--------------------------------------------------------------------------
//
//  Collection of functions to access the WirelessUSB LS Radio
//
//--------------------------------------------------------------------------
// $Archive: /WirelessUSB/LS/Firmware/LS DVK/PSoC/Radio/radio.c $
// $Modtime: 3/11/04 3:52p $
// $Revision: 1.1 $
//--------------------------------------------------------------------------
//
// Copyright 2003-2004, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international 
// treaty provisions. Cypress hereby grants to licensee a personal, 
// non-exclusive, non-transferable license to copy, use, modify, create 
// derivative works of, and compile the Cypress Source Code and derivative 
// works for the sole purpose of creating custom software in support of 
// licensee product to be used only in conjunction with a Cypress integrated 
// circuit as specified in the applicable agreement. Any reproduction, 
// modification, translation, compilation, or representation of this 
// software except as specified above is prohibited without the express 
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED, 
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------


#include "ls_config.h"
#include "spim_1.h"
#include "radio.h"

#define DEADMAN_MAX_COUNT       200

UINT8 rx_data_length = 0;

const UINT8 radio_init_data[] =
{
    REG_DATA_RATE, RADIO_MODE,
    REG_SERDES_CTL, (SERDES_ENABLE | (EOF_LEN_MASK & EOF_BITS)),
    REG_RX_INT_EN, (RX_FULL_A | RX_EOF_A),
    REG_TX_INT_EN, (TX_EMPTY),	
    REG_VALID_TX, 0xFF,
    REG_THRESHOLD_LO, CHIP_ERROR_THRESHOLD,
    REG_THRESHOLD_HI, (NUM_CHIPS - CHIP_ERROR_THRESHOLD),
    REG_ANALOG_CTL, (AGC_RSSI_CTL | PACTL_EN),
    REG_PA, (DEFAULT_POWER_LEVEL & PA_BIAS_MASK),
    REG_XTAL_ADJ, (CLOCK_DISABLE),              // disable 13MHz clock output
    REG_VCO_CAL, MINUS_5_PLUS_5,                 // use the laser trimmed VCO value
    REG_AGC_CTL, AGC_OFF,                       // turn off AGC circuitry to reduce power consumption
    REG_CLOCK_ENABLE, 0x41,                     // set per datasheet
    REG_CLOCK_MANUAL, 0x41                      // set per datasheet
};

//--------------------------------------------------------------------------
// timer_delay_10usec
//--------------------------------------------------------------------------

void timer_delay_10_usec(void)
{
                                   // [13] cycles for lcall
    asm("mov A, 4");               // [4]  cycles
    asm("jmp _delay_usec_loop");   // [5]  cycles
}


//--------------------------------------------------------------------------
// timer_delay_50us
//--------------------------------------------------------------------------

void timer_delay_50_usec(void)
{
                                   // [13] cycles for lcall
    asm("mov A, 28");              // [4]  cycles
    asm("jmp _delay_usec_loop");   // [5]  cycles
}


//--------------------------------------------------------------------------
// timer_delay_100usec
//--------------------------------------------------------------------------

void timer_delay_100_usec(void)
{
                                   // [13] cycles for lcall
    asm("mov A, 58");              // [4]  cycles
    asm("jmp _delay_usec_loop");   // [5]  cycles
}


//--------------------------------------------------------------------------
// delay_usec_loop
//--------------------------------------------------------------------------

static void delay_usec_loop(void)
{
    asm("and  A, FFh");            // [4] cycles
    asm("ljmp fwd");               // [7] cycles
    asm("fwd:");                   // [0] cycles
    asm("dec  A");                 // [4] cycles
    asm("jnz  _delay_usec_loop");  // [5] cycles
    asm("cmp  A, 0");              // [5] cycles
    asm("cmp  A, 0");              // [5] cycles
                                   // [8] cycles for return
}

//--------------------------------------------------------------------------
// timer_delay_msec
//--------------------------------------------------------------------------

void timer_delay_msec(UINT16 num_milliseconds)
{
    UINT8 i;
    
    for(i=0; i<num_milliseconds; i++)
    {
	    timer_delay_100_usec();
	    timer_delay_100_usec();
	    timer_delay_100_usec();
	    timer_delay_100_usec();
	    timer_delay_100_usec();
	    timer_delay_100_usec();
	    timer_delay_100_usec();
	    timer_delay_100_usec();
	    timer_delay_100_usec();
	    timer_delay_100_usec(); 
    }
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_init
//
//  Description: Reset and initialize the LS radio as per radio_init_data table
//
//  Inputs:      Void
//
//  Returns:     Void
//
///////////////////////////////////////////////////////////////////////////////

void radio_init()
{
    UINT8 i;
    
	RADIO_PORT |= N_SS;
	RADIO_PORT &= ~(N_PD | N_RESET);  
   	RADIO_PORT |= N_PD;	
	TIMER_DELAY_MSEC(RADIO_WAKEUP_TIME*2);    // crystal settle time

	SPIM_1_Start(SPIM_1_SPIM_MODE_0 | SPIM_1_SPIM_MSB_FIRST);
 	RADIO_PORT |= (N_RESET);	            // bring radio out of reset...

    // load the configuration data
    for(i = 0; i < sizeof(radio_init_data); i+=2)
    {
        spi_radio_put(radio_init_data[i] , radio_init_data[i+1]);
    }
 }

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_receive_on
//
//  Description: Set up the radio to receive mode
//
//  Inputs:      Void
//
//  Returns:     Void
//
///////////////////////////////////////////////////////////////////////////////

void radio_receive_on(void)
{   
    // clear old data and interrupts
    spi_radio_get(REG_DATA_RX_A);
    spi_radio_get(REG_RX_INT_STAT);
    spi_radio_put(REG_CONTROL, (RX_ENABLE | AUTO_SYNTH_COUNT));	
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_transmit_on
//
//  Description: Set up the radio to transmit mode
//
//  Inputs:      Void
//
//  Returns:     Void
//
///////////////////////////////////////////////////////////////////////////////

void radio_transmit_on(void)
{
    spi_radio_put(REG_CONTROL, (TX_ENABLE | AUTO_SYNTH_COUNT));	// enable transmit mode
}


void radio_off(void)
{                                                                     
    spi_radio_put(REG_CONTROL, 0);
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_set_channel
//
//  Description: Set the radio channel
//
//  Inputs:      channel
//
//  Returns:     Void
//
///////////////////////////////////////////////////////////////////////////////
    
void radio_set_channel(UINT8 channel)
{
    spi_radio_put(REG_CHANNEL, (CHANNEL_MASK & (channel + CHANNEL_OFFSET)));
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_set_pn_code
//
//  Description: Load the PN Code. For more information on PN codes please see
//               the LS Theory of Operation app note.
//
//  Inputs:      pn_code - pointer to the 8-byte PN code.
//
//  Returns:     Void
//
///////////////////////////////////////////////////////////////////////////////
void radio_set_pn_code(const UINT8 *pn_code)
{
    UINT8 i;

    for(i = 0; 7 >= i; i++)
    {
        spi_radio_put(REG_PN_CODE + i, *(pn_code + i));
    }
}


///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_transmit
//
//  Description: Send a packet to the radio, one byte at a time. 
//               If the radio is not already in transmit mode radio_transmit_on()
//               will be called before transmitting data and the radio will be
//               turned off after the packet transmission is complete.
//
//  Inputs:      len     - length of packet
//              *data    - pointer to the packet
//
//  Returns:     Void
//
///////////////////////////////////////////////////////////////////////////////
void radio_transmit(UINT8 buffer_length, UINT8 *tx_buffer)
{
    UINT8 i;
    UINT8 counter;
    UINT8 radio_state;

    radio_state = (spi_radio_get(REG_CONTROL));
    if ((radio_state & TX_ENABLE) != TX_ENABLE)
    {
        radio_transmit_on();
    }

    spi_radio_put(REG_TX_INT_EN, TX_EMPTY);
    // send the packet
    for(i = 0; i < buffer_length; ++i, ++tx_buffer) 
    {
        for (counter = 0; (counter < DEADMAN_MAX_COUNT) && (!(PRT1DR & R_IRQ_MASK)); 
             counter++); //dead man timeout

        spi_radio_put(REG_DATA_TX, *tx_buffer);
    }

    // enable the EOF interrupt and disable the EMPTY interrupt
    spi_radio_put(REG_TX_INT_EN, TX_EOF);

    timer_delay_100_usec();
    
    for (counter = 0; (counter < DEADMAN_MAX_COUNT) && (!(PRT1DR & R_IRQ_MASK)); 
          counter++); //dead man timeout

    if ((radio_state & TX_ENABLE) != TX_ENABLE)
    {
        radio_off();
    }
}

///////////////////////////////////////////////////////////////////////////////
//
//  Function:    radio_receive
//
//  Description: If the radio is not already in receive mode radio_receive_on()
//               is called.  Data is received from the radio until an end-of-
//               frame (EOF) interrupt is set or a timeout is reached. 
//
//  Inputs:      buffer_length - maximum number of bytes that can be received
//                               before a buffer overflow occurs.
//              *rx_buffer -     pointer to the buffer to store received data.
//              *valid_buffer -  pointer to buffer to store received valid bytes.
//               timeout -       length of time to be in receive mode before timing
//                               out. This value is in 10us units.  Once a byte
//                               is received the function will not return until
//                               the EOF interrupt is received, or a buffer overflow
//                               occurs, in order to prevent the function from
//                               returning in the middle of receiving a packet.
//
//  Returns:     rx_data_length - Length of packet received (if an EOF occurs), 
//                                otherwise returns 0.
//            
//
///////////////////////////////////////////////////////////////////////////////
UINT8 radio_receive(UINT8 buffer_length, UINT8 *rx_buffer, UINT8 *valid_buffer, UINT16 timeout)
{
    UINT8 irq_source;
    UINT8 radio_state;
    UINT16 loop_counter = 0;

    radio_state = (spi_radio_get(REG_CONTROL));
    if ((radio_state & RX_ENABLE) != RX_ENABLE)
    {
        radio_receive_on();
    }
    rx_data_length = 0;    

    while ((loop_counter < timeout) || (rx_data_length != 0))    // don't break out of the loop mid-packet..
    {
        // poll for a Radio Module interrupt, interrupts are self clearing
        // when the appropiate status register is read 
        if((PRT1DR & R_IRQ_MASK))
        {	
            // read the Radio Rx Interrupt register
            irq_source = spi_radio_get(REG_RX_INT_STAT);
    		
        	if(irq_source & RX_FULL_A)
        	{
                *rx_buffer = spi_radio_get(REG_DATA_RX_A);
    			
                // don't read the valid register if it is all valid
                if( irq_source & RX_VALID_A )
                {
                    *valid_buffer = 0xFF;
                }
                else
                {
                    *valid_buffer = spi_radio_get(REG_VALID_RX_A);
                }
            
                // filter out bad bytes and don't overwrite the globals
                if((*valid_buffer >= 0x0F) || (rx_data_length > 0))
                {
                    if (rx_data_length < buffer_length)
                    {
                        rx_data_length++;
                        loop_counter = 0;

                        rx_buffer++;
                        valid_buffer++;
                    }
                    else
                    {
                        // buffer overflow... exit receive mode
                        rx_data_length = 0;
                        break;
                    }
                }
        	}
        		
        	if(irq_source & RX_EOF_A)
        	{				
                if (rx_data_length != 0)
                {
                    break;
                }
        	}
            if(irq_source & RX_OVER_A)
            {
                loop_counter = 0;
            }            
        }	//if(LS_IRQ_ASSERTED)
        else
        {
            ++loop_counter;            // only increment loop_counter for each 10us delay...
            TIMER_DELAY_10_USEC();
        }
    } // while(loop_counter...)    
    
    if ((radio_state & RX_ENABLE) != RX_ENABLE)
    {
        radio_off();
    }

    return rx_data_length;
}


UINT8 spi_radio_access(UINT8 address, UINT8 data)
{
	// assert the slave select
	RADIO_PORT &= ~N_SS;
	
    // write the select address
	SPIM_1_TX_BUFFER_REG = address;
    while(!(SPIM_1_CONTROL_REG & SPIM_1_SPIM_SPI_COMPLETE));
	
    // read the data
	SPIM_1_TX_BUFFER_REG = data;
    while(!(SPIM_1_CONTROL_REG & SPIM_1_SPIM_SPI_COMPLETE));

	// deassert the slave select
	RADIO_PORT |= N_SS;

    data = SPIM_1_RX_BUFFER_REG;

	return data;
}

