;****************************************************************************
;
;   file: wusb-ls-main.asm 
;   Description: This code acts as a bridge for a WirelessUSB mouse and keyboard
;   Target: Cypress CY7C63743
;   Version:        1.3.0000
;
;   This code is based on the logo.asm fw 
;
;   Overview:  
;       This firmware executes A single simple program loop.
;       On reset, the firmware initializes the radio and then
;       waits until enumerated by the USB host.
;       It then continuously polls the radio "data ready" 
;       input, and when active it reads and processes
;       radio received data. If valid, the received data
;       is loaded into the EP1 FIFO, and EP1 is armed
;       for transmission.
;                  
;   USB:
;       At bus reset the USB interface is re-initialized,
;       AND  the firmware soft reboots.  We are then able to
;       handle the standard chapter nine requests on 
;       endpoint zero (the control endpoint).  After this
;       device enumerates as A HID mouse on the USB, the
;       requests come to endpoint one (the data endpoint).
;       Endpoint one is used to send mouse displacement AND 
;       button status information.  
;
;   Pin Connections:
;
;                -------------------
;               | P0[0]     P0[4]   |
; RX Data Rdy   | P0[1]     P0[5]   |
;               | P0[2]     P0[6]   |
;               | P0[3]     P0[7]   |
;               | P1[0]     P1[1]   |   
;               | P1[2]     P1[3]   |
;               | P1[4]     P1[5]   |
;               | P1[6]     P1[7]   |
;       GND     | VSS       D+/SCLK |   USB D+ / PS2 SCLK
;       GND     | VPP       D-/SDATA|   USB D- / PS2 SDATA
;   PULLUP      | VREG      VCC     |   +5V
;               | XTALIN    XTALOUT |
;                -------------------
;
; Revisions:
;
;****************************************************************************
; Copyright (2003), Cypress Semiconductor Corporation
; This software is owned by Cypress Semiconductor Corporation (Cypress) and  is
; protected by United States copyright laws and international treaty provisions.
; Cypress hereby grants to Licensee a personal, non-exclusive, non-transferable
; license to copy, use, modify, create derivative works of, and compile the
; Cypress Source Code and derivative works for the sole purpose of creating
; custom software in support of Licensee product ("Licensee Product") to be used
; only in conjunction with a Cypress integrated circuit. Any reproduction,
; modification, translation, compilation, or representation of this software
; except as specified above is prohibited without the express written permission
; of Cypress.
; Disclaimer: Cypress makes no warranty of any kind, express or implied, with
; regard to this material, including, but not limited to, the implied warranties
; of merchantability and fitness for a particular purpose. Cypress reserves the
; right to make changes without further notice to the materials described herein.
; Cypress does not assume any liability arising out of the application or use of
; any product or circuit described herein. Cypress products described herein are
; not authorized for use as components in life-support devices.
; This software is protected by and subject to worldwide patent coverage,
; including U.S. and foreign patents. Use may be limited by and subject to the
; Cypress Software License Agreement.
;
;****************************************************************************

;---------------- assembler directives -------------------- 

INCLUDE "config.inc"            ; includes protocol configuration

IFDEF PDC_9075
    CPU 63743
ENDIF

IFDEF PDC_9168
    CPU 63723
ENDIF

XPAGEON

INCLUDE "637xx.inc"
INCLUDE "ls_headers.inc"


;--------------- interrupt vector table -------------------

ORG 00h         

JMP  reset               ; reset vector      
JMP  error               ; bus reset interrupt
JMP  error               ; 128us interrupt
JMP  ms_timer            ; 1.024ms interrupt
JMP  error               ; endpoint 0 interrupt
JMP  error			     ; endpoint 1 interrupt
JMP  error               ; endpoint 2 interrupt
JMP  error               ; reserved
JMP  error               ; Capture timer A interrupt Vector
JMP  error               ; Capture timer B interrupt Vector
JMP  error               ; GPIO interrupt
JMP  wakeup_isr          ; Wake-up interrupt vector 


;-------------- program listing ---------------------------

ORG  1Ah
error: JMP reset

;-------------------------------------------------------
;   MACROS
;-------------------------------------------------------
MACRO set_port_0 mask
    MOV  A, mask
    OR   [port0Shadow], A
    MOV  A, [port0Shadow]
    IOWR port0
ENDM

MACRO clr_port_0 mask
    MOV  A, ~mask
    AND  [port0Shadow], A
    MOV  A, [port0Shadow]
    IOWR port0
ENDM

MACRO set_port_1 mask
    MOV  A, mask
    OR   [port1Shadow], A
    MOV  A, [port1Shadow]
    IOWR port1
ENDM

MACRO clr_port_1 mask
    MOV  A, ~mask
    AND  [port1Shadow], A
    MOV  A, [port1Shadow]
    IOWR port1
ENDM

;----------------------------------------------------------
;
;   Interrupt handler: wakeup_isr
;
;----------------------------------------------------------
wakeup_isr:
RETI

;----------------------------------------------------------
;
;   Interrupt: ms_timer
;   Purpose: Every 1ms this interrupt handler clears
;       the watchdog timer.
;
;----------------------------------------------------------

ms_timer:
    PUSH A
    PUSH X

    IOWR watchdog
    
    INC  [app_timer]
    INC  [timer]                ; INC  1ms counter

	INC  [led_timer]
	JNC  ms_timer_done
	MOV  A, F0h					; prevent led_timer from overflowing
	MOV  [led_timer], A

ms_timer_done:
    POP  X
    POP  A
    RETI

;----------------------------------------------------------
;
;   Interrupt handler: reset
;   Purpose: The program jumps to this routine when
;        the microcontroller has A power on reset.
;
;----------------------------------------------------------

reset:
    MOV  A, (LVR_ENABLE | INTERNAL_CLK)  ; set for use with internal oscillator  
    IOWR clock_config
    
    MOV  A, DSP_TOP                  ; set data stack pointer
    swap A, dsp                  
    MOV  A, PSP_BASE                 ; set PC stack pointer
    MOV  psp,A   
    
    IOWR watchdog
        
    CALL radio_interface_init       ; configure the GPIOs

    ; clear variables
    MOV  A, 00h                      

    MOV  [c0h], A
    MOV  [c1h], A
    MOV  [c2h], A
    MOV  [c3h], A
    MOV  [c4h], A
    MOV  [c5h], A
    MOV  [c6h], A
    MOV  [c7h], A
    MOV  [c8h], A
    MOV  [c9h], A
    MOV  [cah], A
    MOV  [cbh], A
    MOV  [cch], A
    MOV  [cdh], A
    MOV  [ceh], A
    MOV  [cfh], A

    MOV  [button_pushed], A				   
    			   
    ;initialize application variables
    MOV  [rx_counter], A
    MOV  [sequence_id_a], A
    MOV  [app_timer], A
    MOV  [sequence_id_b], A
    MOV  [timer], A
    MOV  [corrupt_counter], A
    MOV  [tx_ack_pending], A
    MOV  [tx_app_counter], A
    MOV  [bad_packets_a_low], A  
    MOV  [bad_packets_a_high], A 
    MOV  [good_packets_a_low], A    
    MOV  [good_packets_a_high], A
    MOV  [bad_packets_b_low], A  
    MOV  [bad_packets_b_high], A 
    MOV  [good_packets_b_low], A    
    MOV  [good_packets_b_high], A
    MOV  [led_state], A
        
    MOV  A, WAKEUP_ADJUST2|WAKEUP_ADJUST1|WAKEUP_ADJUST0|PRECISION_CLK_ENABLE|INTERNAL_CLK ;128*tWAKE
    IOWR clock_config                 ; set wakeup timer interval AND  disable XTALOUT

    ; enable global interrupts
    MOV  A, (1MS_INT)
    IOWR global_int

	MOV  A, F0h
    MOV  [led_timer], A
    
    MOV  A, 50h                     ; THIS IS DEBUG CODE FOR TWO_WAY_DATA
    MOV  [tx_app_buffer], A

    EI                              ; enable all interrupts
        
    CALL app_init_a

    JMP  power_on_mode

;**********************************************************
;
;       connected_mode:
;
;**********************************************************
connected_mode_init:
connected_mode:

    CALL switch_rx

    MOV  A, 00h                     ; reset RX variables
    MOV  [packet_type], A
    MOV  [rx_counter], A
    MOV  [corrupt_counter], A       ; reset corrupt and RSSI counters
    MOV  [rssi_counter], A

connected_loop:
    IOWR watchdog

    IORD port0
    AND  A, IRQ
    JZ   connected_bind_check       ; if no IRQ check bind button, RSSI, etc.
    CALL process_rx_int             ; process RX interrupt

check_packet:
    CMP  A, bFLAG_EOF               ; check for EOF
    JNZ  connected_loop
    
    CALL verify_packet              ; verify checksum, valid bits, etc.
    CMP  A, bFLAG_VALID
    JNZ  process_corrupt_packet     ; handle corrupt packets

	CALL switch_tx                  ; switch to transmit mode so that the radio is ready to transmit ACK, etc.

dec_corrupt_counter:    ; decrement the corrupt counter for each valid packet received
    DEC  [corrupt_counter]
    JNC  dec_corrupt_counter_complete
    MOV  A, 0
    MOV  [corrupt_counter], A
dec_corrupt_counter_complete:

    MOV  A, [packet_type]           ; Check for DATA packet
    CMP  A, PT_DATA
    JZ   process_data

    CMP  A, PT_CONN_REQ             ; check for CONNECT REQUEST packet
    JZ   process_conn_req

    CMP  A, PT_PING                 ; check for PING packet
    JZ   send_ping_rsp

    CMP  A, PT_NULL                 ; check for NULL packet
    JZ   conn_process_null

    CMP  A, PT_ACK
    JZ   conn_process_ack

process_packet_complete:            ; All of the process functions jump to here when finished
    CALL switch_rx                  ; switch back to RX mode
    MOV  A, 00h                     ; reset RX variables
    MOV  [packet_type], A
    MOV  [rx_counter], A
    JMP  connected_loop_complete
  
connected_bind_check:                
    MOV  A, [rx_counter]
    CMP  A, 0
    JZ   check_rssi_state           ; do not check RSSI, bind button, etc. if we are in the middle of a packet

    MOV  A, [timer]                 
    SUB  A, BIND_POLL_TIMEOUT+3        
    JC   connected_loop             
    JMP  check_bind

check_rssi_state:
    MOV  A, [rssi_state]            ; check to see if RSSI has already been checked this RX period
    CMP  A, bRSSI_NEW
    JNZ  check_bind

    MOV  A, [timer]
    CMP  A, QUICK_RSSI_TIMEOUT      ; check to see if RX mode has been on long enough to trigger RSSI
    JC   check_bind                 ; and to give 1-2 ms gap between RSSI readings

connected_rssi: 
    ;MOV  A, CONNECTED_MODE_RSSI
    CALL check_rssi                 ; check RSSI
    CMP  A, RSSI_LOUD_THRESHOLD
    JNC  connected_rssi_high        ; check for high RSSI
    MOV  A, bRSSI_READ
    MOV  [rssi_state], A            ; signal that RSSI has been read    
    JMP  connected_loop    

connected_rssi_high:
    SWAP A, X
    CMP  A, NUM_HIGH_RSSI_READINGS  ; check for consecutive high RSSI readings
    JC   connected_rssi_reset
    MOV  A, PING_REASON_RSSI        ; jumping to ping due to RSSI
    JMP  ping_mode                  ; Change channels if this channel is too noisy

check_bind:
    MOV  A, [timer]                 ; check to see if it is time to check bind  
    SUB  A, BIND_POLL_TIMEOUT        
    JC   connected_loop             ; if not loop back to connected_loop to check for RX interrupt
    
    MOV  A, [rx_counter]
    CMP  A, 0
    JNZ  connected_loop             ; do not check RSSI or bind if we are in the middle of receiving a packet
    
    CALL check_for_bind             ; if bind button is pushed, check_for_bind returns a non-zero A
    AND  A, ffh
    JZ   connected_rssi_reset
    ; clear any pressed buttons
    CALL app_disconnect_a           ; allows app to send key-up, etc. before entering bind mode
    JMP  bind_mode
       
connected_rssi_reset:
    CALL setup_rx                   ; reset RX mode

connected_loop_complete:   
    MOV  A, REG_TX_VALID            ; check to make sure the radio is still running properly
    MOV  [reg_addr], A              ; this is a fail-safe in case the radio gets reset
    CALL SPI_read                   ; due to brownout, ESD, etc.
    MOV  A, [reg_data]
    CMP  A, FFh
    JZ   connected_loop_complete2
    CALL radio_init

connected_loop_complete2:
    CALL setup_rx    

    MOV  A, 00h
    MOV  [timer], A                 ; reset timer
    CALL connected_leds

    MOV  A, [app_timer]
    CMP  A, APP_IDLE_TIMEOUT
    JC   connected_loop
    
    CALL app_idle_a                 ; calls app_idle_a to handle key timeouts, etc.
    MOV  A, 0
    MOV  [app_timer], A
    JMP  connected_loop
       

;-------------------------------------------------------
;   process_corrupt_packet
;-------------------------------------------------------
process_corrupt_packet:
    MOV  A, [corrupt_counter]       ; check the corrupt threshold (set in verify_packet)
    CMP  A, CORRUPT_THRESHOLD
    JC   process_packet_complete

process_corrupt_disconnect:
    CALL app_disconnect_a           ; if corrupt threshold reached disconnect the app
    MOV  A, PING_REASON_CORRUPT
    JMP  ping_mode                  ; and jump to ping mode to find a quieter channel
    
process_corrupt_okay:    
    INC  [bad_packets_a_low]        ; adjust packet counts for link quality app
    JNC  corrupt_link_qual_b
    INC  [bad_packets_b_high]
    
corrupt_link_qual_b:
    INC  [bad_packets_a_low]
    JNC  process_packet_complete
    INC  [bad_packets_b_high]

    JMP  process_packet_complete

;-------------------------------------------------------
;   process_data
;
;   Checks for retransmitted packet AND  unchanged data
;
;   NOTE: process_data falls through to send_ack
;-------------------------------------------------------
process_data:
    MOV  A, [rx_counter]
    CMP  A, 2                       ; check packet to see if it is too short
    JC   process_packet_complete

process_data_a:
    MOV  A, [sequence_id_a]
	AND  A, FFh
	JZ   store_seq_a				  

    CMP  A, [rx_buffer]             ; compare last received sequence_id with sequence_id byte
    JNZ  store_seq_a
    MOV  X, [rx_counter]
    MOV  A, [X+rx_buffer-1]
    CMP  A, [checksum_a]            ; compare last received checksum with received checksum byte
    JNZ  store_seq_a

    CALL send_ack_a                 ; if packet has same sequence_id and checksum discard it, but send an ACK

    INC  [bad_packets_a_low]        ; increment bad packet count for retransmitted packets
    JNC  check_seq_a_link_qual_b
    INC  [bad_packets_a_high]
check_seq_a_link_qual_b:
    INC  [bad_packets_b_low]
    JNC  process_packet_complete
    INC  [bad_packets_b_high]

    JMP  process_packet_complete

store_seq_a:
    MOV  X, [rx_counter]
    DEC  X                           ; substract header and checksum
    DEC  X
    MOV  A, 00h                      ; to be sent.
    CALL app_verify_packet_a		 ; IF A == 0 then packet is bad, else packet is good
	AND  A, ffh
	JZ   process_packet_complete

    MOV  A, [rx_buffer]              ; store the received sequence_id
    MOV  [sequence_id_a], A

    MOV  X, [rx_counter]
    MOV  A, [X+rx_buffer-1]
    MOV  [checksum_a], A            ; store the checksum

    CALL send_ack_a
    MOV  X, [rx_counter]
    DEC  X                          ; substract header and checksum from packet length
    DEC  X
    MOV  A, 00h                      
    CALL app_data_received_a        ; app-specific routine to format the data and hand it off to app

good_link_qual_a:
    INC  [good_packets_a_low]       ; increment good packet counter (A and B are incremented together)
    JNC  good_link_qual_a_b
    INC  [good_packets_a_high]
    JNC  good_link_qual_a_b         ; reset corrupt packet count if total packet count rolled over
    MOV  A, 0
    MOV  [bad_packets_a_low], A
    MOV  [bad_packets_a_high], A

good_link_qual_a_b:
    INC  [good_packets_b_low]       ; increment good packet counter (A and B are incremented together)
    JNC  process_packet_complete
    INC  [good_packets_b_high]
    JNC  process_packet_complete    ; reset corrupt packet count if total packet count rolled over
    MOV  A, 0
    MOV  [bad_packets_b_low], A
    MOV  [bad_packets_b_high], A
    JMP  process_packet_complete

conn_process_null:
	MOV  A, [rx_counter]	   
    CMP  A, LEN_NULL                ; either 1 or 2 bytes long (2 bytes if CHECKSUM_ALL_PACKETS defined)
    JNZ  process_packet_complete    ; discard packet if length is not correct

    MOV  A, [rx_buffer]
    AND  A, bDEVICE_ID_NULL
    JNZ  conn_null_b                ; check to see if NULL is for A or B

conn_null_a:
    JMP  process_packet_complete

conn_null_b: 
    JMP  process_packet_complete

;-------------------------------------------------------
;       send_ack
;-------------------------------------------------------
send_ack_a:
             
    MOV  A, [batt_report_req_a]	    ; batt_report_req_a can be 0 or 1.   
	MOV  [tx_app_counter], A		; When tx_app_counter = 1 we send ACK_DATA instead of ACK

	; Put the KBD signature in the data byte of ACK_DATA
	; If batt_report_req_a == 0, signature will be overwritten by ACK
	SWAP A,X
	MOV  A, 74h						
	MOV  [X + tx_app_header], A										
	
	MOV  A, 0h						; Reset batt_report_req_a to 0 
    MOV  [batt_report_req_a], A		 

    MOV  A, [sequence_id_a]
    ASR 
    AND  A, 02h                     ; clear everything but the data toggle bit
    JMP  send_ack

send_ack_b:
    MOV  A, [batt_report_req_b]	    ; batt_report_req_b can be 0 or 1. 
	MOV  [tx_app_counter], A		; When tx_app_counter = 1 we send ACK_DATA instead of ACK

	; Put the Mouse signature in the data byte of ACK_DATA
    ; If batt_report_req_b == 0, signature will be overwritten by ACK
	SWAP A,X
	MOV  A, 1Eh															  
	MOV  [X + tx_app_header], A										

    MOV  A, 0h						; Reset batt_report_req_b to 0 
    MOV  [batt_report_req_b], A

    MOV  A, [sequence_id_b]
    ASR 
    AND  A, 02h                     ; clear everything but the data toggle bit
    OR   A, bDEVICE_ID_ACK

send_ack:
    OR   A, PT_ACK                  ; set header nibble as ACK 
    OR   A, bHEADER_FLAG
    MOV  [tx_sys_buffer], A
    CALL set_sys_parity             ; set the parity bit
    
    MOV  A, LEN_ACK                 ; ACK Packet length
    MOV  [tx_sys_counter], A    

    CALL transmit_sys

    RET         


;-------------------------------------------------------
;   conn_process_ack
;-------------------------------------------------------
conn_process_ack:
    MOV  A, [rx_counter]
    CMP  A, LEN_ACK                 ; check the ACK length
    JNZ  process_packet_complete
    MOV  A, [tx_ack_pending]
    AND  A, bWAIT_FOR_ACK_A         ; if we weren't expecting an ACK consider this a corrupt packet
    JNZ  wait_for_ack_received

	INC  [corrupt_counter]	     	; if we receive an unexpected valid ACK someone else is on our channel/PN code
    MOV  A, [corrupt_counter]
    CMP  A, CORRUPT_THRESHOLD
    JC   wait_for_ack_received	   	; check the corrupt threshold
    CALL app_disconnect_a
    MOV  A, PING_REASON_CORRUPT
    JMP  ping_mode  
    
wait_for_ack_received:
    MOV  A, 0                       ; clear ACK pending flag
    MOV  [tx_ack_pending], A
    JMP  process_packet_complete

conn_process_ack_complete:
    JMP  process_packet_complete

    
;-------------- end program listing -----------------------------------------

    INCLUDE "bind_auto.asm"
    INCLUDE "utilities.asm"
    INCLUDE "radio.asm"    
    INCLUDE "dvk_hardware.asm"

    INCLUDE "tutorial_app.asm"