;****************************************************************************
;
;   file:   utilities.asm
;   Description: This file contains miscellaneous protocol procedures
;                   process_rx_int
;                   verify_packet
;                   transmit_sys
;                   transmit_app
;                   set_sys_parity
;                   set_app_parity
;                   set_sys_checksum
;                   set_app_checksum
;                   clear_rx_packet
;                   delay200us
;                   
;
;   Target:  Cypress CY7C63723 Encore Chip
;   $Header: g:/archives/Projects/LS DVK P2/Dongle/Two Way/utilities.asm
;   Version:        1.3.0000
;
;****************************************************************************
; Copyright (2003), Cypress Semiconductor Corporation
; This software is owned by Cypress Semiconductor Corporation (Cypress) and  is
; protected by United States copyright laws and international treaty provisions.
; Cypress hereby grants to Licensee a personal, non-exclusive, non-transferable
; license to copy, use, modify, create derivative works of, and compile the
; Cypress Source Code and derivative works for the sole purpose of creating
; custom software in support of Licensee product ("Licensee Product") to be used
; only in conjunction with a Cypress integrated circuit. Any reproduction,
; modification, translation, compilation, or representation of this software
; except as specified above is prohibited without the express written permission
; of Cypress.
; Disclaimer: Cypress makes no warranty of any kind, express or implied, with
; regard to this material, including, but not limited to, the implied warranties
; of merchantability and fitness for a particular purpose. Cypress reserves the
; right to make changes without further notice to the materials described herein.
; Cypress does not assume any liability arising out of the application or use of
; any product or circuit described herein. Cypress products described herein are
; not authorized for use as components in life-support devices.
; This software is protected by and subject to worldwide patent coverage,
; including U.S. and foreign patents. Use may be limited by and subject to the
; Cypress Software License Agreement.
;
;****************************************************************************

;-------------------------------------------------------
;
;   process_rx_int
;
;   Sets the EOF flags if end of frame interrupt is on
;   AND  reads data 
;-------------------------------------------------------
process_rx_int:
    MOV  A, REG_RX_INT_STAT
    MOV  [reg_addr], A  
    CALL SPI_read               
    MOV  A, [reg_data]
    MOV  [radio_int], A

    AND  A, bRX_FULL_A               ; check for data
    JZ   check_eof
    CALL data_read

check_eof:    
    MOV  A, [radio_int]
    AND  A, bRX_EOF_A
    JZ   process_rx_int_complete      ; check for EOF
    MOV  A, [rx_counter]
    AND  A, ffh
    JZ   process_rx_int_complete      ; don't set EOF flag for 0 byte packets    
    MOV  A, bFLAG_EOF
    RET

process_rx_int_complete:
    MOV  A, 0
    RET 

;-------------------------------------------------------
;   data_read
;-------------------------------------------------------
data_read:
    MOV  X, [rx_counter]         
    MOV  A, [radio_int]
    AND  A, bRX_VALID_A
    JZ   data_read_invalid
    MOV  A, ffh
    MOV  [X+valid_buffer], A
    JMP  data_read_valid

data_read_invalid:
    MOV  A, REG_RX_VALID_A           ; read the rx data valid byte
    MOV  [reg_addr], A               ;
    CALL SPI_read                   ;

    MOV  A, [reg_data]               ; IF only the bottom 2-3 bits are
    CMP  A, 08h                      ; valid, this is probably noise
    JC   reset_byte_counter           ; so throw it away
    MOV  [X+valid_buffer], A         ; store the valid byte (used in error correction)

data_read_valid:
    MOV  A, REG_RX_DATA_A            ; read the rx data byte
    MOV  [reg_addr], A               ;
    CALL SPI_read                   ;
    
    MOV  A, [reg_data]               ; store the data
    MOV  [X+rx_buffer], A            ;
    INC  [rx_counter]                ; inc the counter for next time
    MOV  A, [rx_counter]             ; 
    CMP  A, RX_BUFFER_LEN+1
    JNC  reset_byte_counter          ; buffer overflow check
    
data_read_complete:    
    MOV  A, 00h                      ; reset the radio timer
    MOV  [timer], A                  ; to prevent reset in mid-packet from RSSI, etc.
    RET      
       
reset_byte_counter: 
    MOV  A, REG_RX_DATA_A            ; do a dummy data read to clear the 
    MOV  [reg_addr], A               ; interrupt flag in the radio.
    CALL SPI_read                   ;
        
    MOV  A, 00h                      ; reset the rx byte counter
    MOV  [rx_counter], A             ; AND  return to polling loop
    
    RET 


;-------------------------------------------------------
;
;   transmit_sys
;
;-------------------------------------------------------
transmit_sys:
        MOV  A, 0h
    MOV  [ls_temp],A

    MOV  X, 00h                  

    MOV  A, REG_TX_DATA             
    MOV  [reg_addr], A              

tx_sys_stall:
    IOWR    watchdog

    INC  [ls_temp]                       ; If we incremented past 255, bail-out
    JC   tx_sys_finished

    IORD port0
    AND  A, IRQ
    JZ   tx_sys_stall
    
tx_sys_load:
    MOV  A, [X + tx_sys_buffer]       
    MOV  [reg_data], A
    CALL SPI_write

    MOV  A, 0h
    MOV  [ls_temp],A

    INC  X                           
    MOV  A, X
    SUB  A, [tx_sys_counter]
    JC   tx_sys_stall                 ; wait for a buffer empty interrupt

    MOV  A, REG_TX_INT_EN
    MOV  [reg_addr], A
    MOV  A, bTX_EOF
    MOV  [reg_data], A
    CALL SPI_write

    MOV  A, 0h
    MOV  [ls_temp],A

tx_sys_wait_finish:
    IOWR watchdog

    INC  [ls_temp]                       ; If we incremented past 255, bail-out
    JC   tx_sys_finished

    IORD port0
    AND  A, IRQ
    JZ   tx_sys_wait_finish

    MOV  A, REG_TX_INT_STAT          ; If there is an interrupt we need to read
    MOV  [reg_addr], A               ; the interrupt status register 
    CALL SPI_read                    ; in order to clear the interrupt

tx_sys_finished:
        MOV  A, REG_TX_INT_EN
        MOV  [reg_addr], A
        MOV  A, bTX_EMPTY
        MOV  [reg_data], A
        CALL SPI_write

    CALL radio_idle    
    MOV  A, 0
    MOV  [timer], A                  ; reset the 1 ms timer

    RET  
 


;-------------------------------------------------------
;
;   delay10us
;
;   A 10 us delay at 12 MHz requires 120 cycles...
;
;-------------------------------------------------------
delay10us:
    MOV  A, 8                        ; 8 cycles
    

delay10us_loop:                     ; 9 cycles per loop
    DEC  A                           ; 4 cycles
    JNZ  delay10us_loop              ; 5 cycles (4 if not taken)

    RET                              ; 8 cycles


delay50us:
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    RET 


;-------------------------------------------------------
;
;   delay200us
;
;-------------------------------------------------------
delay200us:
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    CALL delay10us
    RET                           

