;****************************************************************************
;
;   file: wusb-ls-main.asm 
;   Description: This code acts as a bridge for a WirelessUSB mouse and keyboard
;   Target: Cypress CY7C63743
;   Version:        1.3.0000
;
;   This code is based on the logo.asm fw 
;
;   Overview:  
;       This firmware executes A single simple program loop.
;       On reset, the firmware initializes the radio.
;       It then continuously polls the radio "data ready" 
;       input, and when active it reads and processes
;       radio received data. If valid, the received data
;       is loaded into the EP1 FIFO, and EP1 is armed
;       for transmission.
;
;   Pin Connections:
;
;                -------------------
;               | P0[0]     P0[4]   |
; RX Data Rdy   | P0[1]     P0[5]   |
;               | P0[2]     P0[6]   |
;               | P0[3]     P0[7]   |
;               | P1[0]     P1[1]   |   
;               | P1[2]     P1[3]   |
;               | P1[4]     P1[5]   |
;               | P1[6]     P1[7]   |
;       GND     | VSS       D+/SCLK |   USB D+ / PS2 SCLK
;       GND     | VPP       D-/SDATA|   USB D- / PS2 SDATA
;   PULLUP      | VREG      VCC     |   +5V
;               | XTALIN    XTALOUT |
;                -------------------
;
; Revisions:
;
;****************************************************************************
; Copyright (2003), Cypress Semiconductor Corporation
; This software is owned by Cypress Semiconductor Corporation (Cypress) and  is
; protected by United States copyright laws and international treaty provisions.
; Cypress hereby grants to Licensee a personal, non-exclusive, non-transferable
; license to copy, use, modify, create derivative works of, and compile the
; Cypress Source Code and derivative works for the sole purpose of creating
; custom software in support of Licensee product ("Licensee Product") to be used
; only in conjunction with a Cypress integrated circuit. Any reproduction,
; modification, translation, compilation, or representation of this software
; except as specified above is prohibited without the express written permission
; of Cypress.
; Disclaimer: Cypress makes no warranty of any kind, express or implied, with
; regard to this material, including, but not limited to, the implied warranties
; of merchantability and fitness for a particular purpose. Cypress reserves the
; right to make changes without further notice to the materials described herein.
; Cypress does not assume any liability arising out of the application or use of
; any product or circuit described herein. Cypress products described herein are
; not authorized for use as components in life-support devices.
; This software is protected by and subject to worldwide patent coverage,
; including U.S. and foreign patents. Use may be limited by and subject to the
; Cypress Software License Agreement.
;
;****************************************************************************

;---------------- assembler directives -------------------- 

INCLUDE "config.inc"            ; includes protocol configuration

IFDEF PDC_9075
    CPU 63743
ENDIF

IFDEF PDC_9168
    CPU 63723
ENDIF

XPAGEON

INCLUDE "637xx.inc"
INCLUDE "ls_headers.inc"


;--------------- interrupt vector table -------------------

ORG 00h         

JMP  reset               ; reset vector      
JMP  error               ; bus reset interrupt
JMP  error               ; 128us interrupt
JMP  ms_timer            ; 1.024ms interrupt
JMP  error               ; endpoint 0 interrupt
JMP  error			     ; endpoint 1 interrupt
JMP  error               ; endpoint 2 interrupt
JMP  error               ; reserved
JMP  error               ; Capture timer A interrupt Vector
JMP  error               ; Capture timer B interrupt Vector
JMP  error               ; GPIO interrupt
JMP  wakeup_isr          ; Wake-up interrupt vector 


;-------------- program listing ---------------------------

ORG  1Ah
error: JMP reset

;-------------------------------------------------------
;   MACROS
;-------------------------------------------------------
MACRO set_port_0 mask
    MOV  A, mask
    OR   [port0Shadow], A
    MOV  A, [port0Shadow]
    IOWR port0
ENDM

MACRO clr_port_0 mask
    MOV  A, ~mask
    AND  [port0Shadow], A
    MOV  A, [port0Shadow]
    IOWR port0
ENDM

MACRO set_port_1 mask
    MOV  A, mask
    OR   [port1Shadow], A
    MOV  A, [port1Shadow]
    IOWR port1
ENDM

MACRO clr_port_1 mask
    MOV  A, ~mask
    AND  [port1Shadow], A
    MOV  A, [port1Shadow]
    IOWR port1
ENDM

    
;----------------------------------------------------------
;
;   Interrupt handler: wakeup_isr
;
;----------------------------------------------------------
wakeup_isr:
RETI

;----------------------------------------------------------
;
;   Interrupt: ms_timer
;   Purpose: Every 1ms this interrupt handler clears
;       the watchdog timer.
;
;----------------------------------------------------------

ms_timer:
    PUSH A

    IOWR watchdog

    INC  [timer]                ; INC  1ms counter
	INC  [led_timer]
	JNC  ms_timer_done
	MOV  A, F0h					; prevent led_timer from overflowing
	MOV  [led_timer], A

ms_timer_done:
    POP A
    RETI

;----------------------------------------------------------
;
;   Interrupt handler: reset
;   Purpose: The program jumps to this routine when
;        the microcontroller has A power on reset.
;
;----------------------------------------------------------

reset:
    MOV  A, (LVR_ENABLE | INTERNAL_CLK)  ; set for use with internal oscillator  
    IOWR clock_config
    
    MOV  A, DSP_TOP                  ; set data stack pointer
    swap A, dsp                  
    MOV  A, PSP_BASE                 ; set PC stack pointer
    MOV  psp,A   
    
    IOWR watchdog
        
    CALL radio_interface_init       ; configure the GPIOs

    ; clear variables
    MOV  A, 00h                      

    MOV  [button_pushed], A				   
    			   
    ;initialize application variables
    MOV  [rx_counter], A
    MOV  [timer], A
    MOV  [led_state], A

    MOV  A, WAKEUP_ADJUST2|WAKEUP_ADJUST1|WAKEUP_ADJUST0|PRECISION_CLK_ENABLE|INTERNAL_CLK ;128*tWAKE
    IOWR clock_config                 ; set wakeup timer interval AND  disable XTALOUT

    ; enable global interrupts
    MOV  A, (1MS_INT)
    IOWR global_int

    EI                              ; enable all interrupts

        
;**********************************************************
;
;       connected_mode:
;
;**********************************************************
connected_mode_init:
    CALL radio_init             ; set up the radio
	
	MOV A, 00h					; set pn code to 0
	CALL radio_load_pn_code

	MOV A, 07h					; set the channel to 7
    MOV [channel], A            

    CALL setup_rx               ; reset the radio

connected_mode:

connected_loop:
    IOWR watchdog
    CALL check_for_bind         ; if bind is pushed, returns a non-zero A
    AND  A, ffh
    JZ   connected_loop

transmit_byte:
    CALL setup_tx               ; set the radio into TX mode
	MOV A, 55h					; put the data in the transmit buffer
	MOV [tx_sys_buffer], A
	MOV A, 01h					; transmitting a single byte
	MOV [tx_sys_counter], A
	CALL transmit_sys
    CALL setup_rx               ; reset the radio
	JMP connected_loop
	

;-------------- end program listing -----------------------------------------


    INCLUDE "utilities.asm"
    INCLUDE "radio.asm"    
    INCLUDE "dvk_hardware.asm"


